@@ -374,7 +374,7 @@ FileIO_Filename(FileIODescriptor *fd)  // IN:
 
 
 #if defined(_WIN32) || defined(__linux__) || defined(__APPLE__) || \
-    defined(__FreeBSD__)
+    defined(__FreeBSD__) || defined(sun)
 /*
  *----------------------------------------------------------------------
  *
