@@ -32,6 +32,9 @@
 #include <stdlib.h>
 #include <fcntl.h>
 #include <unistd.h>
+#if defined __ANDROID__
+#include <asm/unistd.h> // for __NR_SYSCALL_BASE
+#endif
 #include <errno.h>
 #include <string.h>
 #include <sys/types.h>
@@ -92,13 +95,12 @@
 #include "unicodeOperations.h"
 #include "memaligned.h"
 
-#if defined(__APPLE__) || defined(__linux__)
+#if defined(__linux__)
 #include "hostinfo.h"
 #endif
 
 #if defined(__APPLE__)
 #include <sys/sysctl.h>
-#define XATTR_BACKUP_REENABLED "com.vmware.backupReenabled"
 #endif
 
 /*
@@ -146,12 +148,6 @@ typedef struct FilePosixOptions {
    int aioNumThreads;
 } FilePosixOptions;
 
-#if defined(__APPLE__)
-typedef OSStatus CSBackupSetItemExcludedFunction(CFURLRef item,
-                                                 Boolean exclude,
-                                                 Boolean excludeByPath);
-#endif
-
 
 static FilePosixOptions filePosixOptions;
 
@@ -197,6 +193,8 @@ FileIOErrno2Result(int error)  // IN: errno to convert
       return FILEIO_WRITE_ERROR_DQUOT;
 #endif
    default:
+      Log("%s: Unexpected errno=%d, %s\n", __FUNCTION__,
+          error, Err_Errno2String(error));
       return FILEIO_ERROR;
    }
 }
@@ -858,32 +856,6 @@ FileIO_Create(FileIODescriptor *file,   // OUT:
          goto error;
       }
    }
-
-   /*
-    * Fix for Bug 202805:
-    *   Time Machine backs up EVERY file unless explicitly told not to, so this
-    *   option uses the API to exclude the file that was just opened.
-    */
-
-   if ((access & FILEIO_OPEN_NO_TIME_MACHINE)) {
-      if (!FileIO_SetExcludedFromTimeMachine(pathName, TRUE)) {
-         ret = FILEIO_ERROR;
-         goto error;
-      }
-   } else {
-      /*
-       * Fix for Bug 248644:
-       *   The issue with Time Machine that was causing hangs has been fixed in
-       *   10.5.2, so if the user is in 10.5.2 and the option isn't set, then
-       *   we want to reset the exclusion of the file.
-       *
-       *   Note that this call ignores errors because there are some files
-       *   (like raw devices) that will fail checking xattrs and Time
-       *   Machine Exclusion status, but we can't detect them at this point.
-       */
-
-      FileIO_ResetExcludedFromTimeMachine(pathName);
-   }
 #endif
 
    if (access & FILEIO_OPEN_DELETE_ASAP) {
@@ -1604,7 +1576,8 @@ FileIO_Writev(FileIODescriptor *fd,  // IN:
 }
 
 
-#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__)
+#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) ||\
+    defined(sun)
 
 /*
  *----------------------------------------------------------------------
@@ -1784,7 +1757,8 @@ exit:
 
    return fret;
 }
-#endif /* defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) */
+#endif /* defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) ||
+          defined(sun) */
 #endif /* !defined(N_PLAT_NLM) */
 
 /*
@@ -2177,14 +2151,11 @@ FileIO_PrivilegedPosixOpen(ConstUnicode pathName,  // IN:
  *      Return a FILE * stream equivalent to the given FileIODescriptor.
  *      This is the logical equivalent of Posix dup() then fdopen().
  *
- *      Since the passed descriptor and returned FILE * represent the same
- *      underlying file, and their cursor is shared, you should avoid
- *      interleaving uses to both.
+ *      Caller should fclose the returned descriptor when finished.
  *
  * Results:
- *      A FILE * representing the same underlying file as the passed descriptor
- *      NULL if there was an error.
- *      Caller should fclose the returned descriptor when finished.
+ *      !NULL  A FILE * associated with the file associated with the fd
+ *      NULL   Failure
  *
  * Side effects:
  *      New fd allocated.
@@ -2196,21 +2167,21 @@ FILE *
 FileIO_DescriptorToStream(FileIODescriptor *fdesc,  // IN:
                           Bool textMode)            // IN: unused
 {
-   int dupFd;
+   int dupFD;
    const char *mode;
    int tmpFlags;
    FILE *stream;
 
+   dupFD = dup(fdesc->posix);
+   if (dupFD == -1) {
+      return NULL;
+   }
+
    /* The file you pass us should be valid and opened for *something* */
    ASSERT(FileIO_IsValid(fdesc));
    ASSERT((fdesc->flags & (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_ACCESS_WRITE)) != 0);
    tmpFlags = fdesc->flags & (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_ACCESS_WRITE);
 
-   dupFd = dup(fdesc->posix);
-   if (dupFd == -1) {
-      return NULL;
-   }
-
    if (tmpFlags == (FILEIO_OPEN_ACCESS_READ | FILEIO_OPEN_ACCESS_WRITE)) {
       mode = "r+";
    } else if (tmpFlags == FILEIO_OPEN_ACCESS_WRITE) {
@@ -2219,10 +2190,10 @@ FileIO_DescriptorToStream(FileIODescriptor *fdesc,  // IN:
       mode = "r";
    }
 
-   stream = fdopen(dupFd, mode);
+   stream = fdopen(dupFD, mode);
 
    if (stream == NULL) {
-      close(dupFd);
+      close(dupFD);
    }
 
    return stream;
@@ -2230,158 +2201,6 @@ FileIO_DescriptorToStream(FileIODescriptor *fdesc,  // IN:
 
 
 #if defined(__APPLE__)
-
-/*
- *-----------------------------------------------------------------------------
- *
- * FileIO_ResetExcludedFromTimeMachine --
- *
- *      Request that the given path have its time machine exclusion reset
- *      (turned off). We use a special xattr on the file to mark that we have
- *      done this so that future calls won't clear the file if the user has
- *      explicitly marked it themselves.
- *
- * Results:
- *      A boolean reflecting whether or not reseting the file succeeded.
- *
- * Side effects:
- *      Adds a "backup re-enabled" xattr to the file if wasn't already present
- *      and removing the Time Machine exclusion was successful.
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-FileIO_ResetExcludedFromTimeMachine(char const *pathName)  // IN:
-{
-   bool result = TRUE;
-   char xattr;
-   ssize_t gXattrResult = getxattr(pathName, XATTR_BACKUP_REENABLED,
-                                   &xattr, sizeof(xattr), 0, 0);
-   int sXattrResult;
-
-   if (gXattrResult != -1) {
-      // We have already seen this file, don't touch it again.
-      goto exit;
-   }
-   if (errno != ENOATTR) {
-      LOG_ONCE((LGPFX" %s Couldn't get xattr on path [%s]: %s.\n",
-                __func__, pathName, Err_Errno2String(errno)));
-      result = FALSE;
-      goto exit;
-   }
-   result = FileIO_SetExcludedFromTimeMachine(pathName, FALSE);
-   if (!result) {
-      goto exit;
-   }
-   xattr = '1';
-   sXattrResult = setxattr(pathName, XATTR_BACKUP_REENABLED,
-                           &xattr, sizeof(xattr), 0, 0);
-   if (sXattrResult == -1) {
-      LOG_ONCE((LGPFX" %s Couldn't set xattr on path [%s]: %s.\n",
-                __func__, pathName, Err_Errno2String(errno)));
-      result = FALSE;
-      goto exit;
-   }
-
-exit:
-      return result;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * FileIO_SetExcludedFromTimeMachine --
- *
- *      Request that the given path be excluded from Backup processes (namely
- *      Time Machine).
- *
- * Results:
- *      A boolean reflecting whether or not excluding the file succeeded.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-FileIO_SetExcludedFromTimeMachine(char const *pathName,  // IN:
-                                  Bool isExcluded)       // IN:
-{
-   Bool result = TRUE;
-   CSBackupSetItemExcludedFunction *backupFunc;
-   const char *symbolName = "CSBackupSetItemExcluded";
-   const char *libPath = "/System/Library/Frameworks/CoreServices.framework/"
-      "Versions/Current/Frameworks/CarbonCore.framework/CarbonCore";
-   OSStatus ret = noErr;
-   CFURLRef item = NULL;
-   void *image = NULL;
-
-   if (Config_GetBool(FALSE, "fileMacos.timemachine.enable")) {
-      goto exit;
-   }
-
-   image = dlopen(libPath, RTLD_LAZY | RTLD_GLOBAL);
-
-   if (!image) {
-      LOG_ONCE((LGPFX" %s Couldn't dlopen [%s]: %s.\n",
-               __func__, libPath, Err_Errno2String(errno)));
-      goto exit;
-   }
-
-   backupFunc = (CSBackupSetItemExcludedFunction *) dlsym(image, symbolName);
-
-   if (!backupFunc) {
-      LOG_ONCE((LGPFX" %s Couldn't dlsym [%s]: %s.\n",
-               __func__, symbolName, Err_Errno2String(errno)));
-      goto exit;
-   }
-
-   item = CFURLCreateFromFileSystemRepresentation(NULL, pathName,
-                                                  strlen((const char *)pathName),
-                                                  FALSE);
-
-   if (!item) {
-      Warning("%s: Error creating URL\n", __func__);
-      result = FALSE;
-      goto exit;
-   }
-
-   ret = (*backupFunc)(item, isExcluded, FALSE);
-
-   if (ret != noErr) {
-      /*
-       * TODO: In testing, this only actually fails on NFS-mounted
-       * volumes. (ret == noPerm).  Perhaps we should enumerate the errors
-       * that we want to treat as soft errors versus hard errors?
-       *
-       * We could call CSBackupIsItemExcluded() to make sure this file isn't
-       * participating in backup. This needs to be tested to see if it only
-       * checks the file system extensible meta-data or if it also checks the
-       * policies of the backup daemon.
-       *
-       * Ideally, we would like to log once for every file that we fail to
-       * mark as excluded. Simply logging produces way too many log satements
-       * and logging once would only note when this fails on the first file.
-       */
-
-      goto exit;
-   }
-
-exit:
-   if (item) {
-      CFRelease(item);
-   }
-   if (image) {
-      dlclose(image);
-   }
-
-   return result;
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
