@@ -1054,14 +1054,14 @@ FileLockNormalizePath(ConstUnicode filePath)  // IN:
  *----------------------------------------------------------------------
  */
 
-void *
+FileLockToken *
 FileLock_Lock(ConstUnicode filePath,         // IN:
               const Bool readOnly,           // IN:
               const uint32 msecMaxWaitTime,  // IN:
               int *err)                      // OUT:
 {
-   void *lockToken;
    Unicode normalizedPath;
+   FileLockToken *tokenPtr;
 
    ASSERT(filePath);
    ASSERT(err);
@@ -1070,20 +1070,20 @@ FileLock_Lock(ConstUnicode filePath,         // IN:
    if (normalizedPath == NULL) {
       *err = EINVAL;
 
-      lockToken = NULL;
+      tokenPtr = NULL;
    } else {
       char creationTimeString[32];
 
       Str_Sprintf(creationTimeString, sizeof creationTimeString, "%"FMT64"u",
                   ProcessCreationTime(getpid()));
 
-      lockToken = FileLockIntrinsic(normalizedPath, !readOnly, msecMaxWaitTime,
-                                    creationTimeString, err);
+      tokenPtr = FileLockIntrinsic(normalizedPath, !readOnly, msecMaxWaitTime,
+                                   creationTimeString, err);
 
       Unicode_Free(normalizedPath);
    }
 
-   return lockToken;
+   return tokenPtr;
 }
 
 
@@ -1148,25 +1148,11 @@ FileLock_IsLocked(ConstUnicode filePath,  // IN:
  */
 
 int
-FileLock_Unlock(ConstUnicode filePath,  // IN:
-                const void *lockToken)  // IN:
+FileLock_Unlock(const FileLockToken *lockToken)  // IN:
 {
-   int err;
-   Unicode normalizedPath;
-
-   ASSERT(filePath);
    ASSERT(lockToken);
 
-   normalizedPath = FileLockNormalizePath(filePath);
-   if (normalizedPath == NULL) {
-      err = EINVAL;
-   } else {
-      err = FileUnlockIntrinsic(normalizedPath, lockToken);
-
-      Unicode_Free(normalizedPath);
-   }
-
-   return err;
+   return FileUnlockIntrinsic((FileLockToken *) lockToken);
 }
 
 
