@@ -38,6 +38,9 @@
 #  include <sys/mount.h>
 # else
 #  include <mntent.h>
+#  if defined __ANDROID__
+#   include <paths.h> /* for _PATH_MOUNTED */
+#  endif
 # endif
 #include <signal.h>
 #endif
@@ -659,6 +662,7 @@ File_GetTimes(ConstUnicode pathName,       // IN:
    if (Posix_Lstat(pathName, &statBuf) == -1) {
       Log(LGPFX" %s: error stating file \"%s\": %s\n", __FUNCTION__,
           UTF8(pathName), Err_Errno2String(errno));
+
       return FALSE;
    }
 
@@ -1068,12 +1072,6 @@ File_GetVMFSAttributes(ConstUnicode pathName,             // IN: File to test
 
    *fsAttrs = Util_SafeMalloc(FS_PARTITION_ARR_SIZE(FS_PLIST_DEF_MAX_PARTITIONS));
 
-   if (*fsAttrs == NULL) {
-      Log(LGPFX" %s: failed to allocate memory\n", __func__);
-      ret = -1;
-      goto bail;
-   }
-
    memset(*fsAttrs, 0, FS_PARTITION_ARR_SIZE(FS_PLIST_DEF_MAX_PARTITIONS));
 
    (*fsAttrs)->ioctlAttr.maxPartitions = FS_PLIST_DEF_MAX_PARTITIONS;
@@ -1082,7 +1080,8 @@ File_GetVMFSAttributes(ConstUnicode pathName,             // IN: File to test
    fd = Posix_Open(parentPath, O_RDONLY, 0);
 
    if (fd == -1) {
-      Log(LGPFX" %s: could not open %s.\n", __func__, UTF8(pathName));
+      Log(LGPFX" %s: could not open %s: %s\n", __func__, UTF8(pathName),
+          Err_Errno2String(errno));
       ret = -1;
       goto bail;
    }
@@ -1091,8 +1090,8 @@ File_GetVMFSAttributes(ConstUnicode pathName,             // IN: File to test
 
    close(fd);
    if (ret == -1) {
-      Log(LGPFX" %s: Could not get volume attributes (ret = %d)\n", __func__,
-          ret);
+      Log(LGPFX" %s: Could not get volume attributes (ret = %d): %s\n",
+          __func__, ret, Err_Errno2String(errno));
    }
 
 bail:
@@ -1204,6 +1203,48 @@ File_GetVMFSMountInfo(ConstUnicode pathName,  // IN:
 #endif 
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * FileIsVMFS --
+ *
+ *      Is the given file on a filesystem that supports vmfs-specific
+ *      features like zeroed-thick and multiwriter files?
+ *
+ * Results:
+ *      TRUE if we're on VMFS.
+ *
+ * Side effects:
+ *      None
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+FileIsVMFS(ConstUnicode pathName) // IN
+{
+   Bool result = FALSE;
+
+#if defined(VMX86_SERVER)
+   /* Right now only VMFS supports zeroedThick and multiWriter. */
+   FS_PartitionListResult *fsAttrs = NULL;
+
+   if (File_GetVMFSAttributes(pathName, &fsAttrs) >= 0) {
+      result = strncmp(fsAttrs->fsType, FS_VMFS_ON_ESX,
+                       sizeof(FS_VMFS_ON_ESX)) == 0;
+   } else {
+      Log(LGPFX" %s: File_GetVMFSAttributes failed\n", __func__);
+   }
+
+   if (fsAttrs) {
+      free(fsAttrs);
+   }
+#endif
+ 
+   return result;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1224,27 +1265,35 @@ File_GetVMFSMountInfo(ConstUnicode pathName,  // IN:
  */
 
 Bool
-File_SupportsZeroedThick(ConstUnicode pathName) // IN: File name to test
+File_SupportsZeroedThick(ConstUnicode pathName) // IN
 {
-   Bool result = FALSE;
+   return FileIsVMFS(pathName);
+}
 
-#if defined(VMX86_SERVER)
-   /* Right now only VMFS supports ZeroedThick */
-   FS_PartitionListResult *fsAttrs = NULL;
 
-   if (File_GetVMFSAttributes(pathName, &fsAttrs) >= 0) {
-      result = strncmp(fsAttrs->fsType, FS_VMFS_ON_ESX,
-                       sizeof(FS_VMFS_ON_ESX)) == 0;
-   } else {
-      Log(LGPFX" %s: File_GetVMFSAttributes failed\n", __func__);
-   }
+/*
+ *----------------------------------------------------------------------
+ *
+ * File_SupportsMultiWriter --
+ *  
+ *      Check if the given file is on an FS supports opening files
+ *      in multi-writer mode.
+ *      Currently only VMFS on ESX supports multi-writer mode, but
+ *      this may change in the future.
+ *
+ * Results:
+ *      TRUE if FS supports opening files in multi-writer mode.
+ *
+ * Side effects:
+ *      None
+ *
+ *----------------------------------------------------------------------
+ */
 
-   if (fsAttrs) {
-      free(fsAttrs);
-   }
-#endif
- 
-   return result;
+Bool
+File_SupportsMultiWriter(ConstUnicode pathName) // IN
+{
+   return FileIsVMFS(pathName);
 }
 
 
@@ -1395,7 +1444,16 @@ FilePosixLookupMountPoint(char const *canPath,  // IN: Canonical file path
    size = 4 * FILE_MAXPATH;  // Should suffice for most locales
 
 retry:
+#if defined __ANDROID__
+   /*
+    * Android supports neither setmntent() nor MOUNTED.
+    * The code below is a workaround.
+    */
+   NOT_TESTED();
+   f = fopen(_PATH_MOUNTED, "r");
+#else
    f = setmntent(MOUNTED, "r");
+#endif
    if (f == NULL) {
       return NULL;
    }
@@ -1977,10 +2035,15 @@ File_VMFSSupportsFileSize(ConstUnicode pathName,  // IN:
    if (strcmp(fsAttrs->fsType, FS_VMFS_ON_ESX) == 0) {
       if (fsAttrs->versionNumber == 3) {
          maxFileSize = (VMFS3CONST * (uint64) fsAttrs->fileBlockSize * 1024);
-      } else {
+      } else if (fsAttrs->versionNumber >= 5) {
          /* Get ready for 64 TB on VMFS5 and perform sanity check on version */
-         ASSERT(fsAttrs->versionNumber == 5);
          maxFileSize = (uint64) 0x400000000000ULL;
+      } else {
+         Log(LGPFX" %s: Unsupported filesystem version, %u\n", __func__,
+             fsAttrs->versionNumber);
+         free(fsAttrs);
+
+         return FALSE;
       }
 
       if (fileSize <= maxFileSize && maxFileSize != -1) {
