@@ -114,6 +114,18 @@
  */
 #define CONFNAME_UNITY_BACKGROUNDCOLOR "desktop.backgroundColor"
 
+/**
+ * Lets users enable (or disable) the Protocol Buffer enabled service
+ */
+#define CONFNAME_UNITY_ENABLEPBRPC "pbrpc.enable"
+
+/**
+ * Lets users configure the socket type for the PBRPC Services
+ */
+#define CONFNAME_UNITY_PBRPCSOCKETTYPE "pbrpc.socketType"
+#define CONFNAME_UNITY_PBRPCSOCKETTYPE_IPSOCKET "ipsocket"
+#define CONFNAME_UNITY_PBRPCSOCKETTYPE_VSOCKET "vsocket"
+
 /*
  * END Unity goodies.
  ******************************************************************************
@@ -143,7 +155,4 @@
 /* Wait 5 seconds between polls to see if the conf file has changed */
 #define CONF_POLL_TIME     500
 
-GuestApp_Dict *Conf_Load(void);
-Bool Conf_ReloadFile(GuestApp_Dict **pConfDict);
-
 #endif /* __CONF_H__ */
