@@ -147,10 +147,6 @@ typedef enum {
  * (supported on ESX file systems)
  */
 #define FILEIO_OPEN_MULTIWRITER_LOCK     (1 << 14)
-/*
- * Flag the file as not to be backed up by Time Machine on Mac OS X.
- */
-#define FILEIO_OPEN_NO_TIME_MACHINE      (1 << 15)
 /*
  * Valid only for MacOS. It eventually results into O_EXLOCK flag passed to open
  * system call.
@@ -379,7 +375,7 @@ ConstUnicode FileIO_Filename(FileIODescriptor *fd);
 /*
  *-------------------------------------------------------------------------
  * 
- * FileIO_Success --
+ * FileIO_IsSuccess --
  *
  *      Returns TRUE if the error code is success.
  *
@@ -408,12 +404,6 @@ FileIO_IsSuccess(FileIOResult res)      // IN
 Bool FileIO_IsSuccess(FileIOResult res);
 #endif
 
-#if defined(__APPLE__)
-EXTERN Bool FileIO_ResetExcludedFromTimeMachine(char const *pathName);
-EXTERN Bool FileIO_SetExcludedFromTimeMachine(char const *pathName,
-                                              Bool isExcluded);
-#endif
-
 Bool FileIO_SupportsPrealloc(const char *pathName, Bool fsCheck);
 
 #endif // _FILEIO_H_
