@@ -72,7 +72,7 @@ typedef enum GuestOSType {
    GUEST_OS_SOLARIS9            = GUEST_OS_BASE + 34,
    GUEST_OS_SOLARIS10           = GUEST_OS_BASE + 35,
    GUEST_OS_SOLARIS10_64        = GUEST_OS_BASE + 36,
-   GUEST_OS_VMKERNEL            = GUEST_OS_BASE + 37,
+   GUEST_OS_VMKERNEL            = GUEST_OS_BASE + 37, // ESX 4.x
    GUEST_OS_DARWIN9             = GUEST_OS_BASE + 38, // Mac OS 10.5
    GUEST_OS_DARWIN9_64          = GUEST_OS_BASE + 39,
    GUEST_OS_DARWIN10            = GUEST_OS_BASE + 40, // Mac OS 10.6
@@ -92,6 +92,7 @@ typedef enum GuestOSType {
    GUEST_OS_WINEIGHT            = GUEST_OS_BASE + 54, // Windows 8
    GUEST_OS_WINEIGHT_64         = GUEST_OS_BASE + 55, // Windows 8 x64
    GUEST_OS_WINEIGHTSERVER_64   = GUEST_OS_BASE + 56, // Windows 8 Server X64
+   GUEST_OS_VMKERNEL5           = GUEST_OS_BASE + 57, // ESX 5.x and later
 } GuestOSType;
 
 
@@ -183,79 +184,80 @@ typedef enum GuestOSFamilyType {
 
 #define ALLOS2          (BS(OS2) | BS(ECOMSTATION))
 
+#define ALLVMKERNEL     (BS(VMKERNEL) | BS(VMKERNEL5))
+
 
 /*
  * These constants are generated by GuestInfoGetOSName which is in
  * the bora-vmsoft subtree.
  */
 
+/* vmkernel (ESX) */
+#define STR_OS_ESX_4               "VMware ESX 4"
+#define STR_OS_ESX_5               "VMware ESX 5"
+
 /* Linux */
-#define STR_OS_ANNVIX              "Annvix" 
-#define STR_OS_ARCH                "Arch" 
-#define STR_OS_ARKLINUX            "Arklinux" 
+#define STR_OS_ANNVIX              "Annvix"
+#define STR_OS_ARCH                "Arch"
+#define STR_OS_ARKLINUX            "Arklinux"
 #define STR_OS_ASIANUX_3           "asianux3"
 #define STR_OS_ASIANUX_4           "asianux4"
-#define STR_OS_AUROX               "Aurox" 
-#define STR_OS_ASIANUX             "asianux" 
-#define STR_OS_BLACKCAT            "BlackCat" 
+#define STR_OS_AUROX               "Aurox"
+#define STR_OS_ASIANUX             "asianux"
+#define STR_OS_BLACKCAT            "BlackCat"
 #define STR_OS_CENTOS              "centos"
-#define STR_OS_COBALT              "Cobalt" 
-#define STR_OS_CONECTIVA           "Conectiva" 
+#define STR_OS_COBALT              "Cobalt"
+#define STR_OS_CONECTIVA           "Conectiva"
 #define STR_OS_DEBIAN              "Debian"
 #define STR_OS_DEBIAN_4            "debian4"
 #define STR_OS_DEBIAN_5            "debian5"
-#define STR_OS_FEDORA              "Fedora" 
-#define STR_OS_GENTOO              "Gentoo" 
-#define STR_OS_IMMUNIX             "Immunix" 
-#define STR_OS_LINUX               "linux" 
-#define STR_OS_LINUX_FROM_SCRATCH "Linux-From-Scratch" 
+#define STR_OS_DEBIAN_6            "debian6"
+#define STR_OS_FEDORA              "Fedora"
+#define STR_OS_GENTOO              "Gentoo"
+#define STR_OS_IMMUNIX             "Immunix"
+#define STR_OS_LINUX               "linux"
+#define STR_OS_LINUX_FROM_SCRATCH "Linux-From-Scratch"
 #define STR_OS_LINUX_FULL         "Other Linux"
-#define STR_OS_LINUX_PPC          "Linux-PPC" 
-#define STR_OS_MANDRAKE           "mandrake" 
-#define STR_OS_MANDRAKE_FULL      "Mandrake Linux"   
-#define STR_OS_MANDRIVA           "mandriva"    
-#define STR_OS_MKLINUX            "MkLinux"    
-#define STR_OS_NOVELL             "nld9"    
-#define STR_OS_NOVELL_FULL        "Novell Linux Desktop 9" 
+#define STR_OS_LINUX_PPC          "Linux-PPC"
+#define STR_OS_MANDRAKE           "mandrake"
+#define STR_OS_MANDRAKE_FULL      "Mandrake Linux"  
+#define STR_OS_MANDRIVA           "mandriva"   
+#define STR_OS_MKLINUX            "MkLinux"   
+#define STR_OS_NOVELL             "nld9"   
+#define STR_OS_NOVELL_FULL        "Novell Linux Desktop 9"
 #define STR_OS_ORACLE             "oraclelinux"
-#define STR_OS_OTHER              "otherlinux"    
-#define STR_OS_OTHER_24           "other24xlinux"    
-#define STR_OS_OTHER_24_FULL      "Other Linux 2.4.x kernel" 
-#define STR_OS_OTHER_26           "other26xlinux"    
-#define STR_OS_OTHER_26_FULL      "Other Linux 2.6.x kernel" 
-#define STR_OS_OTHER_FULL         "Other Linux"   
-#define STR_OS_PLD                "PLD"    
-#define STR_OS_RED_HAT            "redhat"   
-#define STR_OS_RED_HAT_EN         "rhel"    
-#define STR_OS_RED_HAT_EN_2       "rhel2"    
-#define STR_OS_RED_HAT_EN_2_FULL  "Red Hat Enterprise Linux 2"
-#define STR_OS_RED_HAT_EN_3       "rhel3"    
-#define STR_OS_RED_HAT_EN_3_FULL  "Red Hat Enterprise Linux 3"
-#define STR_OS_RED_HAT_EN_4       "rhel4"    
-#define STR_OS_RED_HAT_EN_4_FULL  "Red Hat Enterprise Linux 4"
-#define STR_OS_RED_HAT_FULL       "Red Hat Linux"  
-#define STR_OS_SLACKWARE          "Slackware"    
+#define STR_OS_OTHER              "otherlinux"   
+#define STR_OS_OTHER_24           "other24xlinux"   
+#define STR_OS_OTHER_24_FULL      "Other Linux 2.4.x kernel"
+#define STR_OS_OTHER_26           "other26xlinux"   
+#define STR_OS_OTHER_26_FULL      "Other Linux 2.6.x kernel"
+#define STR_OS_OTHER_FULL         "Other Linux"  
+#define STR_OS_PLD                "PLD"   
+#define STR_OS_RED_HAT            "redhat"  
+#define STR_OS_RED_HAT_EN         "rhel"   
+#define STR_OS_RED_HAT_FULL       "Red Hat Linux" 
+#define STR_OS_SLACKWARE          "Slackware"   
+#define STR_OS_SLES               "sles"   
+#define STR_OS_SLES_FULL          "SUSE Linux Enterprise Server"
 #define STR_OS_SLES_10            "sles10"
 #define STR_OS_SLES_10_FULL       "SUSE Linux Enterprise Server 10"
 #define STR_OS_SLES_11            "sles11"
 #define STR_OS_SLES_11_FULL       "SUSE Linux Enterprise Server 11"
-#define STR_OS_SMESERVER          "SMEServer"    
-#define STR_OS_SUN_DESK           "sjds"    
-#define STR_OS_SUN_DESK_FULL      "Sun Java Desktop System" 
-#define STR_OS_SUSE               "suse"    
-#define STR_OS_SUSE_EN            "sles"    
-#define STR_OS_SUSE_EN_FULL       "SUSE Linux Enterprise Server" 
+#define STR_OS_SUSE               "suse"   
 #define STR_OS_SUSE_FULL          "SUSE Linux"
-#define STR_OS_TINYSOFA           "Tiny Sofa"   
-#define STR_OS_TURBO              "turbolinux"    
-#define STR_OS_TURBO_FULL         "Turbolinux"    
-#define STR_OS_UBUNTU             "Ubuntu" 
-#define STR_OS_ULTRAPENGUIN       "UltraPenguin" 
-#define STR_OS_UNITEDLINUX        "UnitedLinux" 
-#define STR_OS_VALINUX            "VALinux" 
+#define STR_OS_OPENSUSE           "opensuse"
+#define STR_OS_SMESERVER          "SMEServer"   
+#define STR_OS_SUN_DESK           "sjds"   
+#define STR_OS_SUN_DESK_FULL      "Sun Java Desktop System"
+#define STR_OS_TINYSOFA           "Tiny Sofa"  
+#define STR_OS_TURBO              "turbolinux"   
+#define STR_OS_TURBO_FULL         "Turbolinux"   
+#define STR_OS_UBUNTU             "Ubuntu"
+#define STR_OS_ULTRAPENGUIN       "UltraPenguin"
+#define STR_OS_UNITEDLINUX        "UnitedLinux"
+#define STR_OS_VALINUX            "VALinux"
 #define STR_OS_YELLOW_DOG         "Yellow Dog"
 #define STR_OS_ECOMSTATION        "eComStation"
-#define STR_OS_ECOMSTATION2       "eComStation2"
 
 /* Windows */
 #define STR_OS_WIN_31                   "win31"
@@ -282,7 +284,7 @@ typedef enum GuestOSFamilyType {
 #define STR_OS_WIN_XP_PRO_FULL          "Windows XP Professional"
 #define STR_OS_WIN_XP_PRO_X64           "winXPPro-64"
 #define STR_OS_WIN_XP_PRO_X64_FULL      "Windows XP Professional x64 Edition"
-#define STR_OS_WIN_NET_WEB              "winNetWeb" 
+#define STR_OS_WIN_NET_WEB              "winNetWeb"
 #define STR_OS_WIN_NET_WEB_FULL         "Windows Server 2003 Web Edition"
 #define STR_OS_WIN_NET_ST               "winNetStandard"
 #define STR_OS_WIN_NET_ST_FULL          "Windows Server 2003 Standard Edition"
