@@ -111,6 +111,9 @@ typedef uint32 HgfsSendFlags;
 #define HGFS_SEND_CAN_DELAY         (1 << 0)
 #define HGFS_SEND_NO_COMPLETE       (1 << 1)
 
+// Channel capability flags
+#define HGFS_CHANNEL_SHARED_MEM     (1 << 0)
+#define HGFS_CHANNEL_ASYNC          (1 << 1)
 
 typedef Bool
 HgfsSessionSendFunc(void *opaqueSession,  // IN
@@ -127,7 +130,7 @@ typedef struct HgfsServerChannelCallbacks {
 }HgfsServerChannelCallbacks;
 
 typedef struct HgfsServerSessionCallbacks {
-   Bool (*connect)(void *, HgfsServerChannelCallbacks *, void **);
+   Bool (*connect)(void *, HgfsServerChannelCallbacks *, uint32 ,void **);
    void (*disconnect)(void *);
    void (*close)(void *);
    void (*receive)(HgfsPacket *packet, void *);
@@ -141,12 +144,6 @@ void HgfsServer_ExitState(void);
 uint32 HgfsServer_GetHandleCounter(void);
 void HgfsServer_SetHandleCounter(uint32 newHandleCounter);
 
-#ifdef VMX86_TOOLS
-void HgfsServer_ProcessPacket(char const *packetIn,
-                              char *packetOut,
-                              size_t *packetSize);
-#endif
-
 /*
  * Function pointers used for getting names in HgfsServerGetDents
  *
@@ -176,4 +173,19 @@ HgfsCleanupFunc(void *);  // IN
 typedef void
 HgfsInvalidateObjectsFunc(DblLnkLst_Links *shares); // IN
 
+/*
+ * Function used to notify HGFS server that a shared folder has been created or updated.
+ * It allows HGFS server to maintain up-to-date list of shared folders and its
+ * properties.
+ */
+typedef uint32 HgfsSharedFolderHandle;
+#define HGFS_INVALID_FOLDER_HANDLE         ((HgfsSharedFolderHandle)~((HgfsSharedFolderHandle)0))
+
+typedef HgfsSharedFolderHandle
+HgfsRegisterSharedFolderFunc(const char *shareName,
+                             const char *sharePath,
+                             Bool addFolder);
+HgfsSharedFolderHandle HgfsServer_RegisterSharedFolder(const char *shareName,
+                                                       const char *sharePath,
+                                                       Bool addFolder);
 #endif // _HGFS_SERVER_H_
