@@ -97,6 +97,7 @@ typedef struct MsgCallback {
    int (*progressList)(MsgList *messages, int percent, Bool cancelButton);
    HintResult (*hintList)(HintOptions options, MsgList *messages);
    void *(*lazyProgressStartList)(MsgList *messages);
+   void (*forceUnblock)(void);
 } MsgCallback;
 
 #define MSG_QUESTION_MAX_BUTTONS   10
@@ -167,7 +168,7 @@ EXTERN int Msg_CompareAnswer(Msg_String const *buttons, unsigned answer,
 EXTERN char *Msg_GetString(const char *idString);
 EXTERN char *Msg_GetStringSafe(const char *idString);
 EXTERN char *Msg_GetPlainButtonText(const char *idString);
-EXTERN const char *Msg_GetLocale(void);
+EXTERN char *Msg_GetLocale(void);
 EXTERN void Msg_SetLocale(const char *locale, const char *binaryName);
 EXTERN void Msg_SetLocaleEx(const char *locale, const char *binaryName,
                             const char *baseDirPath);
@@ -176,6 +177,7 @@ EXTERN char *Msg_GetMessageFilePath(const char *locale, const char *binaryName,
 EXTERN char *Msg_FormatFloat(double value, unsigned int precision);
 EXTERN char *Msg_FormatSizeInBytes(uint64 size);
 EXTERN Bool Msg_LoadMessageFile(const char *locale, const char *fileName);
+EXTERN void Msg_ForceUnblock(void);
 
 
 /*
@@ -189,6 +191,7 @@ EXTERN MsgList *Msg_GetMsgListAndReset(void);
 EXTERN char *Msg_LocalizeList(const MsgList *messages);
 EXTERN void Msg_Reset(Bool log);
 EXTERN Bool Msg_Present(void);
+EXTERN void Msg_ExitThread(void);
 EXTERN void Msg_Exit(void);
 
 
@@ -225,7 +228,10 @@ EXTERN void Msg_Exit(void);
  */
 
 EXTERN void Msg_SetCallback(MsgCallback *cb);
+EXTERN void Msg_SetThreadCallback(MsgCallback *cb);
+
 EXTERN void Msg_GetCallback(MsgCallback *cb);
+EXTERN void Msg_GetThreadCallback(MsgCallback *cb);
 
 
 /*
@@ -240,5 +246,4 @@ EXTERN void Msg_GetCallback(MsgCallback *cb);
 EXTERN const char *Msg_HResult2String(long hr);
 #endif
 
-
 #endif // ifndef _MSG_H_
