@@ -235,6 +235,7 @@
 #define UNITY_RPC_WINDOW_UNSTICK          "unity.window.unstick"
 #define UNITY_RPC_CONFIRM_OPERATION       "unity.operation.confirm"
 #define UNITY_RPC_WINDOW_CONTENTS_REQUEST "unity.window.contents.request"
+#define UNITY_RPC_REGISTER_PBRPCSERVER    "unity.register.pbrpcserver"
 #define UNITY_RPC_SEND_MOUSE_WHEEL        "unity.sendMouseWheel"
 
 #define GHI_RPC_GUEST_SHELL_ACTION                    "ghi.guest.shell.action"
@@ -434,6 +435,37 @@ typedef uint32 UnityIconSize; // Number of pixels on the larger side of the icon
 
 #define UNITY_DEFAULT_COLOR "#c0c0c0"
 
+/*
+ * List of operations that can be interlocked with the host, via a request, confirm,
+ * acknowledge sequence of RPCs.
+ */
+typedef enum {
+   MINIMIZE = 1
+} UnityOperations;
+
+
+/*
+ * List of features (as a bitmask) which may be optionally enabled when entering
+ * Unity mode. By default all these features are disabled.
+ */
+typedef enum {
+   UNITY_ADD_HIDDEN_WINDOWS_TO_TRACKER = 1,
+   UNITY_INTERLOCK_MINIMIZE_OPERATION = 1 << 1,
+   UNITY_SEND_WINDOW_CONTENTS = 1 << 2,
+   UNITY_DISABLE_COMPOSITING_IN_GUEST = 1 << 3
+}  UnityFeatures;
+
+
+/*
+ * UNITY_RPC_REGISTER_PBRPCSERVER includes an enumerated Address Family to distinguish
+ * between PBRPC servers listening on TCP/IP sockets and those listening on VSockets.
+ */
+typedef enum {
+   InvalidAddressFamily = 0,
+   VSocketAddressFamily = 1,
+   INET4AddressFamily = 2
+} UnityPBRPCAddressFamily;
+
 
 /*
  * Multipage Doxygen documentation.
@@ -522,13 +554,17 @@ execPath ::= ? UTF-8 string uniquely identifying JUST the executable ?
    @brief       Get the start menu sub-tree for a given item
    @todo        Move this into a GHI-specific header.
    @code
-   UNITY_RPC_OPEN_LAUNCHMENU root
+   UNITY_RPC_OPEN_LAUNCHMENU root [flags]
    @endcode
-   @param[in] root      Name of sub-tree, or "" for root of start menu.
+   @param[in] root  Name of sub-tree, or "" for root of start menu.
+   @param[in] flags @ref UNITY_START_MENU_FLAG_USE_PROGRAMS_FOLDER_AS_ROOT
    @return
-   <tt>"count handle"</tt>
-   \li @c count Number of items in the sub-tree.
-   \li @c handle Opaque handle passed to @ref UNITY_RPC_GET_LAUNCHMENU_ITEM.
+@verbatim
+retval ::= count handle
+
+count  ::= ? Number of items in the sub-tree. ?
+handle ::= ? Opaque 32-bit integer used with subsequent calls to UNITY_RPC_GET_LAUNCHMENU_ITEM. ?
+@endverbatim
 
 
    @def         UNITY_RPC_GET_LAUNCHMENU_ITEM
@@ -539,7 +575,15 @@ execPath ::= ? UTF-8 string uniquely identifying JUST the executable ?
    @endcode
    @param[in] handle    Handle returned by @ref UNITY_RPC_OPEN_LAUNCHMENU.
    @param[in] index     index of the item to retrieve (zero-indexed).
-   @return Executable path corresponding to menu item as UTF-8, or ""
+   @return
+@verbatim
+retval ::= name '\0' flags '\0' shellPath '\0' localName
+
+name ::= ? Canonical item name used as a unique ID. ?
+flags ::= 0 | 1 (* 0 = leaf node, 1 = directory *)
+shellPath ::= ? Opaque executable path sent to UNITY_RPC_SHELL_OPEN. ?
+localName ::= ? Localized item name displayed by UI. ?
+@endverbatim
    @sa UNITY_RPC_SHELL_OPEN
 
 
@@ -802,6 +846,13 @@ desktop where the upper right <tt>{1,2}</tt> is the currently active desktop.
    @param[in] deltaZ the distance the wheel is rotated in the Z axis
    @param[in] modifierFlags modifier flags pressed during the event
 
+   @def         UNITY_RPC_WINDOW_CONTENTS_REQUEST
+   @brief       Request the asynchronous delivery of window contents.
+   @code
+   UNITY_RPC_WINDOW_CONTENTS_REQUEST XDR_REP
+   @endcode
+   @param[in] XDR_REP XDR Encoded (see unity.x) representation of arguments.
+
    @}
 */
 
@@ -884,6 +935,27 @@ desktop where the upper right <tt>{1,2}</tt> is the currently active desktop.
    @endcode
    @param[in] XDR_REP XDR Encoded (see unity.x) representation of arguments.
 
+   @def         UNITY_RPC_WINDOW_CONTENTS_START
+   @brief       The start of data for the pixel contents of the window.
+   @code
+   UNITY_RPC_WINDOW_CONTENTS_START XDR_REP
+   @endcode
+   @param[in] XDR_REP XDR Encoded (see unity.x) representation of arguments.
+
+   @def         UNITY_RPC_WINDOW_CONTENTS_CHUNK
+   @brief       One (<64KB) chunk of Pixel Data for a previously started window.
+   @code
+   UNITY_RPC_WINDOW_CONTENTS_CHUNK XDR_REP
+   @endcode
+   @param[in] XDR_REP XDR Encoded (see unity.x) representation of arguments.
+
+   @def         UNITY_RPC_WINDOW_CONTENTS_END
+   @brief       The end of data for the pixel contents of the window.
+   @code
+   UNITY_RPC_WINDOW_CONTENTS_END XDR_REP
+   @endcode
+   @param[in] XDR_REP XDR Encoded (see unity.x) representation of arguments.
+
    @}
 */
 
