@@ -169,8 +169,10 @@ enum {
    VIX_E_NO_DISPLAY_SERVER                      = 3043,
    VIX_E_VM_NOT_RECORDING                       = 3044,
    VIX_E_VM_NOT_REPLAYING                       = 3045,
+   VIX_E_TOO_MANY_LOGONS                        = 3046,
+   VIX_E_INVALID_AUTHENTICATION_SESSION         = 3047,
 
-   /* VM Errors */ 
+   /* VM Errors */
    VIX_E_VM_NOT_FOUND                           = 4000,
    VIX_E_NOT_SUPPORTED_FOR_VM_VERSION           = 4001,
    VIX_E_CANNOT_READ_VM_CONFIG                  = 4002,
@@ -304,6 +306,7 @@ enum {
    VIX_E_NOT_A_DIRECTORY                        = 20002,
    VIX_E_NO_SUCH_PROCESS                        = 20003,
    VIX_E_FILE_NAME_TOO_LONG                     = 20004,
+   VIX_E_OPERATION_DISABLED                     = 20005,
 
    /* Tools install errors */
    VIX_E_TOOLS_INSTALL_NO_IMAGE                 = 21000,
@@ -452,6 +455,7 @@ enum {
    VIX_PROPERTY_VM_SUPPORTED_FEATURES                 = 197,
    VIX_PROPERTY_VM_IS_RECORDING                       = 236,
    VIX_PROPERTY_VM_IS_REPLAYING                       = 237,
+   VIX_PROPERTY_VM_SSL_ERROR                          = 293,
 
    /* Result properties; these are returned by various procedures */
    VIX_PROPERTY_JOB_RESULT_ERROR_CODE                 = 3000,
@@ -559,15 +563,17 @@ void Vix_FreeBuffer(void *p);
 typedef int VixHostOptions;
 enum {
    VIX_HOSTOPTION_USE_EVENT_PUMP        = 0x0008,
+   VIX_HOSTOPTION_VERIFY_SSL_CERT       = 0x4000,
 };
 
 typedef int VixServiceProvider;
 enum {
-   VIX_SERVICEPROVIDER_DEFAULT               = 1,
-   VIX_SERVICEPROVIDER_VMWARE_SERVER         = 2,
-   VIX_SERVICEPROVIDER_VMWARE_WORKSTATION    = 3,
-   VIX_SERVICEPROVIDER_VMWARE_PLAYER         = 4,
-   VIX_SERVICEPROVIDER_VMWARE_VI_SERVER      = 10,
+   VIX_SERVICEPROVIDER_DEFAULT                   = 1,
+   VIX_SERVICEPROVIDER_VMWARE_SERVER             = 2,
+   VIX_SERVICEPROVIDER_VMWARE_WORKSTATION        = 3,
+   VIX_SERVICEPROVIDER_VMWARE_PLAYER             = 4,
+   VIX_SERVICEPROVIDER_VMWARE_VI_SERVER          = 10,
+   VIX_SERVICEPROVIDER_VMWARE_WORKSTATION_SHARED = 11,
 };
 
 /*
