@@ -59,6 +59,8 @@
 #define VIX_USER_CREDENTIAL_HOST_CONFIG_SECRET        6
 #define VIX_USER_CREDENTIAL_HOST_CONFIG_HASHED_SECRET 7
 #define VIX_USER_CREDENTIAL_NAMED_INTERACTIVE_USER    8
+#define VIX_USER_CREDENTIAL_TICKETED_SESSION          9
+#define VIX_USER_CREDENTIAL_SSPI                      10
 
 #define VIX_SHARED_SECRET_CONFIG_USER_NAME          "__VMware_Vix_Shared_Secret_1__"
 
@@ -96,6 +98,7 @@ enum {
    VIX_REQUESTMSG_REQUIRES_INTERACTIVE_ENVIRONMENT    = 0x08,
    VIX_REQUESTMSG_INCLUDES_AUTH_DATA_V1               = 0x10,
    VIX_REQUESTMSG_REQUIRES_VMDB_NOTIFICATION          = 0x20,
+   VIX_REQUESTMSG_ESCAPE_XML_DATA                     = 0x40,
 };
 
 
@@ -287,6 +290,29 @@ struct VixCommandNamePassword {
 #include "vmware_pack_end.h"
 VixCommandNamePassword;
 
+/*
+ * **********************************************************
+ * This is a ticketed session for authentication.
+ */
+typedef
+#include "vmware_pack_begin.h"
+struct VixCommandTicketedSession {
+   uint32    ticketLength;
+}
+#include "vmware_pack_end.h"
+VixCommandTicketedSession;
+
+/*
+ * **********************************************************
+ * This is a SSPI token for acquiring credentials
+ */
+typedef
+#include "vmware_pack_begin.h"
+struct VixCommandSSPI {
+   uint32    tokenLength;
+}
+#include "vmware_pack_end.h"
+VixCommandSSPI;
 
 /*
  * **********************************************************
@@ -458,30 +484,6 @@ enum VixMsgConfigObjectType {
    VIX_LAN_SEGMENT_SETTING_CONFIG   = 1,
 };
 
-/*
- * Commands related to Record|Replay State.
- */
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgRecordReplayStateCommandRequest {
-   VixCommandRequestHeader   header;
-   
-   int32                     options;
-   uint32                    propertyListBufferSize;
-}
-#include "vmware_pack_end.h"
-VixMsgRecordReplayStateCommandRequest;
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgRecordReplayStateCommandResponse {
-   VixCommandResponseHeader header;
-   uint32   propertyListBufferSize;
-}
-#include "vmware_pack_end.h"
-VixMsgRecordReplayStateCommandResponse;
-
-
 /*
  * **********************************************************
  * Wait for tools request. The response is just a generic
@@ -695,7 +697,6 @@ struct VixMsgSetGuestFileAttributesRequest {
    VixCommandRequestHeader header;
 
    int32                  fileOptions;
-   int64                  createTime;
    int64                  accessTime;
    int64                  modificationTime;
    int32                  ownerId;
@@ -756,6 +757,18 @@ struct VixMsgListFilesRequest {
 #include "vmware_pack_end.h"
 VixMsgListFilesRequest;
 
+typedef
+#include "vmware_pack_begin.h"
+struct VixCommandInitiateFileTransferToGuestRequest {
+   VixCommandRequestHeader header;
+
+   int32                   options;
+   uint32                  guestPathNameLength;
+   Bool                    overwrite;
+}
+#include "vmware_pack_end.h"
+VixCommandInitiateFileTransferToGuestRequest;
+
 
 /*
  * This is used to reply to several operations, like testing whether
@@ -1211,15 +1224,6 @@ struct VixMsgRecordReplayEvent {
 #include "vmware_pack_end.h"
 VixMsgRecordReplayEvent;
 
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgTimeMarkerEncounteredEvent {
-   VixMsgEventHeader        eventHeader;      
-   uint32                   propertyListSize;
-}
-#include "vmware_pack_end.h"
-VixMsgTimeMarkerEncounteredEvent;
-
 typedef
 #include "vmware_pack_begin.h"
 struct VixMsgDebuggerEvent {
@@ -1233,51 +1237,6 @@ struct VixMsgDebuggerEvent {
 #include "vmware_pack_end.h"
 VixMsgDebuggerEvent;
 
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgGetRecordReplayInfoResponse {
-   VixCommandResponseHeader header;
-   uint32                   propertyListSize;
-}
-#include "vmware_pack_end.h"
-VixMsgGetRecordReplayInfoResponse;
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgVMSnapshotSetReplaySpeedRequest {
-   VixCommandRequestHeader    header;
-
-   int32                      options;
-   int32                      timeType;
-   int64                      timeValue;
-}
-#include "vmware_pack_end.h"
-VixMsgVMSnapshotSetReplaySpeedRequest;
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgVMAddTimeMarkerRequest {
-   VixCommandRequestHeader    header;
-   uint32                     options;
-   uint32                     action;
-   uint32                     propertyListSize;
-}
-#include "vmware_pack_end.h"
-VixMsgVMAddTimeMarkerRequest;
-
-typedef
-#include "vmware_pack_begin.h"
-struct VixMsgVMGetTimeMarkerRequest {
-   VixCommandRequestHeader    header;
-   uint32                     options;
-   uint32                     whence;
-   uint32                     index;
-   uint32                     propertyListSize;
-}
-#include "vmware_pack_end.h"
-VixMsgVMGetTimeMarkerRequest;
-
-
 /*
  * Fault Tolerance Automation
  */
@@ -1593,6 +1552,19 @@ struct VixMsgCreateTempFileRequestEx {
 VixMsgCreateTempFileRequestEx;
 
 
+typedef
+#include "vmware_pack_begin.h"
+struct {
+   VixCommandRequestHeader header;
+
+   int32                   fileOptions;
+   uint32                  guestPathNameLength;
+   uint32                  filePropertiesLength;
+   Bool                    recursive;
+}
+#include "vmware_pack_end.h"
+VixMsgDeleteDirectoryRequest;
+
 /*
  * **********************************************************
  * Connect/Disconnect device request. The response is just a generic
@@ -1631,7 +1603,7 @@ VixMsgGetVProbesResponse;
 typedef
 #include "vmware_pack_begin.h"
 struct VixMsgVProbeLoadRequest {
-   VixCommandResponseHeader header;
+   VixCommandRequestHeader header;
    char   string[1];             /* variable length */
 } 
 #include "vmware_pack_end.h"
@@ -1802,6 +1774,59 @@ struct VixMsgGetPerformanceDataResponse {
 VixMsgGetPerformanceDataResponse;
 
 
+/*
+ * Run a program in guest with (VI version with more args)
+ */
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgStartProgramRequest {
+   VixCommandRequestHeader   header;
+
+   Bool   startMinimized;
+   uint32 programPathLength;
+   uint32 argumentsLength;
+   uint32 workingDirLength;
+   uint32 numEnvVars;
+   uint32 envVarLength;
+
+   // This is followed by the buffer of the args
+}
+#include "vmware_pack_end.h"
+VixMsgStartProgramRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgListProcessesExRequest {
+   VixCommandRequestHeader   header;
+
+   // if we need to make multiple trips, this is the key used to identify
+   // the result being processed
+   uint32 key;
+
+   // if we need to make multiple trips, this is the offset in the reply
+   // from which to send the next chunk
+   uint32 offset;
+   uint32 numPids;
+
+   // This is followed by the list of uint64s
+}
+#include "vmware_pack_end.h"
+VixMsgListProcessesExRequest;
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixMsgReadEnvironmentVariablesRequest {
+   VixCommandRequestHeader   header;
+
+   uint32 numNames;
+   uint32 namesLength;
+
+   // This is followed by the list of NUL-terminated names
+}
+#include "vmware_pack_end.h"
+VixMsgReadEnvironmentVariablesRequest;
+
+
 /*
  * HOWTO: Adding a new Vix Command. Step 3.
  *
@@ -1991,6 +2016,20 @@ struct VixCommandListFileSystemsRequest {
 #include "vmware_pack_end.h"
 VixCommandListFileSystemsRequest;
 
+/*
+ * **********************************************************
+ * Acquire Credentials.
+ */
+
+typedef
+#include "vmware_pack_begin.h"
+struct VixCommandAcquireCredentialsRequest {
+   VixCommandRequestHeader    header;
+
+   int64                      sessionID;
+}
+#include "vmware_pack_end.h"
+VixCommandAcquireCredentialsRequest;
 
 /*
  * **********************************************************
@@ -2091,12 +2130,12 @@ enum {
    VIX_COMMAND_CANCEL_INSTALL_TOOLS             = 26,
    VIX_COMMAND_UPGRADE_VIRTUAL_HARDWARE         = 27,
    VIX_COMMAND_SET_NIC_BANDWIDTH                = 28,
-   VIX_COMMAND_CREATE_DISK                      = 29,
-   VIX_COMMAND_CREATE_FLOPPY                    = 30,
+   /* DEPRECATED VIX_COMMAND_CREATE_DISK                      = 29, */
+   /* DEPRECATED VIX_COMMAND_CREATE_FLOPPY                    = 30, */
    VIX_COMMAND_RELOAD_VM                        = 31,
    VIX_COMMAND_DELETE_VM                        = 32,
-   VIX_COMMAND_SYNCDRIVER_FREEZE                = 33,
-   VIX_COMMAND_SYNCDRIVER_THAW                  = 34,
+   /* DEPRECATED VIX_COMMAND_SYNCDRIVER_FREEZE                = 33, */
+   /* DEPRECATED VIX_COMMAND_SYNCDRIVER_THAW                  = 34, */
    /* DEPRECATED VIX_COMMAND_HOT_ADD_DISK                     = 35, */
    /* DEPRECATED VIX_COMMAND_HOT_REMOVE_DISK                  = 36, */
    VIX_COMMAND_SET_GUEST_PRINTER                = 37,
@@ -2182,17 +2221,17 @@ enum {
    VIX_COMMAND_VM_PAUSE                         = 122,
    VIX_COMMAND_VM_UNPAUSE                       = 123,
    /* DEPRECATED VIX_COMMAND_GET_SNAPSHOT_LOG_INFO            = 124, */
-   VIX_COMMAND_SET_REPLAY_SPEED                 = 125,
+   /* DEPRECATED VIX_COMMAND_SET_REPLAY_SPEED                 = 125, */
 
    /* DEPRECATED VIX_COMMAND_ANSWER_USER_MESSAGE              = 126, */
    /* DEPRECATED VIX_COMMAND_SET_CLIENT_LOCALE                = 127, */
 
    VIX_COMMAND_GET_PERFORMANCE_DATA             = 128,
 
-   VIX_COMMAND_REFRESH_RUNTIME_PROPERTIES       = 129,
+   /* DEPRECATED VIX_COMMAND_REFRESH_RUNTIME_PROPERTIES       = 129, */
 
    VIX_COMMAND_GET_SNAPSHOT_SCREENSHOT          = 130,
-   VIX_COMMAND_ADD_TIMEMARKER                   = 131,
+   /* DEPRECATED VIX_COMMAND_ADD_TIMEMARKER                   = 131, */
 
    VIX_COMMAND_WAIT_FOR_USER_ACTION_IN_GUEST    = 132,
    /* DEPRECATED VIX_COMMAND_VMDB_END_TRANSACTION             = 133, */
@@ -2209,20 +2248,20 @@ enum {
    VIX_COMMAND_DEBUGGER_DETACH                  = 141,
    VIX_COMMAND_DEBUGGER_SEND_COMMAND            = 142,
 
-   VIX_COMMAND_GET_RECORD_STATE                 = 143,
-   VIX_COMMAND_SET_RECORD_STATE                 = 144,
-   VIX_COMMAND_REMOVE_RECORD_STATE              = 145,
-   VIX_COMMAND_GET_REPLAY_STATE                 = 146,
-   VIX_COMMAND_SET_REPLAY_STATE                 = 147,
-   VIX_COMMAND_REMOVE_REPLAY_STATE              = 148,
+   /* DEPRECATED VIX_COMMAND_GET_RECORD_STATE                 = 143, */
+   /* DEPRECATED VIX_COMMAND_SET_RECORD_STATE                 = 144, */
+   /* DEPRECATEDVIX_COMMAND_REMOVE_RECORD_STATE              = 145, */
+   /* VIX_COMMAND_GET_REPLAY_STATE                 = 146, */
+   /* VIX_COMMAND_SET_REPLAY_STATE                 = 147, */
+   /* VIX_COMMAND_REMOVE_REPLAY_STATE              = 148, */
 
    /* DEPRECATED VIX_COMMAND_CANCEL_USER_PROGRESS_MESSAGE     = 150, */
    
    VIX_COMMAND_GET_VMX_DEVICE_STATE             = 151,
 
-   VIX_COMMAND_GET_NUM_TIMEMARKERS              = 152,
-   VIX_COMMAND_GET_TIMEMARKER                   = 153,
-   VIX_COMMAND_REMOVE_TIMEMARKER                = 154,
+   /* DEPRECATED VIX_COMMAND_GET_NUM_TIMEMARKERS              = 152, */
+   /* DEPRECATED VIX_COMMAND_GET_TIMEMARKER                   = 153, */
+   /* DEPRECATED VIX_COMMAND_REMOVE_TIMEMARKER                = 154, */
 
    VIX_COMMAND_SET_SNAPSHOT_INFO                = 155,
    VIX_COMMAND_SNAPSHOT_SET_MRU                 = 156,
@@ -2275,6 +2314,23 @@ enum {
    VIX_COMMAND_SET_GUEST_FILE_ATTRIBUTES        = 183,
 
    VIX_COMMAND_COPY_FILE_FROM_GUEST_TO_READER   = 184,
+
+   VIX_COMMAND_START_PROGRAM                    = 185,
+
+   VIX_COMMAND_LIST_PROCESSES_EX                = 186,
+
+   VIX_COMMAND_READ_ENV_VARIABLES               = 187,
+
+   VIX_COMMAND_INITIATE_FILE_TRANSFER_FROM_GUEST   = 188,
+   VIX_COMMAND_INITIATE_FILE_TRANSFER_TO_GUEST     = 189,
+
+   VIX_COMMAND_ACQUIRE_CREDENTIALS              = 190,
+   VIX_COMMAND_RELEASE_CREDENTIALS              = 191,
+   VIX_COMMAND_VALIDATE_CREDENTIALS             = 192,
+   VIX_COMMAND_TERMINATE_PROCESS                = 193,
+   VIX_COMMAND_DELETE_GUEST_FILE_EX             = 194,
+   VIX_COMMAND_DELETE_GUEST_DIRECTORY_EX        = 195,
+
    /*
     * HOWTO: Adding a new Vix Command. Step 2a.
     *
@@ -2283,9 +2339,9 @@ enum {
     * When people add new command id's in different tree, they may collide and use
     * the same ID values. This can merge without conflicts, and cause runtime bugs.
     * Once a new command is added here, a command info field needs to be added
-    * in bora/lib/foundryMsg. as well.
+    * in bora/lib/foundryMsg/foundryMsg.c as well.
     */
-   VIX_COMMAND_LAST_NORMAL_COMMAND              = 185,
+   VIX_COMMAND_LAST_NORMAL_COMMAND              = 196,
 
    VIX_TEST_UNSUPPORTED_TOOLS_OPCODE_COMMAND    = 998,
    VIX_TEST_UNSUPPORTED_VMX_OPCODE_COMMAND      = 999,
@@ -2368,6 +2424,35 @@ enum VixRunProgramResultValues {
 #endif
 
 
+/*
+ * This is used to denote that the contents of a VIX XML-like response
+ * string has been escaped. Old Tools did not escape the contents.
+ * This tag is only used for existing commands that did not originally perform
+ * escaping. Any new command must always escape any strings passed in XML.
+ * See ListProcessesInGuest as an example.
+ * The protocol works as follows:
+ * 1) A client library that internally knows how to handle escaped XML opts in
+ *    by including the VIX_REQUESTMSG_ESCAPE_XML_DATA in relevent requests.
+ * 2) Tools that understands the VIX_REQUESTMSG_ESCAPE_XML_DATA flag sees that
+ *    it is set in the request, and then escapes all string data within the
+ *    XML response. To indicate to the client that it has understood the
+ *    request, it include the VIX_XML_ESCAPED_TAG in the response (at the
+ *    begining of the response).
+ * 3) When the client library receives the response, it searches for the
+ *    VIX_XML_ESCAPED_TAG. If it is present, it then unescapes all string
+ *    data in the response. If the tag is not present, the client library
+ *    assumes that the Tools did not understand VIX_REQUESTMSG_ESCAPE_XML_DATA
+ *    and that the string data is not escaped.
+ * The following characters are escaped: '<', '>', and '%'.
+ * For new commands (starting with those released in M/N for the vSphere
+ * guest ops project), the escaping is exactly the same, but the
+ * VIX_REQUESTMSG_ESCAPE_XML_DATA flag and the VIX_XML_ESCAPED_TAG are not
+ * used, since both ends expect escaping.
+ */
+#define VIX_XML_ESCAPED_TAG   "<escaped/>"
+
+#define VIX_XML_ESCAPE_CHARACTER '%'
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -2412,16 +2497,17 @@ VixError VixMsg_ParseWriteVariableRequest(VixMsgWriteVariableRequest *msg,
                                           char **valueName,
                                           char **value);
 
-char *VixMsg_ObfuscateNamePassword(const char *userName,
-                                   const char *password);
+VixError VixMsg_ObfuscateNamePassword(const char *userName,
+                                      const char *password,
+                                      char **result);
 
-Bool VixMsg_DeObfuscateNamePassword(const char *packagedName,
-                                    char **userNameResult,
-                                    char **passwordResult);
+VixError VixMsg_DeObfuscateNamePassword(const char *packagedName,
+                                        char **userNameResult,
+                                        char **passwordResult);
 
-char *VixMsg_EncodeString(const char *str);
+VixError VixMsg_EncodeString(const char *str, char **result);
 
-char *VixMsg_DecodeString(const char *str);
+VixError VixMsg_DecodeString(const char *str, char **result);
 
 Bool VixMsg_ValidateCommandInfoTable(void);
 
@@ -2450,10 +2536,108 @@ VixError VixMsg_ParseGenericRequestMsg(const VixCommandGenericRequest *request,
                                        int *options,
                                        VixPropertyListImpl *propertyList);
 
+VixError
+VixMsg_ParseSimpleResponseWithString(const VixCommandResponseHeader *response,
+                                     const char **result);
+
 void *VixMsg_MallocClientData(size_t size);
 void *VixMsg_ReallocClientData(void *ptr, size_t size);
 char *VixMsg_StrdupClientData(const char *s, Bool *allocateFailed);
 
+/*
+ * Parser state used by VMAutomationMsgParser* group of functions.
+ */
+typedef struct {
+   const char *currentPtr;
+   const char *endPtr;
+} VMAutomationMsgParser;
+
+/* Keep the original type name around all the old code can stay the same. */
+typedef VMAutomationMsgParser VMAutomationRequestParser;
+
+
+#define VMAutomationRequestParserInit VMAutomationMsgParserInitRequest
+#define VMAutomationMsgParserInitRequest(state, msg, fixedLength) \
+   __VMAutomationMsgParserInitRequest(__FUNCTION__, __LINE__, state, msg, fixedLength)
+VixError
+__VMAutomationMsgParserInitRequest(const char *caller,
+                                   unsigned int line,
+                                   VMAutomationMsgParser *state,
+                                   const struct VixCommandRequestHeader *msg,
+                                   size_t fixedLength);
+
+#define VMAutomationMsgParserInitResponse(state, msg, fixedLength) \
+   __VMAutomationMsgParserInitResponse(__FUNCTION__, __LINE__, state, msg, fixedLength)
+VixError
+__VMAutomationMsgParserInitResponse(const char *caller,
+                                    unsigned int line,
+                                    VMAutomationMsgParser *state,
+                                    const struct VixCommandResponseHeader *msg,
+                                    size_t fixedLength);
+
+#define VMAutomationRequestParserGetRemainingData \
+   VMAutomationMsgParserGetRemainingData
+const void *
+VMAutomationMsgParserGetRemainingData(VMAutomationMsgParser *state,
+                                      size_t *length);
+
+#define VMAutomationRequestParserGetData VMAutomationMsgParserGetData
+#define VMAutomationMsgParserGetData(state, length, result) \
+   __VMAutomationMsgParserGetData(__FUNCTION__, __LINE__,               \
+                                  state, length, (const char **)result)
+VixError __VMAutomationMsgParserGetData(const char *caller,
+                                        unsigned int line,
+                                        VMAutomationMsgParser *state,
+                                        size_t length,
+                                        const char **result);
+
+#define VMAutomationRequestParserGetOptionalString \
+   VMAutomationMsgParserGetOptionalString
+#define VMAutomationMsgParserGetOptionalString(state, length, result) \
+   __VMAutomationMsgParserGetOptionalString(__FUNCTION__, __LINE__,     \
+                                            state, length, result)
+VixError __VMAutomationMsgParserGetOptionalString(const char *caller,
+                                                  unsigned int line,
+                                                  VMAutomationMsgParser *state,
+                                                  size_t length,
+                                                  const char **result);
+
+#define VMAutomationRequestParserGetOptionalStrings \
+   VMAutomationMsgParserGetOptionalStrings
+#define VMAutomationMsgParserGetOptionalStrings(state, count, length,     \
+           result)                                                            \
+   __VMAutomationMsgParserGetOptionalStrings(__FUNCTION__, __LINE__,      \
+                                             state, count, length, result)
+VixError __VMAutomationMsgParserGetOptionalStrings
+   (const char *caller,
+    unsigned int line,
+    VMAutomationMsgParser *state,
+    uint32 count,
+    size_t length,
+    const char **result);
+
+#define VMAutomationRequestParserGetString VMAutomationMsgParserGetString
+#define VMAutomationMsgParserGetString(state, length, result) \
+   __VMAutomationMsgParserGetString(__FUNCTION__, __LINE__,             \
+                                    state, length, result)
+VixError __VMAutomationMsgParserGetString(const char *caller,
+                                          unsigned int line,
+                                          VMAutomationMsgParser *state,
+                                          size_t length,
+                                          const char **result);
+
+#define VMAutomationRequestParserGetPropertyList \
+   VMAutomationMsgParserGetPropertyList
+#define VMAutomationMsgParserGetPropertyList(state, length, propList) \
+   __VMAutomationMsgParserGetPropertyList(__FUNCTION__, __LINE__,       \
+                                          state, length, propList)
+VixError
+__VMAutomationMsgParserGetPropertyList(const char *caller,
+                                       unsigned int line,
+                                       VMAutomationMsgParser *state,
+                                       size_t length,
+                                       VixPropertyListImpl *propList);
+
 #endif   // VIX_HIDE_FROM_JAVA
 
 
