@@ -81,7 +81,7 @@ extern "C" {
 /*
  * It seems x86 & x86-64 windows still implements these intrinsic
  * functions.  The documentation for the x86-64 suggest the
- * __inbyte/__outbyte intrinsics eventhough the _in/_out work fine and
+ * __inbyte/__outbyte intrinsics even though the _in/_out work fine and
  * __inbyte/__outbyte aren't supported on x86.
  */
 int            _inp(unsigned short);
@@ -126,6 +126,14 @@ unsigned char  _BitScanForward(unsigned long *, unsigned long);
 unsigned char  _BitScanReverse(unsigned long *, unsigned long);
 #pragma intrinsic(_BitScanForward, _BitScanReverse)
 
+unsigned char  _bittestandset(long *, long);
+unsigned char  _bittestandreset(long *, long);
+#pragma intrinsic(_bittestandset, _bittestandreset)
+#ifdef VM_X86_64
+unsigned char  _bittestandset64(__int64 *, __int64);
+unsigned char  _bittestandreset64(__int64 *, __int64);
+#pragma intrinsic(_bittestandset64, _bittestandreset64)
+#endif /* VM_X86_64 */
 #ifdef __cplusplus
 }
 #endif
@@ -498,11 +506,23 @@ mssb64(const uint64 value)
 }
 
 #ifdef __GNUC__
-#if defined(__i386__) || defined(__x86_64__)
+#if defined(__i386__) || defined(__x86_64__) || defined(__arm__)
 
 static INLINE void *
 uint16set(void *dst, uint16 val, size_t count)
 {
+#ifdef __arm__
+   if (count <= 0)
+       return dst;
+   __asm__ __volatile__ ("\t"
+                         "1: strh %0, [%1]     \n\t"
+                         "   subs %2, %2, #1   \n\t"
+                         "   bne 1b                "
+                         :: "r" (val), "r" (dst), "r" (count)
+                         : "memory"
+        );
+   return dst;
+#else
    int dummy0;
    int dummy1;
 
@@ -515,11 +535,24 @@ uint16set(void *dst, uint16 val, size_t count)
       );
 
    return dst;
+#endif
 }
 
 static INLINE void *
 uint32set(void *dst, uint32 val, size_t count)
 {
+#ifdef __arm__
+   if (count <= 0)
+       return dst;
+   __asm__ __volatile__ ("\t"
+                         "1: str %0, [%1]     \n\t"
+                         "   subs %2, %2, #1  \n\t"
+                         "   bne 1b               "
+                         :: "r" (val), "r" (dst), "r" (count)
+                         : "memory"
+        );
+   return dst;
+#else
    int dummy0;
    int dummy1;
 
@@ -532,6 +565,7 @@ uint32set(void *dst, uint32 val, size_t count)
       );
 
    return dst;
+#endif
 }
 
 #else /* unknown system: rely on C to write */
@@ -554,7 +588,7 @@ uint32set(void *dst, uint32 val, size_t count)
    }
    return dst;
 }
-#endif // defined(__i386__) || defined(__x86_64__)
+#endif // defined(__i386__) || defined(__x86_64__) || defined(__arm__)
 #elif defined(_MSC_VER)
 
 static INLINE void *
@@ -626,7 +660,11 @@ Bswap16(uint16 v)
 static INLINE uint32
 Bswap32(uint32 v) // IN
 {
-#if defined(__GNUC__) && (defined(__i386__) || defined(__x86_64__)) // {
+#if defined(__GNUC__) && (defined(__i386__) || defined(__x86_64__)) || defined(__arm__) // {
+#ifdef __arm__
+    __asm__("rev %0, %0" : "+r"(v));
+    return v;
+#else // __arm__
    /* Checked against the Intel manual and GCC. --hpreg */
    __asm__(
       "bswap %0"
@@ -634,6 +672,7 @@ Bswap32(uint32 v) // IN
       : "0" (v)
    );
    return v;
+#endif // !__arm__
 #else // } {
    return    (v >> 24)
           | ((v >>  8) & 0xFF00)
@@ -661,7 +700,7 @@ Bswap64(uint64 v) // IN
 }
 
 
-#if defined(__i386__) || defined(__x86_64__)
+#if defined(__i386__) || defined(__x86_64__) || defined(__arm__)
 /*
  * COMPILER_MEM_BARRIER prevents the compiler from re-ordering memory
  * references accross the barrier.  NOTE: It does not generate any
@@ -671,7 +710,7 @@ Bswap64(uint64 v) // IN
 #define COMPILER_MEM_BARRIER()   __asm__ __volatile__ ("": : :"memory")
 #define COMPILER_READ_BARRIER()  COMPILER_MEM_BARRIER()
 #define COMPILER_WRITE_BARRIER() COMPILER_MEM_BARRIER()
-#elif _MSC_VER
+#elif defined(_MSC_VER)
 #define COMPILER_MEM_BARRIER()   _ReadWriteBarrier()
 #define COMPILER_READ_BARRIER()  _ReadBarrier()
 #define COMPILER_WRITE_BARRIER() _WriteBarrier()
@@ -687,7 +726,14 @@ static INLINE void
 PAUSE(void)
 #ifdef __GNUC__
 {
+#ifdef __arm__
+   /*
+    * ARM has no instruction to execute "spin-wait loop", just leave it
+    * empty.
+    */
+#else
    __asm__ __volatile__( "pause" :);
+#endif
 }
 #elif defined(_MSC_VER)
 #ifdef VM_X86_64
@@ -725,7 +771,7 @@ RDTSC(void)
    );
 
    return tscHigh << 32 | tscLow;
-#else
+#elif defined(__i386__)
    uint64 tim;
 
    __asm__ __volatile__(
@@ -734,6 +780,11 @@ RDTSC(void)
    );
 
    return tim;
+#else
+   /*
+    * For platform without cheap timer, just return 0.
+    */
+   return 0;
 #endif
 }
 #elif defined(_MSC_VER)
@@ -768,6 +819,85 @@ RDTSC(void)
 #else
 #define DEBUGBREAK()   __asm__ (" int $3 ")
 #endif
-#endif // defined(__i386__) || defined(__x86_64__)
+#endif // defined(__i386__) || defined(__x86_64__) || defined(__arm__)
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * {Clear,Set}Bit{32,64} --
+ *
+ *    Sets or clears a specified single bit in the provided variable.  
+ *    The index input value specifies which bit to modify and is 0-based. 
+ *    Index is truncated by hardware to a 5-bit or 6-bit offset for the 
+ *    32 and 64-bit flavors, respectively, but input values are not validated
+ *    with asserts to avoid include dependencies.
+ *    64-bit flavors are not provided for 32-bit builds because the inlined
+ *    version can defeat user or compiler optimizations.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE void
+SetBit32(uint32 *var, unsigned index)
+{
+#ifdef __GNUC__
+   __asm__ (
+      "bts %1, %0"
+      : "+mr" (*var)
+      : "ri" (index)
+      : "cc"
+   );
+#elif defined(_MSC_VER)
+   _bittestandset((long *)var, index);
+#endif
+}
+
+static INLINE void
+ClearBit32(uint32 *var, unsigned index)
+{
+#ifdef __GNUC__
+   __asm__ (
+      "btr %1, %0"
+      : "+mr" (*var)
+      : "ri" (index)
+      : "cc"
+   );
+#elif defined(_MSC_VER)
+   _bittestandreset((long *)var, index);
+#endif
+}
+
+#if defined(VM_X86_64)
+static INLINE void
+SetBit64(uint64 *var, unsigned index)
+{
+#ifdef __GNUC__
+   __asm__ (
+      "bts %1, %0"
+      : "+mr" (*var)
+      : "ri" (index)
+      : "cc"
+   );
+#elif defined _MSC_VER
+   _bittestandset64((__int64 *)var, index);
+#endif
+}
+
+static INLINE void
+ClearBit64(uint64 *var, unsigned index)
+{
+#ifdef __GNUC__
+   __asm__ (
+      "btr %1, %0"
+      : "+mr" (*var)
+      : "ri" (index)
+      : "cc"
+   );
+#elif defined _MSC_VER
+   _bittestandreset64((__int64 *)var, index);
+#endif
+}
+#endif /* VM_X86_64 */
 
 #endif // _VM_BASIC_ASM_H_
