@@ -95,34 +95,36 @@ uint64 __shiftright128(uint64 lowPart, uint64 highPart, uint8 shift);
 #if defined(__GNUC__)
 
 static INLINE void 
-FXSAVE_ES1(uint8 *save)
+FXSAVE_ES1(void *save)
 {
-   __asm__ __volatile__ ("fxsaveq %0  \n" : "=m" (*save) : : "memory");
+   __asm__ __volatile__ ("fxsaveq %0  \n" : "=m" (*(uint8 *)save) : : "memory");
 }
 
 static INLINE void 
-FXSAVE_COMPAT_ES1(uint8 *save)
+FXSAVE_COMPAT_ES1(void *save)
 {
-   __asm__ __volatile__ ("fxsave %0  \n" : "=m" (*save) : : "memory");
+   __asm__ __volatile__ ("fxsave %0  \n" : "=m" (*(uint8 *)save) : : "memory");
 }
 
 static INLINE void 
-FXRSTOR_ES1(const uint8 *load)
+FXRSTOR_ES1(const void *load)
 {
-   __asm__ __volatile__ ("fxrstorq %0 \n" : : "m" (*load) : "memory");
+   __asm__ __volatile__ ("fxrstorq %0 \n"
+                         : : "m" (*(const uint8 *)load) : "memory");
 }
 
 static INLINE void 
-FXRSTOR_COMPAT_ES1(const uint8 *load)
+FXRSTOR_COMPAT_ES1(const void *load)
 {
-   __asm__ __volatile__ ("fxrstor %0 \n" : : "m" (*load) : "memory");
+   __asm__ __volatile__ ("fxrstor %0 \n"
+                         : : "m" (*(const uint8 *)load) : "memory");
 }
 
 static INLINE void 
-FXRSTOR_AMD_ES0(const uint8 *load)
+FXRSTOR_AMD_ES0(const void *load)
 {
    uint64 dummy = 0;
-      
+
    __asm__ __volatile__ 
        ("fnstsw  %%ax    \n"     // Grab x87 ES bit
         "bt      $7,%%ax \n"     // Test ES bit
@@ -134,7 +136,7 @@ FXRSTOR_AMD_ES0(const uint8 *load)
                                  // x87 exception pointers.
         "fxrstorq %1 \n"
         :
-        : "m" (dummy), "m" (*load)
+        : "m" (dummy), "m" (*(const uint8 *)load)
         : "ax", "memory");
 }
 
@@ -150,82 +152,96 @@ FXRSTOR_AMD_ES0(const uint8 *load)
 #if defined(__GNUC__) && (defined(VMM) || defined(VMKERNEL) || defined(FROBOS))
 
 static INLINE void 
-XSAVE_ES1(uint8 *save, uint64 mask)
+XSAVE_ES1(void *save, uint64 mask)
 {
 #if __GNUC__ < 4 || __GNUC__ == 4 && __GNUC_MINOR__ == 1
    __asm__ __volatile__ (
         ".byte 0x48, 0x0f, 0xae, 0x21 \n"
         :
-        : "c" (save), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "c" ((uint8 *)save), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #else
    __asm__ __volatile__ (
         "xsaveq %0 \n"
-        : "=m" (*save)
+        : "=m" (*(uint8 *)save)
         : "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #endif
 }
 
 static INLINE void 
-XSAVE_COMPAT_ES1(uint8 *save, uint64 mask)
+XSAVE_COMPAT_ES1(void *save, uint64 mask)
 {
 #if __GNUC__ < 4 || __GNUC__ == 4 && __GNUC_MINOR__ == 1
    __asm__ __volatile__ (
         ".byte 0x0f, 0xae, 0x21 \n"
         :
-        : "c" (save), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "c" ((uint8 *)save), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #else
    __asm__ __volatile__ (
         "xsave %0 \n"
-        : "=m" (*save)
+        : "=m" (*(uint8 *)save)
         : "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #endif
 }
 
 static INLINE void 
-XRSTOR_ES1(const uint8 *load, uint64 mask)
+XSAVEOPT_ES1(void *save, uint64 mask)
+{
+   __asm__ __volatile__ (
+        ".byte 0x48, 0x0f, 0xae, 0x31 \n"
+        :
+        : "c" ((uint8 *)save), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "memory");
+}
+
+static INLINE void 
+XRSTOR_ES1(const void *load, uint64 mask)
 {
 #if __GNUC__ < 4 || __GNUC__ == 4 && __GNUC_MINOR__ == 1
    __asm__ __volatile__ (
         ".byte 0x48, 0x0f, 0xae, 0x29 \n"
         :
-        : "c" (load), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "c" ((const uint8 *)load),
+          "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #else
    __asm__ __volatile__ (
         "xrstorq %0 \n"
         :
-        : "m" (*load), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "m" (*(const uint8 *)load),
+          "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #endif
 }
 
 static INLINE void 
-XRSTOR_COMPAT_ES1(const uint8 *load, uint64 mask)
+XRSTOR_COMPAT_ES1(const void *load, uint64 mask)
 {
 #if __GNUC__ < 4 || __GNUC__ == 4 && __GNUC_MINOR__ == 1
    __asm__ __volatile__ (
         ".byte 0x0f, 0xae, 0x29 \n"
         :
-        : "c" (load), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "c" ((const uint8 *)load),
+          "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #else
    __asm__ __volatile__ (
         "xrstor %0 \n"
         :
-        : "m" (*load), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "m" (*(const uint8 *)load),
+          "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #endif
 }
 
 static INLINE void 
-XRSTOR_AMD_ES0(const uint8 *load, uint64 mask)
+XRSTOR_AMD_ES0(const void *load, uint64 mask)
 {
    uint64 dummy = 0;
-      
+
    __asm__ __volatile__ 
        ("fnstsw  %%ax    \n"     // Grab x87 ES bit
         "bt      $7,%%ax \n"     // Test ES bit
@@ -239,12 +255,13 @@ XRSTOR_AMD_ES0(const uint8 *load, uint64 mask)
 #if __GNUC__ < 4 || __GNUC__ == 4 && __GNUC_MINOR__ == 1
         ".byte 0x48, 0x0f, 0xae, 0x29 \n"
         :
-        : "m" (dummy),
-          "c" (load), "b" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "m" (dummy), "c" ((const uint8 *)load),
+          "b" ((uint32)mask), "d" ((uint32)(mask >> 32))
 #else
         "xrstorq %1 \n"
         :
-        : "m" (dummy), "m" (*load), "b" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "m" (dummy), "m" (*(const uint8 *)load),
+          "b" ((uint32)mask), "d" ((uint32)(mask >> 32))
 #endif
         : "eax", "memory");
 }
@@ -255,13 +272,13 @@ XRSTOR_AMD_ES0(const uint8 *load, uint64 mask)
 /*
  *-----------------------------------------------------------------------------
  *
- * Mul64x3264 --
+ * Mul64x6464 --
  *
  *    Unsigned integer by fixed point multiplication:
  *       result = multiplicand * multiplier >> shift
  * 
  *       Unsigned 64-bit integer multiplicand.
- *       Unsigned 32-bit fixed point multiplier, represented as
+ *       Unsigned 64-bit fixed point multiplier, represented as
  *         multiplier >> shift, where shift < 64.
  *       Unsigned 64-bit integer product.
  *
@@ -279,18 +296,17 @@ XRSTOR_AMD_ES0(const uint8 *load, uint64 mask)
 #if defined(__GNUC__)
 
 static INLINE uint64
-Mul64x3264(uint64 multiplicand,
-           uint32 multiplier,
+Mul64x6464(uint64 multiplicand,
+           uint64 multiplier,
            uint32 shift)
 {
    uint64 result, dummy;
-   const uint64 multiplier64 = multiplier;
 
    __asm__("mulq    %3      \n\t"
            "shrdq   %1, %0  \n\t"
            : "=a" (result),
              "=d" (dummy)
-           : "0"  (multiplier64),
+           : "0"  (multiplier),
              "rm" (multiplicand),
          "c"  (shift)
            : "cc");
@@ -300,7 +316,7 @@ Mul64x3264(uint64 multiplicand,
 #elif defined(_MSC_VER)
 
 static INLINE uint64
-Mul64x3264(uint64 multiplicand, uint32 multiplier, uint32 shift)
+Mul64x6464(uint64 multiplicand, uint64 multiplier, uint32 shift)
 {
    uint64 tmplo, tmphi;
    tmplo = _umul128(multiplicand, multiplier, &tmphi);
@@ -312,13 +328,13 @@ Mul64x3264(uint64 multiplicand, uint32 multiplier, uint32 shift)
 /*
  *-----------------------------------------------------------------------------
  *
- * Muls64x32s64 --
+ * Muls64x64s64 --
  *
  *    Signed integer by fixed point multiplication:
  *       result = multiplicand * multiplier >> shift
  * 
  *       Signed 64-bit integer multiplicand.
- *       Unsigned 32-bit fixed point multiplier, represented as
+ *       Unsigned 64-bit fixed point multiplier, represented as
  *         multiplier >> shift, where shift < 64.
  *       Signed 64-bit integer product.
  *
@@ -340,16 +356,15 @@ Mul64x3264(uint64 multiplicand, uint32 multiplier, uint32 shift)
 #if defined(__GNUC__)
 
 static inline int64
-Muls64x32s64(int64 multiplicand, uint32 multiplier, uint32 shift)
+Muls64x64s64(int64 multiplicand, int64 multiplier, uint32 shift)
 {
    int64 result, dummy;
-   const int64 multiplier64 = multiplier;
 
    __asm__("imulq   %3      \n\t"
        "shrdq   %1, %0  \n\t"
        : "=a" (result),
          "=d" (dummy)
-       : "0"  (multiplier64),
+       : "0"  (multiplier),
          "rm" (multiplicand),
          "c"  (shift)
        : "cc");
@@ -359,15 +374,76 @@ Muls64x32s64(int64 multiplicand, uint32 multiplier, uint32 shift)
 #elif defined(_MSC_VER)
 
 static INLINE int64
-Muls64x32s64(int64 multiplicand, uint32 multiplier, uint32 shift)
+Muls64x64s64(int64 multiplicand, int64 multiplier, uint32 shift)
 {
    int64 tmplo, tmphi;
+
    tmplo = _mul128(multiplicand, multiplier, &tmphi);
    return __shiftright128(tmplo, tmphi, (uint8) shift);
 }
 
 #endif
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Mul64x3264 --
+ *
+ *    Unsigned integer by fixed point multiplication:
+ *       result = multiplicand * multiplier >> shift
+ * 
+ *       Unsigned 64-bit integer multiplicand.
+ *       Unsigned 32-bit fixed point multiplier, represented as
+ *         multiplier >> shift, where shift < 64.
+ *       Unsigned 64-bit integer product.
+ *
+ * Implementation:
+ *    Multiply 64x64 bits to yield a full 128-bit product.
+ *    Shift result in RDX:RAX right by "shift".
+ *    Return the low-order 64 bits of the above.
+ *
+ * Result:
+ *    Return the low-order 64 bits of ((multiplicand * multiplier) >> shift)
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE uint64
+Mul64x3264(uint64 multiplicand, uint32 multiplier, uint32 shift)
+{
+   return Mul64x6464(multiplicand, multiplier, shift);
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Muls64x32s64 --
+ *
+ *    Signed integer by fixed point multiplication:
+ *       result = (multiplicand * multiplier) >> shift
+ * 
+ *       Signed 64-bit integer multiplicand.
+ *       Unsigned 32-bit fixed point multiplier, represented as
+ *         multiplier >> shift, where shift < 64.
+ *       Signed 64-bit integer product.
+ *
+ * Implementation:
+ *    Multiply 64x64 bits to yield a full 128-bit product.
+ *    Shift result in RDX:RAX right by "shift".
+ *    Return the low-order 64 bits of the above.
+ *
+ * Result:
+ *    Return the low-order 64 bits of ((multiplicand * multiplier) >> shift)
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE int64
+Muls64x32s64(int64 multiplicand, uint32 multiplier, uint32 shift)
+{
+   return Muls64x64s64(multiplicand, multiplier, shift);
+}
+
 
 #if defined(__GNUC__)
 
