@@ -82,7 +82,9 @@
 #include "includeCheck.h"
 
 /* STRICT ANSI means the Xserver build and X defines Bool differently. */
-#if !defined(__STRICT_ANSI__) || defined(__FreeBSD__) || defined(__MINGW32__)
+#if !defined(_XTYPEDEF_BOOL) && \
+    (!defined(__STRICT_ANSI__) || defined(__FreeBSD__) || defined(__MINGW32__))
+#define _XTYPEDEF_BOOL
 typedef char           Bool;
 #endif
 
@@ -248,6 +250,8 @@ typedef signed char        int8;
 #         include <stdlib.h>
 #         include <stdint.h>
 #      endif
+#   elif defined __ANDROID__
+#      include <stdint.h>
 #   else
 #      if !defined(__intptr_t_defined) && !defined(intptr_t)
 #         ifdef VM_I386
@@ -645,6 +649,19 @@ typedef void * UserVA;
 #endif
 
 
+/*
+ * At present, we effectively require a compiler that is at least
+ * gcc-3.3 (circa 2003).  Enforce this here, various things below
+ * this line depend upon it.
+ *
+ * In practice, most things presently compile with gcc-4.1 or gcc-4.4.
+ * The various linux kernel modules may use older (gcc-3.3) compilers.
+ */
+#if defined __GNUC__ && (__GNUC__ < 3 || (__GNUC__ == 3 && __GNUC_MINOR__ < 3))
+#error "gcc version is to old to compile assembly, need gcc-3.3 or better"
+#endif
+
+
 /*
  * Consider the following reasons functions are inlined:
  *
@@ -657,7 +674,7 @@ typedef void * UserVA;
  * are added the inline-ness should be removed.
  */
 
-#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 3)
+#if defined __GNUC__
 /*
  * Starting at version 3.3, gcc does not always inline functions marked
  * 'inline' (it depends on their size). To force gcc to do so, one must use the
@@ -685,23 +702,12 @@ typedef void * UserVA;
 
 #ifdef _MSC_VER
 #define NORETURN __declspec(noreturn)
-#elif __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 9)
+#elif defined __GNUC__
 #define NORETURN __attribute__((__noreturn__))
 #else
 #define NORETURN
 #endif
 
-/*
- * GCC 3.2 inline asm needs the + constraint for input/ouput memory operands.
- * Older GCCs don't know about it --hpreg
- */
-
-#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 2)
-#   define VM_ASM_PLUS 1
-#else
-#   define VM_ASM_PLUS 0
-#endif
-
 /*
  * Branch prediction hints:
  *     LIKELY(exp)   - Expression exp is likely TRUE.
@@ -715,7 +721,7 @@ typedef void * UserVA;
  * all others we don't so we do nothing.
  */
 
-#if (__GNUC__ >= 3)
+#if defined __GNUC__
 /*
  * gcc3 uses __builtin_expect() to inform the compiler of an expected value.
  * We use this to inform the static branch predictor. The '!!' in LIKELY
@@ -799,17 +805,6 @@ typedef void * UserVA;
 #define ALIGNED(n)
 #endif
 
-/*
- * __func__ is a stringified function name that is part of the C99 standard. The block
- * below defines __func__ on older systems where the compiler does not support that
- * macro.
- */
-#if defined(__GNUC__) \
-   && ((__GNUC__ == 2 && __GNUC_MINOR < 96) \
-       || (__GNUC__ < 2))
-#   define __func__ __FUNCTION__
-#endif
-
 /*
  * Once upon a time, this was used to silence compiler warnings that
  * get generated when the compiler thinks that a function returns
@@ -862,7 +857,7 @@ typedef void * UserVA;
 
 #   if !defined(FROBOS) && !defined(_SSIZE_T) && !defined(_SSIZE_T_) && \
        !defined(ssize_t) && !defined(__ssize_t_defined) && \
-       !defined(_SSIZE_T_DECLARED)
+       !defined(_SSIZE_T_DECLARED) && !defined(_SSIZE_T_DEFINED_)
 #      ifdef VM_I386
 #         define _SSIZE_T
 #         define __ssize_t_defined
@@ -876,7 +871,7 @@ typedef void * UserVA;
 #         define _SSIZE_T
 #         define __ssize_t_defined
 #         define _SSIZE_T_DECLARED
-          typedef int32 ssize_t;
+             typedef int32 ssize_t;
 #      endif
 #   endif
 
