@@ -121,6 +121,10 @@
 
 #define PRODUCT_VPX_NAME MAKE_NAME("VirtualCenter")
 
+#define PRODUCT_VPXA_NAME PRODUCT_VPX_NAME " Agent"
+
+#define PRODUCT_FDM_NAME MAKE_NAME("Fault Domain Manager")
+
 #define PRODUCT_WBC_NAME MAKE_NAME("WebCenter")
 
 #define PRODUCT_SDK_NAME MAKE_NAME("SDK")
@@ -136,6 +140,8 @@
 #define PRODUCT_VDM_CLIENT_NAME MAKE_NAME("View Client")
 #define PRODUCT_VDM_CLIENT_NAME_FOR_LICENSE PRODUCT_VDM_CLIENT_NAME
 
+#define PRODUCT_XVP_SHORT_NAME "XVP"
+#define PRODUCT_XVP_NAME MAKE_NAME("vCenter XVP Manager")
 #define PRODUCT_RMKSCONTAINER_NAME MAKE_NAME("Remote MKS Container")
 
 // XXX VMvisor is the underlying technology for possibly several products,
@@ -144,6 +150,12 @@
 // XXX Only one product for now so just hardcode it.
 #define PRODUCT_VMVISOR_NAME MAKE_NAME(PRODUCT_SCALABLE_SERVER_BRIEF_NAME "i")
 
+#if defined(__linux__) || defined(__FreeBSD__)
+#define PRODUCT_NETDUMP_NAME PRODUCT_GENERIC_NAME_LOWER "-netdumper"
+#else
+#define PRODUCT_NETDUMP_NAME PRODUCT_VMVISOR_NAME " dump collector"
+#endif
+
 /*
  * VMware Remote Console (VMRC) version definitions
  */
@@ -163,7 +175,7 @@
  * VMRC ActiveX CLSIDs and ProgIDs
  */
 #define PRODUCT_VMRC_PLUGIN_GUID_EMBEDDED      0567856D-8D8E-43ad-9241-7772F392B3A3
-#define PRODUCT_VMRC_PLUGIN_GUID_TYPELIB       A51B0CB2-1545-43AD-B66E-AD322009406C
+#define PRODUCT_VMRC_PLUGIN_GUID_TYPELIB       AFAD9CB7-273F-4f59-8C78-D8AF13845222
 #define PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED_BASE "VMware.VMwareRemoteConsole"
 #define PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED    PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED_BASE "." \
                                                XSTR(VMRC_PLUGIN_VERSION_BASE)
@@ -191,15 +203,16 @@
  *     the future.
  */
 #define PRODUCT_VMRC_PLUGIN_PREVIOUS_MIMETYPES \
-   "application/x-vmware-vmrc;version=2.5.0.199067" PRODUCT_VMRC_MIMETYPE_SEPARATOR \
-   "application/x-vmware-vmrc;version=2.5.0.158248" PRODUCT_VMRC_MIMETYPE_SEPARATOR \
-   "application/x-vmware-vmrc;version=2.5.0.116460" PRODUCT_VMRC_MIMETYPE_SEPARATOR \
-   "application/x-vmware-vmrc;version=2.5.0.0"
+   "application/x-vmware-vmrc;version=4.0.0.0"
 
 /*
  * legacy plugin mimetypes (currently unused but here for reference)
  */
 #define PRODUCT_VMRC_PLUGIN_LEGACY_MIMETYPES \
+   "application/x-vmware-vmrc;version=2.5.0.199067" PRODUCT_VMRC_MIMETYPE_SEPARATOR \
+   "application/x-vmware-vmrc;version=2.5.0.158248" PRODUCT_VMRC_MIMETYPE_SEPARATOR \
+   "application/x-vmware-vmrc;version=2.5.0.116460" PRODUCT_VMRC_MIMETYPE_SEPARATOR \
+   "application/x-vmware-vmrc;version=2.5.0.0" PRODUCT_VMRC_MIMETYPE_SEPARATOR \
    "application/x-vmware-mks;version=2.1.0.0" PRODUCT_VMRC_MIMETYPE_SEPARATOR \
    "application/x-vmware-mks;version=2.0.1.0" PRODUCT_VMRC_MIMETYPE_SEPARATOR \
    "application/x-vmware-mks;version=2.0.0.0" PRODUCT_VMRC_MIMETYPE_SEPARATOR
@@ -250,7 +263,8 @@
       || defined(VMX86_VLICENSE) \
       || defined(VMX86_P2V)      \
       || defined(VMX86_DDK)      \
-      || defined(VMX86_VDDK))
+      || defined(VMX86_VDDK)     \
+      || defined(VMX86_NETDUMP))
 #   if defined(_WIN32) || defined(__APPLE__)
       /*
        * XXX Make the product be Workstation by default if none of the defines
@@ -294,7 +308,15 @@
 #     define PRODUCT_SHORT_NAME PRODUCT_API_SCRIPTING_PERL_NAME
 #  endif
 #elif defined(VMX86_VPX)
-# define PRODUCT_SHORT_NAME PRODUCT_VPX_NAME
+#  if defined(CSI_FDM)
+#     define PRODUCT_SHORT_NAME PRODUCT_FDM_NAME
+#  elif defined(VPXA)
+#     define PRODUCT_SHORT_NAME PRODUCT_VPXA_NAME
+#  elif defined(XVP)
+#     define PRODUCT_SHORT_NAME PRODUCT_XVP_NAME
+#  else
+#     define PRODUCT_SHORT_NAME PRODUCT_VPX_NAME
+#  endif
 #elif defined(VMX86_WBC)
 # define PRODUCT_SHORT_NAME PRODUCT_WBC_NAME
 #elif defined(VMX86_SDK)
@@ -315,6 +337,8 @@
 # define PRODUCT_SHORT_NAME PRODUCT_DDK_NAME
 #elif defined(VMX86_VDDK)
 # define PRODUCT_SHORT_NAME PRODUCT_VDDK_NAME
+#elif defined(VMX86_NETDUMP)
+# define PRODUCT_SHORT_NAME PRODUCT_NETDUMP_NAME
 #endif
 
 
@@ -381,6 +405,9 @@
 #         define PRODUCT_NAME_FOR_LICENSE "VMware Workstation"
 #      endif
 #      define PRODUCT_SMP_NAME_FOR_LICENSE "" // None
+#   elif defined(VMX86_VPX)
+#      define PRODUCT_NAME_FOR_LICENSE PRODUCT_NAME " Server"
+#      define PRODUCT_SMP_NAME_FOR_LICENSE "" // None
 #   elif defined(VMX86_WGS_MIGRATION)
 #      define PRODUCT_NAME_FOR_LICENSE PRODUCT_NAME " for " PRODUCT_OS
 #      define PRODUCT_SMP_NAME_FOR_LICENSE "" // None
@@ -390,6 +417,9 @@
 #   elif defined(VMX86_SYSIMAGE)
 #      define PRODUCT_NAME_FOR_LICENSE PRODUCT_NAME
 #      define PRODUCT_SMP_NAME_FOR_LICENSE "" // None
+#   elif defined(VMX86_NETDUMP)
+#      define PRODUCT_NAME_FOR_LICENSE PRODUCT_NETDUMP_NAME
+#      define PRODUCT_SMP_NAME_FOR_LICENSE "" //None
 #   else   /* It is a product that doesn't use a license */
 #      define PRODUCT_NAME_FOR_LICENSE PRODUCT_NAME
 #      define PRODUCT_SMP_NAME_FOR_LICENSE "" // None
@@ -487,6 +517,9 @@
 #         define PRODUCT_NAME_FOR_LICENSE "VMware Workstation"
 #      endif
 #      define PRODUCT_SMP_NAME_FOR_LICENSE "" // None
+#   elif defined(VMX86_VPX)
+#      define PRODUCT_NAME_FOR_LICENSE PRODUCT_NAME " Server"
+#      define PRODUCT_SMP_NAME_FOR_LICENSE "" // None
 #   elif defined(VMX86_WGS_MIGRATION)
 #      define PRODUCT_NAME_FOR_LICENSE PRODUCT_NAME " for Win32"
 #      define PRODUCT_SMP_NAME_FOR_LICENSE "" // None
@@ -529,6 +562,8 @@
 #  define PRODUCT_NAME_PLATFORM         PRODUCT_NAME " for Solaris"
 #elif defined(__APPLE__)
 #  define PRODUCT_NAME_PLATFORM         PRODUCT_NAME " for Mac OS X"
+#elif defined __ANDROID__
+#  define PRODUCT_NAME_PLATFORM         PRODUCT_NAME " for Android"
 #else
 #  ifdef VMX86_TOOLS
 #    error "Define a product string for this platform."
@@ -561,4 +596,15 @@
  */
 #define HOST_AGENT_SERVICE_NAME     "VMwareHostd"
 
+// Name of the environment variable that controls which proxy server to use
+// while opening connections to vCenter and vSphere servers. Currently vCloud
+// VMRC uses it.
+#define VMWARE_HTTPSPROXY  "VMWARE_HTTPSPROXY"
+
+// Name of the environment variable that controls whether MKS traffic should
+// be tunnelled over HTTPS through the console proxy. Currently vCloud VMRC
+// uses it.
+#define VMWARE_HTTPSTUNNEL_MKS   "VMWARE_HTTPSTUNNEL_MKS"
+
+
 #endif /* VM_PRODUCT_H */
