@@ -1011,9 +1011,30 @@ typedef uint32 ToolsVersion;
 #define   TOOLS_VERSION_HOSTED10_UPDATE2_V_MNR  4
 #define   TOOLS_VERSION_HOSTED10_UPDATE2_V_BASE 4
 
-#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_HOSTED10_UPDATE2
-#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_HOSTED10_UPDATE2)
-#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_HOSTED10_UPDATE2)
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_ESX50_BETA1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_ESX50_BETA1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_ESX50_BETA1_V_MJR  8
+#define   TOOLS_VERSION_ESX50_BETA1_V_MNR  5
+#define   TOOLS_VERSION_ESX50_BETA1_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_ESX50_RC1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_ESX50_RC1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_ESX50_RC1_V_MJR  8
+#define   TOOLS_VERSION_ESX50_RC1_V_MNR  5
+#define   TOOLS_VERSION_ESX50_RC1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_NEXT TOOLS_VERSION_TO_UINT(TOOLS_VERSION_NEXT_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_NEXT_V_MJR  8
+#define   TOOLS_VERSION_NEXT_V_MNR  7
+#define   TOOLS_VERSION_NEXT_V_BASE 0
+
+#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_NEXT
+#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_NEXT)
+#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_NEXT)
 
 /*
  * The extended Tools version is the current Tools version with the
