@@ -69,7 +69,7 @@ static Atomic_Ptr hashTableMem;
  *-----------------------------------------------------------------------------
  */
 
-MXUserPerThread *
+static MXUserPerThread *
 MXUserGetPerThread(void *tid,      // IN: native thread ID
                    Bool mayAlloc)  // IN: alloc perThread if not present?
 {
@@ -109,6 +109,43 @@ MXUserGetPerThread(void *tid,      // IN: native thread ID
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUserListLocks
+ *
+ *      Allow a caller to list, via warnings, the list of locks the caller
+ *      has acquired. Ensure that no memory for lock tracking is allocated
+ *      if no locks have been taken.
+ *
+ * Results:
+ *      The list is printed
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+MXUserListLocks(void)
+{
+   MXUserPerThread *perThread = MXUserGetPerThread(MXUserGetNativeTID(),
+                                                   FALSE);
+
+   if (perThread != NULL) {
+      uint32 i;
+
+      for (i = 0; i < perThread->locksHeld; i++) {
+         MXUserHeader *hdr = perThread->lockArray[i];
+
+         Warning("\tMXUser lock %s (@%p) rank 0x%x\n", hdr->name, hdr,
+                 hdr->rank);
+      }
+   }
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -136,6 +173,90 @@ MXUser_IsCurThreadHoldingLocks(void)
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUserThreadRank --
+ *
+ *      Return the highest rank held by the specified thread via MXUser locks.
+ *
+ * Results:
+ *      As above
+ *
+ * Side effects:
+ *      Can optionally determine if a lock has been locked before.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+MX_Rank
+MXUserThreadRank(MXUserPerThread *perThread,  // IN:
+                 MXUserHeader *header,        // IN:
+                 Bool *firstUse)              // OUT:
+{
+   uint32 i;
+   Bool foundOnce = TRUE;
+   MX_Rank maxRank = RANK_UNRANKED;
+
+   ASSERT(perThread);
+
+   /*
+    * Determine the maximum rank held. Note if the lock being acquired
+    * was previously entered into the tracking system.
+    */
+
+   for (i = 0; i < perThread->locksHeld; i++) {
+      MXUserHeader *chkHdr = perThread->lockArray[i];
+
+      maxRank = MAX(chkHdr->rank, maxRank);
+
+      if (chkHdr == header) {
+         foundOnce = FALSE;
+      }
+   }
+
+   if (firstUse) {
+      *firstUse = foundOnce;
+   }
+
+   return maxRank;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUserCurrentRank --
+ *
+ *      Return the highest rank held by the current thread via MXUser locks.
+ *
+ * Results:
+ *      As above
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+MX_Rank
+MXUserCurrentRank(void)
+{
+   MX_Rank maxRank;
+   MXUserPerThread *perThread; 
+
+   perThread = MXUserGetPerThread(MXUserGetNativeTID(), FALSE);
+
+   if (perThread == NULL) {
+      maxRank = RANK_UNRANKED;
+   } else {
+      maxRank = MXUserThreadRank(perThread, NULL, NULL);
+   }
+
+   return maxRank;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -161,32 +282,20 @@ MXUserAcquisitionTracking(MXUserHeader *header,  // IN:
 
    ASSERT_NOT_IMPLEMENTED(perThread->locksHeld < MXUSER_MAX_LOCKS_PER_THREAD);
 
-#if defined(DISABLE_MXUSER_LOCK_RANKS)
-   checkRank = FALSE;
-#endif
-
    /* Rank checking anyone? */
    if (checkRank && (header->rank != RANK_UNRANKED)) {
-      uint32 i;
       MX_Rank maxRank;
       Bool firstInstance = TRUE;
 
-      /* Check MX locks when they are present */
-      maxRank = (MXUserMxCheckRank) ? (*MXUserMxCheckRank)() : RANK_UNRANKED;
-
       /*
-       * Determine the maximum rank held. Note if the lock being acquired
-       * was previously entered into the tracking system.
+       * Determine the highest rank held by the calling thread. Check for
+       * MX locks if they are present.
        */
 
-      for (i = 0; i < perThread->locksHeld; i++) {
-         MXUserHeader *chkHdr = perThread->lockArray[i];
+      maxRank = MXUserThreadRank(perThread, header, &firstInstance);
 
-         maxRank = MAX(chkHdr->rank, maxRank);
-
-         if (chkHdr == header) {
-            firstInstance = FALSE;
-         }
+      if (MXUserMxCheckRank) {
+         maxRank = MAX(maxRank, (*MXUserMxCheckRank)());
       }
 
       /*
@@ -500,42 +609,3 @@ MXUserInstallMxHooks(void (*theLockListFunc)(void),
             );
    }
 }
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUserListLocks
- *
- *      Allow a caller to list, via warnings, the list of locks the caller
- *      has acquired. Ensure that no memory for lock tracking is allocated
- *      if no locks have been taken.
- *
- * Results:
- *      The list is printed
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-MXUserListLocks(void)
-{
-#if defined(MXUSER_DEBUG)
-   MXUserPerThread *perThread = MXUserGetPerThread(MXUserGetNativeTID(),
-                                                   FALSE);
-
-   if (perThread != NULL) {
-      uint32 i;
-
-      for (i = 0; i < perThread->locksHeld; i++) {
-         MXUserHeader *hdr = perThread->lockArray[i];
-
-         Warning("\tMXUser lock %s (@%p) rank 0x%x\n", hdr->name, hdr,
-                 hdr->rank);
-      }
-   }
-#endif
-}
