@@ -30,31 +30,40 @@
 #include "hostinfo.h"
 #include "log.h"
 #include "logFixed.h"
-#if defined(MXUSER_STATS)
-#include "statsLog.h"
-#endif
 
 #define BINS_PER_DECADE 100
 
 static double mxUserContentionRatio = 0.0;  // always "off"
 static uint64 mxUserContentionCount = 0;    // always "off"
 
-#if defined(MXUSER_STATS)
 static Atomic_Ptr mxLockMemPtr;   // internal singleton lock
 static ListItem *mxUserLockList;  // list of all MXUser locks
-#endif
+
+typedef struct {
+   void   *address;
+   uint64  timeValue;
+} TopOwner;
+
+#define TOPOWNERS 10
 
 struct MXUserHisto {
-   char    *typeName;      // Type (name) of histogram
-   uint64  *binData;       // hash table bins
-   uint64   totalSamples;  // Population sample size
-   uint64   minValue;      // min value allowed
-   uint64   maxValue;      // max value allowed
-   uint32   numBins;       // number of histogram bins
+   char     *typeName;               // Type (name) of histogram
+   uint64   *binData;                // Hash table bins
+   uint64    totalSamples;           // Population sample size
+   uint64    minValue;               // Min value allowed
+   uint64    maxValue;               // Max value allowed
+   uint32    numBins;                // Number of histogram bins
+   TopOwner  ownerArray[TOPOWNERS];  // List of top owners
 };
 
+static char   *mxUserHistoLine = NULL;
+static uint32  mxUserMaxLineLength = 0;
+static void   *mxUserStatsContext = NULL;
+static void  (*mxUserStatsFunc)(void *context,
+                               const char *fmt,
+                               va_list ap) = NULL;
+
 
-#if defined(MXUSER_STATS)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -78,7 +87,7 @@ MXUserAddToList(MXUserHeader *header)  // IN:
 
    /* Tolerate a failure. This is too low down to log */
    if (listLock) {
-      MXRecLockAcquire(listLock, GetReturnAddress());
+      MXRecLockAcquire(listLock);
       LIST_QUEUE(&header->item, &mxUserLockList);
       MXRecLockRelease(listLock);
    }
@@ -108,12 +117,11 @@ MXUserRemoveFromList(MXUserHeader *header)  // IN:
 
    /* Tolerate a failure. This is too low down to log */
    if (listLock) {
-      MXRecLockAcquire(listLock, GetReturnAddress());
+      MXRecLockAcquire(listLock);
       LIST_DEL(&header->item, &mxUserLockList);
       MXRecLockRelease(listLock);
    }
 }
-#endif
 
 
 /*
@@ -128,7 +136,8 @@ MXUserRemoveFromList(MXUserHeader *header)  // IN:
  *      (uint32) (BINS_PER_DECADE * log10(value))
  *
  * Side effects:
- *      None
+ *      The computed value may actually be larger than expected by a tiny
+ *      amount - the log10 method is a ratio of two integers.
  *
  *-----------------------------------------------------------------------------
  */
@@ -185,7 +194,7 @@ MXUserHistoSetUp(char *typeName,   // type (name) of histogram
    ASSERT(decades > 0);
    ASSERT((minValue != 0) && ((minValue == 1) || ((minValue % 10) == 0)));
 
-   histo = Util_SafeMalloc(sizeof(*histo));
+   histo = Util_SafeCalloc(sizeof(*histo), 1);
 
    histo->typeName = Util_SafeStrdup(typeName);
    histo->numBins = BINS_PER_DECADE * decades;
@@ -251,8 +260,10 @@ MXUserHistoTearDown(MXUserHisto *histo)  // IN:
 
 void
 MXUserHistoSample(MXUserHisto *histo,  // IN/OUT:
-                  uint64 durationNS)   // IN:
+                  uint64 durationNS,   // IN:
+                  void *ownerRetAddr)  // IN:
 {
+   uint32 i;
    uint32 index;
 
    ASSERT(histo);
@@ -262,16 +273,65 @@ MXUserHistoSample(MXUserHisto *histo,  // IN/OUT:
    if (durationNS < histo->minValue) {
       index = 0;
    } else {
-      if (durationNS > histo->maxValue) {
+      index = MXUserHistoIndex(durationNS / histo->minValue);
+
+      if (index > histo->numBins - 1) {
          index = histo->numBins - 1;
-      } else {
-         index = MXUserHistoIndex(durationNS / histo->minValue);
       }
    }
 
    ASSERT(index < histo->numBins);
 
    histo->binData[index]++;
+
+   index = 0;
+
+   for (i = 0; i < TOPOWNERS; i++) {
+      if (histo->ownerArray[i].address == ownerRetAddr) {
+         index = i;
+         break;
+      }
+
+      if (histo->ownerArray[i].timeValue <
+          histo->ownerArray[index].timeValue) {
+         index = i;
+      }
+   }
+
+   if (durationNS > histo->ownerArray[index].timeValue) {
+      histo->ownerArray[index].address = ownerRetAddr;
+      histo->ownerArray[index].timeValue = durationNS;
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUserStatsLog --
+ *
+ *      Output the statistics data
+ *
+ * Results:
+ *      As above
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE void
+MXUserStatsLog(const char *fmt,  // IN:
+               ...)              // IN:
+{
+   va_list ap;
+
+   ASSERT(mxUserStatsFunc);
+
+   va_start(ap, fmt);
+   (*mxUserStatsFunc)(mxUserStatsContext, fmt, ap);
+   va_end(ap);
 }
 
 
@@ -298,30 +358,16 @@ MXUserHistoDump(MXUserHisto *histo,    // IN:
    ASSERT(header);
    ASSERT(histo);
 
-#if defined(MXUSER_STATS)
    if (histo->totalSamples) {
       char *p;
       uint32 i;
       uint32 spaceLeft;
 
-      static uint32 maxLine = 0;
-      static char *histoLine = NULL;
+      ASSERT(mxUserHistoLine);
 
-      /*
-       * Statistics are reported from a single thread. This avoids allocating
-       * a potentially large buffer on the stack.
-       */ 
-
-      if (maxLine == 0) {
-         maxLine = Log_MaxLineLength();  // includes terminating NUL
-         ASSERT(maxLine >= 1024);        // assert a rational minimum
-
-         histoLine = Util_SafeMalloc(maxLine);
-      }
-
-      i = Str_Sprintf(histoLine, maxLine,
+      i = Str_Sprintf(mxUserHistoLine, mxUserMaxLineLength,
                       "MXUser: h l=%u t=%s min=%"FMT64"u max=%"FMT64"u\n",
-                      header->identifier, histo->typeName, histo->minValue,
+                      header->serialNumber, histo->typeName, histo->minValue,
                       histo->maxValue);
 
       /*
@@ -330,8 +376,8 @@ MXUserHistoDump(MXUserHisto *histo,    // IN:
        * properly terminated no matter what happens.
        */
 
-      p = &histoLine[i - 1];
-      spaceLeft = maxLine - i - 2;
+      p = &mxUserHistoLine[i - 1];
+      spaceLeft = mxUserMaxLineLength - i - 2;
 
       /* Add as many histogram bins as possible within the line limitations */
       for (i = 0; i < histo->numBins; i++) {
@@ -339,10 +385,8 @@ MXUserHistoDump(MXUserHisto *histo,    // IN:
             uint32 len;
             char binEntry[32];
 
-            Str_Sprintf(binEntry, sizeof binEntry, " %u-%"FMT64"u\n",
-                        i, histo->binData[i]);
-
-            len = strlen(binEntry);
+            len = Str_Sprintf(binEntry, sizeof binEntry, " %u-%"FMT64"u\n",
+                              i, histo->binData[i]);
 
             if (len < spaceLeft) {
                /*
@@ -362,9 +406,45 @@ MXUserHistoDump(MXUserHisto *histo,    // IN:
          }
       }
 
-      StatsLog(histoLine);
+      MXUserStatsLog("%s", mxUserHistoLine);
+
+      i = Str_Sprintf(mxUserHistoLine, mxUserMaxLineLength,
+                      "MXUser: ht l=%u t=%s\n", header->serialNumber,
+                      histo->typeName);
+
+      p = &mxUserHistoLine[i - 1];
+      spaceLeft = mxUserMaxLineLength - i - 2;
+
+      for (i = 0; i < TOPOWNERS; i++) {
+         if (histo->ownerArray[i].address != NULL) {
+            uint32 len;
+            char binEntry[32];
+
+            /* Use a debugger to change the address to a symbol */
+            len = Str_Sprintf(binEntry, sizeof binEntry, " %p-%"FMT64"u\n",
+                              histo->ownerArray[i].address,
+                              histo->ownerArray[i].timeValue);
+
+            if (len < spaceLeft) {
+               /*
+                * Append the address, time value pair to the end of the
+                * string. This includes the terminating "\n\0". Update the
+                * pointer to the next free place to point to the '\n'. If
+                * another entry is made, things work out properly. If not
+                * the string is properly terminated as a line.
+                */
+
+               Str_Strcpy(p, binEntry, len + 1);
+               p += len - 1;
+               spaceLeft -= len;
+            } else {
+               break;
+            }
+         }
+      }
+
+      MXUserStatsLog("%s", mxUserHistoLine);
    }
-#endif
 }
 
 
@@ -452,11 +532,24 @@ MXUserBasicStatsSetUp(MXUserBasicStats *stats,  // IN/OUT:
  *-----------------------------------------------------------------------------
  */
 
+static double
+MXUserSqrt(double x)  // IN: hack until next round when FP goes away
+{
+   double xn;
+   double xn1 = x;
+
+   do {
+      xn = xn1;
+      xn1 = (xn + x/xn) / 2.0;
+   } while (fabs(xn1 - xn) > 1E-10);
+
+   return xn1;
+}
+
 void
 MXUserDumpBasicStats(MXUserBasicStats *stats,  // IN:
                      MXUserHeader *header)     // IN:
 {
-#if defined(MXUSER_STATS)
    uint64 stdDev;
 
    if (stats->numSamples < 2) {
@@ -481,15 +574,14 @@ MXUserDumpBasicStats(MXUserBasicStats *stats,  // IN:
       mean = ((double) stats->timeSum) / num;
       variance = (stats->timeSquaredSum - (num*mean*mean)) / (num - 1.0);
 
-      stdDev = (variance < 0.0) ? 0 : (uint64) (sqrt(variance) + 0.5);
+      stdDev = (variance < 0.0) ? 0 : (uint64) (MXUserSqrt(variance) + 0.5);
    }
 
-   StatsLog("MXUser: e l=%u t=%s c=%"FMT64"u min=%"FMT64"u "
-            "max=%"FMT64"u mean=%"FMT64"u sd=%"FMT64"u\n",
-            header->identifier, stats->typeName,
-            stats->numSamples, stats->minTime, stats->maxTime,
-            stats->timeSum/stats->numSamples, stdDev);
-#endif
+   MXUserStatsLog("MXUser: e l=%u t=%s c=%"FMT64"u min=%"FMT64"u "
+                  "max=%"FMT64"u mean=%"FMT64"u sd=%"FMT64"u\n",
+                  header->serialNumber, stats->typeName,
+                  stats->numSamples, stats->minTime, stats->maxTime,
+                  stats->timeSum/stats->numSamples, stdDev);
 }
 
 
@@ -610,22 +702,20 @@ void
 MXUserDumpAcquisitionStats(MXUserAcquisitionStats *stats,  // IN:
                            MXUserHeader *header)           // IN:
 {
-#if defined(MXUSER_STATS)
    if (stats->numAttempts > 0) {
       if (stats->numSuccesses > 0) {
          MXUserDumpBasicStats(&stats->basicStats, header);
       }
 
-      StatsLog("MXUser: ce l=%u a=%"FMT64"u s=%"FMT64"u sc=%"FMT64"u "
-               "sct=%"FMT64"u t=%"FMT64"u\n",
-               header->identifier,
-               stats->numAttempts,
-               stats->numSuccesses,
-               stats->numSuccessesContended,
-               stats->successContentionTime,
-               stats->totalContentionTime);
+      MXUserStatsLog("MXUser: ce l=%u a=%"FMT64"u s=%"FMT64"u sc=%"FMT64"u "
+                     "sct=%"FMT64"u t=%"FMT64"u\n",
+                     header->serialNumber,
+                     stats->numAttempts,
+                     stats->numSuccesses,
+                     stats->numSuccessesContended,
+                     stats->successContentionTime,
+                     stats->totalContentionTime);
    }
-#endif
 }
 
 
@@ -809,7 +899,68 @@ MXUserForceHisto(Atomic_Ptr *histoPtr,  // IN/OUT:
 }
 
 
-#if defined(MXUSER_STATS)
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUserStatsEnabled --
+ *
+ *      Are statistics keeping enabled
+ *
+ * Results:
+ *      TRUE   Yes
+ *      FALSE  NO
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+MXUserStatsEnabled(void)
+{
+   return (mxUserStatsFunc != NULL) && (mxUserMaxLineLength > 0);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_SetStatsFunc --
+ *
+ *      Establish statistics taking and reporting. This is done by registering
+ *      a statistics context, a reporting function and a maximum line length.
+ *
+ *      A maxLineLength of zero (0) and/or a statsFunc of NULL will
+ *      disable/prevent statistics gathering.
+ *
+ * Results:
+ *      As above
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+MXUser_SetStatsFunc(void *context,                    // IN:
+                    uint32 maxLineLength,             // IN:
+                    void (*statsFunc)(void *context,  // IN:
+                                      const char *fmt,
+                                      va_list ap))
+{
+   ASSERT(maxLineLength >= 1024);   // assert a rational minimum
+
+   free(mxUserHistoLine);
+   mxUserHistoLine = Util_SafeMalloc(maxLineLength);
+
+   mxUserStatsContext = context;
+   mxUserMaxLineLength = maxLineLength;
+   mxUserStatsFunc = statsFunc;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -839,23 +990,27 @@ MXUser_PerLockData(void)
 {
    MXRecLock *listLock = MXUserInternalSingleton(&mxLockMemPtr);
 
-   if (listLock && MXRecLockTryAcquire(listLock, GetReturnAddress())) {
-      ListItem *entry;
-      uint32 highestID;
-      static uint32 lastReportedID = 0;
+   if (mxUserStatsFunc == NULL) {
+      return;
+   }
 
-      highestID = lastReportedID;
+   if (listLock && MXRecLockTryAcquire(listLock)) {
+      ListItem *entry;
+      uint32 highestSerialNumber;
+      static uint32 lastReportedSerialNumber = 0;
+
+      highestSerialNumber = lastReportedSerialNumber;
 
       LIST_SCAN(entry, mxUserLockList) {
          MXUserHeader *header = LIST_CONTAINER(entry, MXUserHeader, item);
 
          /* Log the ID information for a lock that did exist previously */
-         if (header->identifier > lastReportedID) {
-            StatsLog("MXUser: n n=%s l=%d r=0x%x\n", header->name,
-                     header->identifier, header->rank);
+         if (header->serialNumber > lastReportedSerialNumber) {
+            MXUserStatsLog("MXUser: n n=%s l=%d r=0x%x\n", header->name,
+                           header->serialNumber, header->rank);
 
-            if (header->identifier > highestID) {
-               highestID = header->identifier;
+            if (header->serialNumber > highestSerialNumber) {
+               highestSerialNumber = header->serialNumber;
             }
          }
 
@@ -868,22 +1023,21 @@ MXUser_PerLockData(void)
          }
       }
 
-      lastReportedID = highestID;
+      lastReportedSerialNumber = highestSerialNumber;
 
       MXRecLockRelease(listLock);
    }
 }
-#endif
 
 
 /*
  *-----------------------------------------------------------------------------
  *
- * MXUserAllocID --
+ * MXUserAllocSerialNumber --
  *
- *      Allocate and return an MXUser identifier
+ *      Allocate and return an MXUser serial number.
  *
- *      MXUser identifiers are never recycled.
+ *      MXUser serial numbers are never recycled.
  *
  * Results:
  *      As above.
@@ -895,9 +1049,9 @@ MXUser_PerLockData(void)
  */
 
 uint32
-MXUserAllocID(void)
+MXUserAllocSerialNumber(void)
 {
-   static Atomic_uint32 firstFreeID = { 1 };  // must start not zero
+   static Atomic_uint32 firstFreeSerialNumber = { 1 };  // must start not zero
 
-   return Atomic_FetchAndInc(&firstFreeID);
+   return Atomic_FetchAndInc(&firstFreeSerialNumber);
 }
