@@ -65,7 +65,9 @@
 #include <sys/vfs.h>
 #endif
 #if !defined(sun) && (!defined(USING_AUTOCONF) || (defined(HAVE_SYS_IO_H) && defined(HAVE_SYS_SYSINFO_H)))
-#include <sys/io.h>
+# ifndef __ANDROID__
+# include <sys/io.h>
+# endif
 #include <sys/sysinfo.h>
 #ifndef HAVE_SYSINFO
 #define HAVE_SYSINFO 1
@@ -140,7 +142,7 @@ typedef struct lsb_distro_info {
 } LSBDistroInfo;
 
 
-static LSBDistroInfo lsbFields[] = {
+static const LSBDistroInfo lsbFields[] = {
    {"DISTRIB_ID=",          "DISTRIB_ID=%s"},
    {"DISTRIB_RELEASE=",     "DISTRIB_RELEASE=%s"},
    {"DISTRIB_CODENAME=",    "DISTRIB_CODENAME=%s"},
@@ -155,7 +157,7 @@ typedef struct distro_info {
 } DistroInfo;
 
 
-static DistroInfo distroArray[] = {
+static const DistroInfo distroArray[] = {
    {"RedHat",             "/etc/redhat-release"},
    {"RedHat",             "/etc/redhat_version"},
    {"Sun",                "/etc/sun-release"},
@@ -499,6 +501,8 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
       } else {
          Str_Strcpy(distroShort, STR_OS_RED_HAT, distroShortSize);
       }
+   } else if (strstr(distroLower, "opensuse")) {
+      Str_Strcpy(distroShort, STR_OS_OPENSUSE, distroShortSize);
    } else if (strstr(distroLower, "suse")) {
       if (strstr(distroLower, "enterprise")) {
          if (strstr(distroLower, "server 11") ||
@@ -508,7 +512,7 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
                     strstr(distroLower, "desktop 10")) {
             Str_Strcpy(distroShort, STR_OS_SLES_10, distroShortSize);
          } else {
-            Str_Strcpy(distroShort, STR_OS_SUSE_EN, distroShortSize);
+            Str_Strcpy(distroShort, STR_OS_SLES, distroShortSize);
          }
       } else if (strstr(distroLower, "sun")) {
          Str_Strcpy(distroShort, STR_OS_SUN_DESK, distroShortSize);
@@ -1928,6 +1932,47 @@ Hostinfo_NestingSupported(void)
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ *  Hostinfo_VCPUInfoBackdoor --
+ *
+ *      Access the backdoor with an VCPU info query. This is used to
+ *      determine whether a VCPU supports a particular feature,
+ *      determined by 'bit'.  This function should only be called after
+ *      determining that the backdoor is present with
+ *      Hostinfo_TouchBackdoor().
+ *
+ * Results:
+ *      TRUE if the outer VM supports the feature.
+ *	FALSE otherwise.
+ *
+ * Side effects:
+ *	Exception if not in a VM, so don't do that!
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+Hostinfo_VCPUInfoBackdoor(unsigned bit)
+{
+#if defined(__i386__) || defined(__x86_64__)
+   uint32 result;
+   __asm__ __volatile__(
+      "inl %%dx, %%eax"
+      : "=a" (result)
+      :	"0"  (BDOOR_MAGIC),
+        "c"  (BDOOR_CMD_GET_VCPU_INFO),
+        "d"  (BDOOR_PORT)
+   );
+   /* If reserved bit is 1, this command wasn't implemented. */
+   return (result & (1 << BDOOR_CMD_VCPU_RESERVED)) == 0 &&
+          (result & (1 << bit))                     != 0;
+#endif
+   return FALSE;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
