@@ -34,7 +34,6 @@
 #endif // Win32 vs Posix
 
 #include "vmware.h"
-#include "productState.h"
 #include "vm_version.h"
 #include "log.h"
 #include "panic.h"
@@ -554,9 +553,10 @@ Panic_Panic(const char *format,
 
    Panic_PostPanicMsg(buf);
 
-   /* 
+   /*
     * Bye
     */
+   Log("Exiting\n");
 
    exit(-1);
    NOT_REACHED();
