@@ -24,9 +24,6 @@
 
 #ifdef __linux__
 #  include "driver-config.h"
-
-#  define EXPORT_SYMTAB
-
 #  include <linux/module.h>
 #  include "compat_kernel.h"
 #  include "compat_pci.h"
@@ -40,16 +37,17 @@
 #  error "Platform not support by VMCI datagram API."
 #endif // linux
 
+#define LGPFX "VMCIDatagram: "
+
 #include "vmci_kernel_if.h"
 #include "vm_basic_types.h"
 #include "vm_assert.h"
 #include "vmci_defs.h"
 #include "vmci_infrastructure.h"
-#include "vmciInt.h"
-#include "vmciUtil.h"
 #include "vmciDatagram.h"
-#include "vmciCommonInt.h"
+#include "vmciInt.h"
 #include "vmciKernelAPI.h"
+#include "vmciUtil.h"
 
 typedef struct DatagramHashEntry {
    struct DatagramHashEntry  *next;
@@ -92,7 +90,6 @@ static DatagramHashEntry *DatagramHashGetEntry(VMCIHandle handle);
 static DatagramHashEntry *DatagramHashGetEntryAnyCid(VMCIHandle handle);
 static void DatagramHashReleaseEntry(DatagramHashEntry *entry);
 static Bool DatagramHandleUniqueLockedAnyCid(VMCIHandle handle);
-static int DatagramProcessNotify(void *clientData, VMCIDatagram *msg);
 
 DatagramHashTable hashTable;
 
@@ -147,6 +144,13 @@ DatagramHashAddEntry(DatagramHashEntry *entry, // IN:
    ASSERT(entry);
 
    VMCI_GrabLock_BH(&hashTable.lock, &flags);
+
+   /* Do not allow addition of a new handle if the device is being shutdown. */
+   if (VMCI_DeviceShutdown()) {
+      VMCI_ReleaseLock_BH(&hashTable.lock, flags);
+      return VMCI_ERROR_DEVICE_NOT_FOUND;
+   }
+
    if (!VMCI_HANDLE_INVALID(entry->handle) &&
        !DatagramHandleUniqueLockedAnyCid(entry->handle)) {
       VMCI_ReleaseLock_BH(&hashTable.lock, flags);
@@ -396,7 +400,7 @@ DatagramHandleUniqueLockedAnyCid(VMCIHandle handle) // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIDatagramCreateHndInt --
+ * VMCIDatagram_CreateHnd --
  *
  *      Creates a datagram endpoint and returns a handle to it.
  *
@@ -409,12 +413,13 @@ DatagramHandleUniqueLockedAnyCid(VMCIHandle handle) // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIDatagram_CreateHnd)
 int
-VMCIDatagramCreateHndInt(VMCIId resourceID,          // IN:
-                         uint32 flags,               // IN:
-                         VMCIDatagramRecvCB recvCB,  // IN:
-                         void *clientData,           // IN:
-                         VMCIHandle *outHandle)      // OUT:
+VMCIDatagram_CreateHnd(VMCIId resourceID,          // IN
+                       uint32 flags,               // IN
+                       VMCIDatagramRecvCB recvCB,  // IN
+                       void *clientData,           // IN
+                       VMCIHandle *outHandle)      // OUT
 {
    int result;
    DatagramHashEntry *entry;
@@ -435,7 +440,6 @@ VMCIDatagramCreateHndInt(VMCIId resourceID,          // IN:
       }
    }
 
-
    if ((flags & VMCI_FLAG_WELLKNOWN_DG_HND) != 0) {
       VMCIDatagramWellKnownMapMsg wkMsg;
       if (resourceID == VMCI_INVALID_ID) {
@@ -448,8 +452,8 @@ VMCIDatagramCreateHndInt(VMCIId resourceID,          // IN:
       wkMsg.wellKnownID = resourceID;
       result = VMCI_SendDatagram((VMCIDatagram *)&wkMsg);
       if (result < VMCI_SUCCESS) {
-         VMCI_LOG(("Failed to reserve wellknown id %d, error %d.\n",
-                   resourceID, result));
+         VMCI_DEBUG_LOG(4, (LGPFX"Failed to reserve wellknown id %d, "
+                            "error %d.\n", resourceID, result));
          return result;
       }
 
@@ -487,7 +491,7 @@ VMCIDatagramCreateHndInt(VMCIId resourceID,          // IN:
 
    result = DatagramHashAddEntry(entry, contextID);
    if (result != VMCI_SUCCESS) {
-      VMCI_LOG(("Failed to add new entry, err 0x%x.\n", result));
+      VMCI_DEBUG_LOG(4, (LGPFX"Failed to add new entry, err 0x%x.\n", result));
       VMCI_DestroyEvent(&entry->destroyEvent);
       VMCI_FreeKernelMem(entry, sizeof *entry);
       return result;
@@ -499,37 +503,6 @@ VMCIDatagramCreateHndInt(VMCIId resourceID,          // IN:
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIDatagram_CreateHnd --
- *
- *      Creates a datagram endpoint and returns a handle to it.
- *
- * Results:
- *      Returns handle if success, negative errno value otherwise.
- *
- * Side effects:
- *      Datagram endpoint is created both in guest and on host.
- *
- *-----------------------------------------------------------------------------
- */
-
-#ifdef __linux__
-EXPORT_SYMBOL(VMCIDatagram_CreateHnd);
-#endif
-
-int
-VMCIDatagram_CreateHnd(VMCIId resourceID,          // IN:
-                       uint32 flags,               // IN:
-                       VMCIDatagramRecvCB recvCB,  // IN:
-                       void *clientData,           // IN:
-                       VMCIHandle *outHandle)      // OUT:
-{
-   return VMCIDatagramCreateHndInt(resourceID, flags, recvCB, clientData, outHandle);
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -548,10 +521,7 @@ VMCIDatagram_CreateHnd(VMCIId resourceID,          // IN:
  *-----------------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCIDatagram_CreateHndPriv);
-#endif
-
+VMCI_EXPORT_SYMBOL(VMCIDatagram_CreateHndPriv)
 int
 VMCIDatagram_CreateHndPriv(VMCIId resourceID,            // IN:
                            uint32 flags,                 // IN:
@@ -567,37 +537,7 @@ VMCIDatagram_CreateHndPriv(VMCIId resourceID,            // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIDatagramCreateHndPriv --
- *
- *      API provided for compatibility with the host vmci API. This function
- *      doesn't ever succeed since you can't ask for elevated privileges from
- *      the guest. Use VMCIDatagramCreateHndInt instead.
- *
- * Results:
- *      Returns VMCI_ERROR_NO_ACCESS.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-VMCIDatagramCreateHndPriv(VMCIId resourceID,            // IN:
-                          uint32 flags,                 // IN:
-                          VMCIPrivilegeFlags privFlags, // IN:
-                          VMCIDatagramRecvCB recvCB,    // IN:
-                          void *clientData,             // IN:
-                          VMCIHandle *outHandle)        // OUT:
-{
-   return VMCI_ERROR_NO_ACCESS;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIDatagramDestroyHndInt --
+ * VMCIDatagram_DestroyHnd --
  *
  *      Destroys a handle.
  *
@@ -610,8 +550,9 @@ VMCIDatagramCreateHndPriv(VMCIId resourceID,            // IN:
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIDatagram_DestroyHnd)
 int
-VMCIDatagramDestroyHndInt(VMCIHandle handle)       // IN
+VMCIDatagram_DestroyHnd(VMCIHandle handle)       // IN
 {
    DatagramHashEntry *entry = DatagramHashGetEntry(handle);
    if (entry == NULL) {
@@ -638,8 +579,8 @@ VMCIDatagramDestroyHndInt(VMCIHandle handle)       // IN
       wkMsg.wellKnownID = entry->handle.resource;
       result = VMCI_SendDatagram((VMCIDatagram *)&wkMsg);
       if (result < VMCI_SUCCESS) {
-	 VMCI_LOG(("Failed to remove well-known mapping for resource %d.\n",
-		   entry->handle.resource));
+	 VMCI_WARNING((LGPFX"Failed to remove well-known mapping for "
+                       "resource %d.\n", entry->handle.resource));
       }
    }
 
@@ -651,33 +592,6 @@ VMCIDatagramDestroyHndInt(VMCIHandle handle)       // IN
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIDatagram_DestroyHnd --
- *
- *      Destroys a handle.
- *
- * Results:
- *      VMCI_SUCCESS or error code.
- *
- * Side effects:
- *      Host and guest state is cleaned up.
- *
- *-----------------------------------------------------------------------------
- */
-
-#ifdef __linux__
-EXPORT_SYMBOL(VMCIDatagram_DestroyHnd);
-#endif
-
-int
-VMCIDatagram_DestroyHnd(VMCIHandle handle)       // IN
-{
-   return VMCIDatagramDestroyHndInt(handle);
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -694,10 +608,7 @@ VMCIDatagram_DestroyHnd(VMCIHandle handle)       // IN
  *-----------------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCIDatagram_Send);
-#endif
-
+VMCI_EXPORT_SYMBOL(VMCIDatagram_Send)
 int
 VMCIDatagram_Send(VMCIDatagram *msg) // IN
 {
@@ -706,20 +617,21 @@ VMCIDatagram_Send(VMCIDatagram *msg) // IN
    VMCIId contextId;
 
    if (msg == NULL) {
-      VMCI_LOG(("Invalid datagram.\n"));
+      VMCI_DEBUG_LOG(4, (LGPFX"Invalid datagram.\n"));
       return VMCI_ERROR_INVALID_ARGS;
    }
 
    if (VMCI_DG_SIZE(msg) > VMCI_MAX_DG_SIZE) {
-      VMCI_LOG(("Payload size %"FMT64"u too big to send.\n", msg->payloadSize));
+      VMCI_DEBUG_LOG(4, (LGPFX"Payload size %"FMT64"u too big to send.\n",
+                         msg->payloadSize));
       return VMCI_ERROR_INVALID_ARGS;
    }
 
    /* Check srcHandle exists otherwise fail. */
    entry = DatagramHashGetEntry(msg->src);
    if (entry == NULL) {
-      VMCI_LOG(("Couldn't find handle 0x%x:0x%x.\n",
-		msg->src.context, msg->src.resource));
+      VMCI_DEBUG_LOG(4, (LGPFX"Couldn't find handle 0x%x:0x%x.\n",
+                         msg->src.context, msg->src.resource));
       return VMCI_ERROR_INVALID_ARGS;
    }
 
@@ -796,15 +708,15 @@ VMCIDatagram_Dispatch(VMCIId contextID,  // IN: unused
 
    entry = DatagramHashGetEntryAnyCid(msg->dst);
    if (entry == NULL) {
-      VMCI_LOG(("destination handle 0x%x:0x%x doesn't exist.\n",
-		msg->dst.context, msg->dst.resource));
+      VMCI_DEBUG_LOG(4, (LGPFX"destination handle 0x%x:0x%x doesn't exist.\n",
+                         msg->dst.context, msg->dst.resource));
       return VMCI_ERROR_NO_HANDLE;
    }
 
    if (!entry->recvCB) {
-      VMCI_LOG(("no handle callback for handle 0x%x:0x%x payload of "
-                "size %"FMT64"d.\n",
-                msg->dst.context, msg->dst.resource, msg->payloadSize));
+      VMCI_WARNING((LGPFX"no handle callback for handle 0x%x:0x%x payload of "
+                    "size %"FMT64"d.\n", msg->dst.context, msg->dst.resource,
+                    msg->payloadSize));
       goto out;
    }
 
@@ -892,15 +804,13 @@ VMCIDatagram_CheckHostCapabilities(void)
 
 
 /*
- *-----------------------------------------------------------------------------
+ * VMCIDatagram_Sync --
  *
- * DatagramProcessNotify --
- *
- *      Callback to send a notificaton to a vmci process. Creates datagram
- *      copy and signals the process.
+ *      Use this as a synchronization point when setting globals, for example,
+ *      during device shutdown.
  *
  * Results:
- *      VMCI_SUCCESS on success, appropriate error code otherwise.
+ *      None.
  *
  * Side effects:
  *      None.
@@ -908,264 +818,10 @@ VMCIDatagram_CheckHostCapabilities(void)
  *-----------------------------------------------------------------------------
  */
 
-static int
-DatagramProcessNotify(void *clientData,   // IN:
-                      VMCIDatagram *msg)  // IN:
-{
-   VMCIDatagramProcess *dgmProc = (VMCIDatagramProcess *) clientData;
-   size_t dgmSize;
-   VMCIDatagram *dgm;
-   DatagramQueueEntry *dqEntry;
-   VMCILockFlags flags;
-
-   ASSERT(dgmProc != NULL && msg != NULL);
-   dgmSize = VMCI_DG_SIZE(msg);
-   ASSERT(dgmSize <= VMCI_MAX_DG_SIZE);
-
-   dgm = VMCI_AllocKernelMem(dgmSize,
-			     VMCI_MEMORY_NONPAGED | VMCI_MEMORY_ATOMIC);
-   if (!dgm) {
-      VMCI_LOG(("VMCI: Failed to allocate datagram of size %d bytes.\n",
-		(uint32)dgmSize));
-      return VMCI_ERROR_NO_MEM;
-   }
-   memcpy(dgm, msg, dgmSize);
-
-   /* Allocate datagram queue entry and add it to the target fd's queue. */
-   dqEntry = VMCI_AllocKernelMem(sizeof *dqEntry,
-				 VMCI_MEMORY_NONPAGED | VMCI_MEMORY_ATOMIC);
-   if (dqEntry == NULL) {
-      VMCI_FreeKernelMem(dgm, dgmSize);
-      VMCI_LOG(("VMCI: Failed to allocate memory for process datagram.\n"));
-      return VMCI_ERROR_NO_MEM;
-   }
-   dqEntry->dg = dgm;
-
-   VMCI_GrabLock_BH(&dgmProc->datagramQueueLock, &flags);
-   if (dgmProc->datagramQueueSize + dgmSize >= VMCI_MAX_DATAGRAM_QUEUE_SIZE) {
-      VMCI_ReleaseLock_BH(&dgmProc->datagramQueueLock, flags);
-      VMCI_FreeKernelMem(dgm, dgmSize);
-      VMCI_FreeKernelMem(dqEntry, sizeof *dqEntry);
-      VMCI_LOG(("VMCI: Datagram process receive queue is full.\n"));
-      return VMCI_ERROR_NO_RESOURCES;
-   }
-
-   LIST_QUEUE(&dqEntry->listItem, &dgmProc->datagramQueue);
-   dgmProc->pendingDatagrams++;
-   dgmProc->datagramQueueSize += dgmSize;
-#ifdef SOLARIS
-   /*
-    * Release the lock here for Solaris. Otherwise, a deadlock
-    * may occur since pollwakeup(9F) (invoked from VMCIHost_SignalCall)
-    * and poll_common (invoked from poll(2)) try to grab a common lock.
-    * The man pages of pollwakeup(9F) and chpoll(9E) talk about this.
-    */
-   VMCI_ReleaseLock_BH(&dgmProc->datagramQueueLock, flags);
-#endif
-   VMCIHost_SignalCall(&dgmProc->host);
-#ifndef SOLARIS
-   /* For platforms other than Solaris, release the lock here. */
-   VMCI_ReleaseLock_BH(&dgmProc->datagramQueueLock, flags);
-#endif
-
-   DEBUG_ONLY(VMCI_LOG(("VMCI: Sent datagram with resource id %d and size %u.\n",
-                        msg->dst.resource, (uint32)dgmSize));)
-   /* dqEntry and dgm are freed when user reads call.. */
-
-   return VMCI_SUCCESS;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * VMCIDatagramProcess_Create --
- *
- *      Creates a new VMCIDatagramProcess object.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-VMCIDatagramProcess_Create(VMCIDatagramProcess **outDgmProc,          // OUT:
-                           VMCIDatagramCreateProcessInfo *createInfo, // IN:
-                           uintptr_t eventHnd)                        // IN:
-{
-   VMCIDatagramProcess *dgmProc;
-
-   ASSERT(createInfo);
-   ASSERT(outDgmProc);
-   dgmProc = VMCI_AllocKernelMem(sizeof *dgmProc, VMCI_MEMORY_NONPAGED);
-   if (dgmProc == NULL) {
-      return VMCI_ERROR_NO_MEM;
-   }
-
-   VMCI_InitLock(&dgmProc->datagramQueueLock, "VMCIDgmProc",
-		 VMCI_LOCK_RANK_MIDDLE_BH);
-   VMCIHost_InitContext(&dgmProc->host, eventHnd);
-   dgmProc->pendingDatagrams = 0;
-   dgmProc->datagramQueueSize = 0;
-   dgmProc->datagramQueue = NULL;
-
-   /*
-    * We pass the result and corresponding handle to user level via the
-    * createInfo.
-    */
-   createInfo->result = VMCIDatagram_CreateHnd(createInfo->resourceID,
-                                               createInfo->flags,
-                                               DatagramProcessNotify,
-                                               (void *)dgmProc,
-                                               &dgmProc->handle);
-   if (createInfo->result < VMCI_SUCCESS) {
-      VMCI_FreeKernelMem(dgmProc, sizeof *dgmProc);
-      return createInfo->result;
-   }
-   createInfo->handle = dgmProc->handle;
-
-   *outDgmProc = dgmProc;
-   return VMCI_SUCCESS;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * VMCIDatagramProcess_Destroy --
- *
- *      Destroys a VMCIDatagramProcess object.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
 void
-VMCIDatagramProcess_Destroy(VMCIDatagramProcess *dgmProc) // IN:
+VMCIDatagram_Sync(void)
 {
-   ListItem *curr, *next;
-   DatagramQueueEntry *dqEntry;
    VMCILockFlags flags;
-
-   if (!dgmProc) {
-      return;
-   }
-
-   if (!VMCI_HANDLE_EQUAL(dgmProc->handle, VMCI_INVALID_HANDLE)) {
-
-      /*
-       * We block in destroy so we know that there can be no more
-       * callbacks to DatagramProcessNotifyCB when we return from
-       * this call.
-       */
-      VMCIDatagram_DestroyHnd(dgmProc->handle);
-      dgmProc->handle = VMCI_INVALID_HANDLE;
-   }
-
-   /* Flush dgmProc's call queue. */
-   VMCI_GrabLock_BH(&dgmProc->datagramQueueLock, &flags);
-   LIST_SCAN_SAFE(curr, next, dgmProc->datagramQueue) {
-      dqEntry = LIST_CONTAINER(curr, DatagramQueueEntry, listItem);
-      LIST_DEL(curr, &dgmProc->datagramQueue);
-      ASSERT(dqEntry && dqEntry->dg);
-      VMCI_FreeKernelMem(dqEntry->dg, VMCI_DG_SIZE(dqEntry->dg));
-      VMCI_FreeKernelMem(dqEntry, sizeof *dqEntry);
-   }
-   VMCI_ReleaseLock_BH(&dgmProc->datagramQueueLock, flags);
-   VMCIHost_ReleaseContext(&dgmProc->host);
-   VMCI_CleanupLock(&dgmProc->datagramQueueLock);
-   VMCI_FreeKernelMem(dgmProc, sizeof *dgmProc);
+   VMCI_GrabLock_BH(&hashTable.lock, &flags);
+   VMCI_ReleaseLock_BH(&hashTable.lock, flags);
 }
-
-
-/*
- *----------------------------------------------------------------------
- *
- * VMCIDatagramProcess_ReadCall --
- *
- *      Dequeues the next guest call and returns it to user level.
- *
- * Results:
- *      0 on success, appropriate error code otherwise.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-VMCIDatagramProcess_ReadCall(VMCIDatagramProcess *dgmProc, // IN:
-                             size_t maxSize,               // IN: max size of dg
-                             VMCIDatagram **dg)            // OUT:
-{
-   DatagramQueueEntry *dqEntry;
-   ListItem *listItem;
-   VMCILockFlags flags;
-
-   ASSERT(dgmProc);
-   ASSERT(dg);
-
-   /* Dequeue the next dgmProc datagram queue entry. */
-   VMCI_GrabLock_BH(&dgmProc->datagramQueueLock, &flags);
-
-   /*
-    * Currently, we do not support blocking read of datagrams on Mac and
-    * Solaris. XXX: This will go away soon.
-    */
-
-#if defined(SOLARIS) || defined(__APPLE__)
-   if (dgmProc->pendingDatagrams == 0) {
-      VMCIHost_ClearCall(&dgmProc->host);
-      VMCI_ReleaseLock_BH(&dgmProc->datagramQueueLock, flags);
-      VMCI_LOG(("VMCI: No datagrams pending.\n"));
-      return VMCI_ERROR_NO_MORE_DATAGRAMS;
-   }
-#else
-   while (dgmProc->pendingDatagrams == 0) {
-      VMCIHost_ClearCall(&dgmProc->host);
-      if (!VMCIHost_WaitForCallLocked(&dgmProc->host, &dgmProc->datagramQueueLock,
-                                      &flags, TRUE)) {
-         VMCI_ReleaseLock_BH(&dgmProc->datagramQueueLock, flags);
-         VMCI_LOG(("VMCI: Blocking read of datagram interrupted.\n"));
-         return VMCI_ERROR_NO_MORE_DATAGRAMS;
-      }
-   }
-#endif
-
-   listItem = LIST_FIRST(dgmProc->datagramQueue);
-   ASSERT (listItem != NULL);
-
-   dqEntry = LIST_CONTAINER(listItem, DatagramQueueEntry, listItem);
-   ASSERT(dqEntry->dg);
-
-   /* Check the size of the userland buffer. */
-   if (maxSize < VMCI_DG_SIZE(dqEntry->dg)) {
-      VMCI_ReleaseLock_BH(&dgmProc->datagramQueueLock, flags);
-      VMCI_LOG(("VMCI: Caller's buffer is too small.\n"));
-      return VMCI_ERROR_NO_MEM;
-   }
-
-   LIST_DEL(listItem, &dgmProc->datagramQueue);
-   dgmProc->pendingDatagrams--;
-   dgmProc->datagramQueueSize -= VMCI_DG_SIZE(dqEntry->dg);
-   if (dgmProc->pendingDatagrams == 0) {
-      VMCIHost_ClearCall(&dgmProc->host);
-   }
-   VMCI_ReleaseLock_BH(&dgmProc->datagramQueueLock, flags);
-
-   *dg = dqEntry->dg;
-   VMCI_FreeKernelMem(dqEntry, sizeof *dqEntry);
-
-   return VMCI_SUCCESS;
-}
-
