@@ -17,12 +17,12 @@
  *********************************************************/
 
 /*
- * vmciKernelIf.c -- 
- * 
+ * vmciKernelIf.c --
+ *
  *      This file implements defines and helper functions for VMCI
  *      host _and_ guest kernel code. This is the linux specific
  *      implementation.
- */ 
+ */
 
 /* Must come before any kernel header file */
 #include "driver-config.h"
@@ -35,9 +35,6 @@
 #  error "Linux kernels before 2.6.9 are not supported."
 #endif
 
-#define EXPORT_SYMTAB
-#define __NO_VERSION__
-
 #include <linux/mm.h>           /* For vmalloc_to_page() and get_user_pages()*/
 #include <linux/pagemap.h>      /* For page_cache_release() */
 #include <linux/socket.h>       /* For memcpy_{to,from}iovec(). */
@@ -1238,7 +1235,7 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
          err = memcpy_fromiovec((uint8 *)va + pageOffset, iov, toCopy);
          if (err != 0) {
             kunmap(kernelIf->page[pageIndex]);
-            return err;
+            return VMCI_ERROR_INVALID_ARGS;
          }
       } else {
          memcpy((uint8 *)va + pageOffset, (uint8 *)src + bytesCopied, toCopy);
@@ -1248,7 +1245,7 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
       kunmap(kernelIf->page[pageIndex]);
    }
 
-   return 0;
+   return VMCI_SUCCESS;
 }
 
 
@@ -1303,7 +1300,7 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
          err = memcpy_toiovec(iov, (uint8 *)va + pageOffset, toCopy);
          if (err != 0) {
             kunmap(kernelIf->page[pageIndex]);
-            return err;
+            return VMCI_ERROR_INVALID_ARGS;
          }
       } else {
          memcpy((uint8 *)dest + bytesCopied, (uint8 *)va + pageOffset, toCopy);
@@ -1313,7 +1310,7 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
       kunmap(kernelIf->page[pageIndex]);
    }
 
-   return 0;
+   return VMCI_SUCCESS;
 }
 
 
@@ -1333,8 +1330,6 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
  *-----------------------------------------------------------------------------
  */
 
-EXPORT_SYMBOL(VMCIMemcpyToQueue);
-
 int
 VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
                   uint64 queueOffset, // IN:
@@ -1364,8 +1359,6 @@ VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
  *-----------------------------------------------------------------------------
  */
 
-EXPORT_SYMBOL(VMCIMemcpyFromQueue);
-
 int
 VMCIMemcpyFromQueue(void *dest,             // OUT:
                     size_t destOffset,      // IN:
@@ -1395,8 +1388,6 @@ VMCIMemcpyFromQueue(void *dest,             // OUT:
  *----------------------------------------------------------------------------
  */
 
-EXPORT_SYMBOL(VMCIMemcpyToQueueV);
-
 int
 VMCIMemcpyToQueueV(VMCIQueue *queue,      // OUT:
                    uint64 queueOffset,    // IN:
@@ -1430,8 +1421,6 @@ VMCIMemcpyToQueueV(VMCIQueue *queue,      // OUT:
  *----------------------------------------------------------------------------
  */
 
-EXPORT_SYMBOL(VMCIMemcpyFromQueueV);
-
 int
 VMCIMemcpyFromQueueV(void *dest,              // OUT: iovec
                      size_t destOffset,       // IN: ignored
@@ -1630,6 +1619,9 @@ VMCIHost_GetUserMemory(PageStoreAttachInfo *attach,      // IN/OUT
       err = VMCI_ERROR_NO_MEM;
    }
 
+out:
+   up_write(&current->mm->mmap_sem);
+
    if (err == VMCI_SUCCESS) {
       produceQ->qHeader = kmap(attach->producePages[0]);
       produceQ->kernelIf->page = &attach->producePages[1];
@@ -1637,9 +1629,6 @@ VMCIHost_GetUserMemory(PageStoreAttachInfo *attach,      // IN/OUT
       consumeQ->kernelIf->page = &attach->consumePages[1];
    }
 
-out:
-   up_write(&current->mm->mmap_sem);
-
 errorDealloc:
    if (err < VMCI_SUCCESS) {
       if (attach->producePages != NULL) {
