@@ -39,6 +39,8 @@
 #include "vmciUtil.h"
 #include "circList.h"
 
+#define LGPFX "VMCINotifications: "
+
 #if !defined(SOLARIS) && !defined(__APPLE__)
 
 /*
@@ -208,6 +210,93 @@ VMCINotifications_Exit(void)
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCINotifications_Sync --
+ *
+ *      Use this as a synchronization point when setting globals, for example,
+ *      during device shutdown.
+ *
+ * Results:
+ *      TRUE.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+VMCINotifications_Sync(void)
+{
+   VMCILockFlags flags;
+   VMCI_GrabLock_BH(&vmciNotifyHT.lock, &flags);
+   VMCI_ReleaseLock_BH(&vmciNotifyHT.lock, flags);
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * VMCINotifications_Hibernate --
+ *
+ *    When a guest leaves hibernation, the device driver state is out
+ *    of sync with the device state, since the driver state has
+ *    doorbells registered that aren't known to the device. This
+ *    function takes care of reregistering any doorbells. In case an
+ *    error occurs during reregistration (this is highly unlikely
+ *    since 1) it succeeded the first time 2) the device driver is the
+ *    only source of doorbell registrations), we simply log the
+ *    error. The doorbell can still be destroyed using
+ *    VMCIDoorbell_Destroy.
+ *
+ * Results:
+ *    None.
+ *
+ * Side effects:
+ *    None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+VMCINotifications_Hibernate(Bool enterHibernate)  // IN
+{
+   VMCILockFlags flags;
+   uint32 bucket;
+   ListItem *iter;
+
+   if (enterHibernate) {
+      /*
+       * Nothing to do when entering hibernation.
+       */
+
+      return;
+   }
+
+   VMCI_GrabLock_BH(&vmciNotifyHT.lock, &flags);
+
+   for (bucket = 0; bucket < HASH_TABLE_SIZE; bucket++) {
+      LIST_SCAN(iter, vmciNotifyHT.entriesByIdx[bucket]) {
+         VMCINotifyHashEntry *cur;
+         int result;
+
+         cur = LIST_CONTAINER(iter, VMCINotifyHashEntry, idxListItem);
+         result = LinkNotificationHypercall(cur->handle, cur->doorbell, cur->idx);
+         if (result != VMCI_SUCCESS && result != VMCI_ERROR_DUPLICATE_ENTRY) {
+            VMCI_WARNING((LGPFX"Failed to reregister doorbell handle 0x%x:0x%x "
+                          "of resource %s to index (error: %d).\n",
+                          cur->handle.context, cur->handle.resource,
+                          cur->doorbell ? "doorbell" : "queue pair", result));
+         }
+      }
+   }
+
+   VMCI_ReleaseLock_BH(&vmciNotifyHT.lock, flags);
+}
+
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -238,6 +327,12 @@ VMCINotifyHashAddEntry(VMCINotifyHashEntry *entry) // IN
 
    VMCI_GrabLock_BH(&vmciNotifyHT.lock, &flags);
 
+   /* Do not allow addition of a new handle if the device is being shutdown. */
+   if (VMCI_DeviceShutdown()) {
+      result = VMCI_ERROR_DEVICE_NOT_FOUND;
+      goto out;
+   }
+
    if (VMCI_HANDLE_INVALID(entry->handle)) {
       VMCIHandle newHandle;
       VMCIId oldRID = notifyRID;
@@ -257,6 +352,14 @@ VMCINotifyHashAddEntry(VMCINotifyHashEntry *entry) // IN
          goto out;
       }
       entry->handle = newHandle;
+   } else if (VMCI_GetContextID() != entry->handle.context) {
+      /*
+       * The context id passed down should either be invalid or
+       * the context id of the guest.
+       */
+
+      result = VMCI_ERROR_INVALID_ARGS;
+      goto out;
    }
 
    if (VMCINotifyHashFindByHandle(entry->handle, entry->doorbell, &bucket)) {
@@ -775,9 +878,10 @@ VMCINotificationRegister(VMCIHandle *handle,     // IN
 
    result = LinkNotificationHypercall(entry->handle, doorbell, entry->idx);
    if (result != VMCI_SUCCESS) {
-      VMCI_LOG(("Failed to link handle 0x%x:0x%x of resource %s to index, "
-                "err 0x%x.\n", entry->handle.context, entry->handle.resource,
-                entry->doorbell ? "doorbell" : "queue pair", result));
+      VMCI_DEBUG_LOG(4, (LGPFX"Failed to link handle 0x%x:0x%x of resource %s "
+                         "to index (error: %d).\n",
+                         entry->handle.context, entry->handle.resource,
+                         entry->doorbell ? "doorbell" : "queue pair", result));
       VMCINotifyHashRemoveEntry(entry->handle, entry->doorbell);
       VMCI_DestroyEvent(&entry->destroyEvent);
       VMCI_FreeKernelMem(entry, sizeof *entry);
@@ -823,7 +927,6 @@ VMCINotificationUnregister(VMCIHandle handle, // IN
 
    entry = VMCINotifyHashRemoveEntry(handle, doorbell);
    if (!entry) {
-      ASSERT(FALSE);
       return VMCI_ERROR_NOT_FOUND;
    }
 
@@ -837,15 +940,17 @@ VMCINotificationUnregister(VMCIHandle handle, // IN
        * The only reason this should fail would be an inconsistency
        * between guest and hypervisor state, where the guest believes
        * it has an active registration whereas the hypervisor
-       * doesn't. Since the handle has now been removed in the guest,
-       * we just print a warning and return success.
+       * doesn't. One case where this may happen is if a doorbell is
+       * unregistered following a hibernation at a time where the
+       * doorbell state hasn't been restored on the hypervisor side
+       * yet. Since the handle has now been removed in the guest, we
+       * just print a warning and return success.
        */
 
-      ASSERT(FALSE);
-
-      VMCI_LOG(("Unlink of %s  handle 0x%x:0x%x unknown by hypervisor.\n",
-                doorbell ? "doorbell" : "queuepair",
-                handle.context, handle.resource));
+      VMCI_DEBUG_LOG(4, (LGPFX"Unlink of %s  handle 0x%x:0x%x unknown by "
+                         "hypervisor (error: %d).\n",
+                         doorbell ? "doorbell" : "queuepair",
+                         handle.context, handle.resource, result));
    }
    return VMCI_SUCCESS;
 }
@@ -883,8 +988,9 @@ VMCI_RegisterNotificationBitmap(PPN bitmapPPN) // IN
 
    result = VMCI_SendDatagram((VMCIDatagram *)&bitmapSetMsg);
    if (result != VMCI_SUCCESS) {
-      VMCI_LOG(("VMCINotifications: Failed to register PPN %u as notification "
-                "bitmap (error : %d).\n", bitmapPPN, result));
+      VMCI_DEBUG_LOG(4, (LGPFX"VMCINotifications: Failed to register PPN %u as "
+                         "notification bitmap (error: %d).\n",
+                         bitmapPPN, result));
       return FALSE;
    }
    return TRUE;
@@ -944,6 +1050,7 @@ VMCI_ScanNotificationBitmap(uint8 *bitmap) // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIDoorbell_Create)
 int
 VMCIDoorbell_Create(VMCIHandle *handle,            // IN
                     uint32 flags,                  // IN
@@ -962,10 +1069,6 @@ VMCIDoorbell_Create(VMCIHandle *handle,            // IN
                                    notifyCB, clientData);
 }
 
-#if defined(__linux__)
-EXPORT_SYMBOL(VMCIDoorbell_Create);
-#endif
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -985,6 +1088,7 @@ EXPORT_SYMBOL(VMCIDoorbell_Create);
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIDoorbell_Destroy)
 int
 VMCIDoorbell_Destroy(VMCIHandle handle) // IN
 {
@@ -994,10 +1098,6 @@ VMCIDoorbell_Destroy(VMCIHandle handle) // IN
    return VMCINotificationUnregister(handle, TRUE);
 }
 
-#if defined(__linux__)
-EXPORT_SYMBOL(VMCIDoorbell_Destroy);
-#endif
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -1016,6 +1116,7 @@ EXPORT_SYMBOL(VMCIDoorbell_Destroy);
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIDoorbell_Notify)
 int
 VMCIDoorbell_Notify(VMCIHandle handle,             // IN
                     VMCIPrivilegeFlags privFlags)  // IN: Unused in guest
@@ -1041,9 +1142,6 @@ VMCIDoorbell_Notify(VMCIHandle handle,             // IN
    return result;
 }
 
-#if defined(__linux__)
-EXPORT_SYMBOL(VMCIDoorbell_Notify);
-#endif
 
 #else // defined(SOLARIS) || defined(__APPLE__)
 
@@ -1064,9 +1162,10 @@ EXPORT_SYMBOL(VMCIDoorbell_Notify);
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIDoorbell_Create)
 int
 VMCIDoorbell_Create(VMCIHandle *handle,            // IN
-                    uint32 flags,                  // IN
+                    uint32 flags,                  // I
                     VMCIPrivilegeFlags privFlags,  // IN
                     VMCICallback notifyCB,         // IN
                     void *clientData)              // IN
@@ -1075,6 +1174,7 @@ VMCIDoorbell_Create(VMCIHandle *handle,            // IN
 }
 
 
+VMCI_EXPORT_SYMBOL(VMCIDoorbell_Destroy)
 int
 VMCIDoorbell_Destroy(VMCIHandle handle)  // IN
 {
@@ -1082,6 +1182,7 @@ VMCIDoorbell_Destroy(VMCIHandle handle)  // IN
 }
 
 
+VMCI_EXPORT_SYMBOL(VMCIDoorbell_Notify)
 int
 VMCIDoorbell_Notify(VMCIHandle handle,             // IN
                     VMCIPrivilegeFlags privFlags)  // IN
