@@ -63,7 +63,6 @@
 
 /* Must come before any kernel header file. */
 #if defined(__linux__) && !defined(VMKERNEL)
-#  define EXPORT_SYMTAB
 #  include "driver-config.h"
 #  include "compat_module.h"
 #endif
@@ -101,6 +100,9 @@ struct VMCIQPair {
    VMCIPrivilegeFlags privFlags;
 };
 
+#define VMCI_QPAIR_NO_QUEUE(_qp) (!(_qp)->produceQ->qHeader || \
+                                  !(_qp)->consumeQ->qHeader)
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -123,6 +125,7 @@ struct VMCIQPair {
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_Alloc)
 int
 VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
                 VMCIHandle *handle,           // OUT
@@ -178,7 +181,7 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
  *      VMCIQPair structure, too.
  *
  * Results:
- *      None.
+ *      An error, if < 0.
  *
  * Side effects:
  *      Will clear the caller's pointer to the VMCIQPair structure.
@@ -186,35 +189,41 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
  *-----------------------------------------------------------------------------
  */
 
-void
+VMCI_EXPORT_SYMBOL(VMCIQPair_Detach)
+int
 VMCIQPair_Detach(VMCIQPair **qpair) // IN/OUT
 {
-   VMCIQPair *oldQPair = *qpair;
+   int result;
+   VMCIQPair *oldQPair;
 
-   VMCIQueuePair_Detach(oldQPair->handle);
+   if (!qpair || !(*qpair)) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
 
+   oldQPair = *qpair;
+   result = VMCIQueuePair_Detach(oldQPair->handle);
+
+   if  (result >= VMCI_SUCCESS) {
 #ifdef DEBUG
-   oldQPair->handle = VMCI_INVALID_HANDLE;
-   oldQPair->produceQ = NULL;
-   oldQPair->consumeQ = NULL;
-   oldQPair->produceQSize = 0;
-   oldQPair->consumeQSize = 0;
-   oldQPair->flags = 0;
-   oldQPair->privFlags = 0;
-   oldQPair->peer = VMCI_INVALID_ID;
+      oldQPair->handle = VMCI_INVALID_HANDLE;
+      oldQPair->produceQ = NULL;
+      oldQPair->consumeQ = NULL;
+      oldQPair->produceQSize = 0;
+      oldQPair->consumeQSize = 0;
+      oldQPair->flags = 0;
+      oldQPair->privFlags = 0;
+      oldQPair->peer = VMCI_INVALID_ID;
 #endif
 
-   VMCI_FreeKernelMem(oldQPair, sizeof *oldQPair);
+      VMCI_FreeKernelMem(oldQPair, sizeof *oldQPair);
 
-   *qpair = NULL;
+      *qpair = NULL;
+   }
+
+   return result;
 }
 
 
-#if defined __linux__ && !defined VMKERNEL
-EXPORT_SYMBOL(VMCIQPair_Alloc);
-EXPORT_SYMBOL(VMCIQPair_Detach);
-#endif
-
 /*
  * "Windows blocking call."
  *
@@ -246,8 +255,8 @@ EXPORT_SYMBOL(VMCIQPair_Detach);
 static INLINE void
 VMCIQPairLock(const VMCIQPair *qpair) // IN
 {
-#if !defined VMX86_TOOLS && !defined VMX86_VMX
-   VMCIHost_AcquireQueueMutex(qpair->produceQ);
+#if !defined VMX86_VMX
+   VMCI_AcquireQueueMutex(qpair->produceQ);
 #endif
 }
 
@@ -271,42 +280,12 @@ VMCIQPairLock(const VMCIQPair *qpair) // IN
 static INLINE void
 VMCIQPairUnlock(const VMCIQPair *qpair) // IN
 {
-#if !defined VMX86_TOOLS && !defined VMX86_VMX
-   VMCIHost_ReleaseQueueMutex(qpair->produceQ);
+#if !defined VMX86_VMX
+   VMCI_ReleaseQueueMutex(qpair->produceQ);
 #endif
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIQPair_Init --
- *
- *      This is the client interface for initializing the producer's
- *      pointers.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      Windows blocking call.
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-VMCIQPair_Init(VMCIQPair *qpair)
-{
-   VMCIQPairLock(qpair);
-
-   if (NULL != qpair->produceQ && NULL != qpair->produceQ->qHeader) {
-      VMCIQueueHeader_Init(qpair->produceQ->qHeader, qpair->handle);
-   }
-
-   VMCIQPairUnlock(qpair);
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -316,7 +295,8 @@ VMCIQPair_Init(VMCIQPair *qpair)
  *      QPair from the point of the view of the caller as the producer.
  *
  * Results:
- *      None.
+ *      err, if < 0
+ *      Success otherwise.
  *
  * Side effects:
  *      Windows blocking call.
@@ -324,19 +304,39 @@ VMCIQPair_Init(VMCIQPair *qpair)
  *-----------------------------------------------------------------------------
  */
 
-void
+VMCI_EXPORT_SYMBOL(VMCIQPair_GetProduceIndexes)
+int
 VMCIQPair_GetProduceIndexes(const VMCIQPair *qpair, // IN
                             uint64 *producerTail,   // OUT
                             uint64 *consumerHead)   // OUT
 {
+   int result;
+
+   if (!qpair) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    VMCIQPairLock(qpair);
 
-   VMCIQueueHeader_GetPointers(qpair->produceQ->qHeader,
-                               qpair->consumeQ->qHeader,
-                               producerTail,
-                               consumerHead);
+   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
+      result = VMCI_ERROR_QUEUEPAIR_NOTATTACHED;
+   } else {
+      VMCIQueueHeader_GetPointers(qpair->produceQ->qHeader,
+                                  qpair->consumeQ->qHeader,
+                                  producerTail,
+                                  consumerHead);
+      result = VMCI_SUCCESS;
+   }
 
    VMCIQPairUnlock(qpair);
+
+   if (result == VMCI_SUCCESS &&
+       ((producerTail && *producerTail >= qpair->produceQSize) ||
+        (consumerHead && *consumerHead >= qpair->produceQSize))) {
+      return VMCI_ERROR_INVALID_SIZE;
+   }
+
+   return result;
 }
 
 
@@ -349,7 +349,8 @@ VMCIQPair_GetProduceIndexes(const VMCIQPair *qpair, // IN
  *      QPair from the point of the view of the caller as the consumer.
  *
  * Results:
- *      None.
+ *      err, if < 0
+ *      Success otherwise.
  *
  * Side effects:
  *      Windows blocking call.
@@ -357,26 +358,40 @@ VMCIQPair_GetProduceIndexes(const VMCIQPair *qpair, // IN
  *-----------------------------------------------------------------------------
  */
 
-void
+VMCI_EXPORT_SYMBOL(VMCIQPair_GetConsumeIndexes)
+int
 VMCIQPair_GetConsumeIndexes(const VMCIQPair *qpair, // IN
                             uint64 *consumerTail,   // OUT
                             uint64 *producerHead)   // OUT
 {
+   int result;
+
+   if (!qpair) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    VMCIQPairLock(qpair);
 
-   VMCIQueueHeader_GetPointers(qpair->consumeQ->qHeader,
-                               qpair->produceQ->qHeader,
-                               consumerTail,
-                               producerHead);
+   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
+      result = VMCI_ERROR_QUEUEPAIR_NOTATTACHED;
+   } else {
+      VMCIQueueHeader_GetPointers(qpair->consumeQ->qHeader,
+                                  qpair->produceQ->qHeader,
+                                  consumerTail,
+                                  producerHead);
+      result = VMCI_SUCCESS;
+   }
 
    VMCIQPairUnlock(qpair);
-}
 
-#if defined __linux__ && !defined VMKERNEL
-EXPORT_SYMBOL(VMCIQPair_Init);
-EXPORT_SYMBOL(VMCIQPair_GetProduceIndexes);
-EXPORT_SYMBOL(VMCIQPair_GetConsumeIndexes);
-#endif
+   if (result == VMCI_SUCCESS &&
+       ((consumerTail && *consumerTail >= qpair->consumeQSize) ||
+        (producerHead && *producerHead >= qpair->consumeQSize))) {
+      return VMCI_ERROR_INVALID_SIZE;
+   }
+
+   return result;
+}
 
 
 /*
@@ -399,16 +414,25 @@ EXPORT_SYMBOL(VMCIQPair_GetConsumeIndexes);
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_ProduceFreeSpace)
 int64
 VMCIQPair_ProduceFreeSpace(const VMCIQPair *qpair) // IN
 {
    int64 result;
 
+   if (!qpair) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    VMCIQPairLock(qpair);
 
-   result = VMCIQueueHeader_FreeSpace(qpair->produceQ->qHeader,
-                                      qpair->consumeQ->qHeader,
-                                      qpair->produceQSize);
+   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
+      result = 0;
+   } else {
+      result = VMCIQueueHeader_FreeSpace(qpair->produceQ->qHeader,
+                                         qpair->consumeQ->qHeader,
+                                         qpair->produceQSize);
+   }
 
    VMCIQPairUnlock(qpair);
 
@@ -437,16 +461,25 @@ VMCIQPair_ProduceFreeSpace(const VMCIQPair *qpair) // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_ConsumeFreeSpace)
 int64
 VMCIQPair_ConsumeFreeSpace(const VMCIQPair *qpair) // IN
 {
    int64 result;
 
+   if (!qpair) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    VMCIQPairLock(qpair);
 
-   result = VMCIQueueHeader_FreeSpace(qpair->consumeQ->qHeader,
-                                      qpair->produceQ->qHeader,
-                                      qpair->consumeQSize);
+   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
+      result = 0;
+   } else {
+      result = VMCIQueueHeader_FreeSpace(qpair->consumeQ->qHeader,
+                                         qpair->produceQ->qHeader,
+                                         qpair->consumeQSize);
+   }
 
    VMCIQPairUnlock(qpair);
 
@@ -475,16 +508,25 @@ VMCIQPair_ConsumeFreeSpace(const VMCIQPair *qpair) // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_ProduceBufReady)
 int64
 VMCIQPair_ProduceBufReady(const VMCIQPair *qpair) // IN
 {
    int64 result;
 
+   if (!qpair) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    VMCIQPairLock(qpair);
 
-   result = VMCIQueueHeader_BufReady(qpair->produceQ->qHeader,
-                                     qpair->consumeQ->qHeader,
-                                     qpair->produceQSize);
+   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
+      result = 0;
+   } else {
+      result = VMCIQueueHeader_BufReady(qpair->produceQ->qHeader,
+                                        qpair->consumeQ->qHeader,
+                                        qpair->produceQSize);
+   }
 
    VMCIQPairUnlock(qpair);
 
@@ -512,29 +554,31 @@ VMCIQPair_ProduceBufReady(const VMCIQPair *qpair) // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_ConsumeBufReady)
 int64
 VMCIQPair_ConsumeBufReady(const VMCIQPair *qpair) // IN
 {
    int64 result;
 
+   if (!qpair) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    VMCIQPairLock(qpair);
 
-   result = VMCIQueueHeader_BufReady(qpair->consumeQ->qHeader,
-                                     qpair->produceQ->qHeader,
-                                     qpair->consumeQSize);
+   if (UNLIKELY(VMCI_QPAIR_NO_QUEUE(qpair))) {
+      result = 0;
+   } else {
+      result = VMCIQueueHeader_BufReady(qpair->consumeQ->qHeader,
+                                        qpair->produceQ->qHeader,
+                                        qpair->consumeQSize);
+   }
 
    VMCIQPairUnlock(qpair);
 
    return result;
 }
 
-#if defined __linux__ && !defined VMKERNEL
-EXPORT_SYMBOL(VMCIQPair_ProduceFreeSpace);
-EXPORT_SYMBOL(VMCIQPair_ConsumeFreeSpace);
-EXPORT_SYMBOL(VMCIQPair_ProduceBufReady);
-EXPORT_SYMBOL(VMCIQPair_ConsumeBufReady);
-#endif
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -551,6 +595,9 @@ EXPORT_SYMBOL(VMCIQPair_ConsumeBufReady);
  *      VMCI_ERROR_QUEUEPAIR_NOSPACE if no space was available to enqueue data.
  *      VMCI_ERROR_INVALID_SIZE, if any queue pointer is outside the queue
  *      (as defined by the queue size).
+ *      VMCI_ERROR_INVALID_ARGS, if an error occured when accessing the buffer.
+ *      VMCI_ERROR_QUEUEPAIR_NOTATTACHED, if the queue pair pages aren't
+ *      available.
  *      Otherwise, the number of bytes written to the queue is returned.
  *
  * Side effects:
@@ -571,9 +618,10 @@ EnqueueLocked(VMCIQueue *produceQ,                   // IN
    int64 freeSpace;
    uint64 tail;
    size_t written;
+   ssize_t result;
 
-#if !defined VMX86_TOOLS && !defined VMX86_VMX
-   if (UNLIKELY(VMCIHost_EnqueueToDevNull(produceQ))) {
+#if !defined VMX86_VMX
+   if (UNLIKELY(VMCI_EnqueueToDevNull(produceQ))) {
       return (ssize_t) bufSize;
    }
 
@@ -596,18 +644,23 @@ EnqueueLocked(VMCIQueue *produceQ,                   // IN
    written = (size_t)(freeSpace > bufSize ? bufSize : freeSpace);
    tail = VMCIQueueHeader_ProducerTail(produceQ->qHeader);
    if (LIKELY(tail + written < produceQSize)) {
-      memcpyToQueue(produceQ, tail, buf, 0, written, bufType);
+      result = memcpyToQueue(produceQ, tail, buf, 0, written, bufType);
    } else {
       /* Tail pointer wraps around. */
 
       const size_t tmp = (size_t)(produceQSize - tail);
 
-      memcpyToQueue(produceQ, tail, buf, 0, tmp, bufType);
-      memcpyToQueue(produceQ, 0, buf, tmp, written - tmp, bufType);
+      result = memcpyToQueue(produceQ, tail, buf, 0, tmp, bufType);
+      if (result >= VMCI_SUCCESS) {
+         result = memcpyToQueue(produceQ, 0, buf, tmp, written - tmp, bufType);
+      }
+   }
+
+   if (result < VMCI_SUCCESS) {
+      return result;
    }
 
    VMCIQueueHeader_AddProducerTail(produceQ->qHeader, written, produceQSize);
-
    return written;
 }
 
@@ -626,6 +679,7 @@ EnqueueLocked(VMCIQueue *produceQ,                   // IN
  *      VMCI_ERROR_QUEUEPAIR_NODATA if no data was available to dequeue.
  *      VMCI_ERROR_INVALID_SIZE, if any queue pointer is outside the queue
  *      (as defined by the queue size).
+ *      VMCI_ERROR_INVALID_ARGS, if an error occured when accessing the buffer.
  *      Otherwise the number of bytes dequeued is returned.
  *
  * Side effects:
@@ -646,14 +700,15 @@ DequeueLocked(VMCIQueue *produceQ,                        // IN
 {
    int64 bufReady;
    uint64 head;
-   size_t written;
+   size_t read;
+   ssize_t result;
 
-#if defined _WIN32 && !defined VMX86_TOOLS && !defined VMX86_VMX
+#if !defined VMX86_TOOLS && !defined VMX86_VMX
    if (UNLIKELY(!produceQ->qHeader ||
                 !consumeQ->qHeader)) {
       return VMCI_ERROR_QUEUEPAIR_NODATA;
    }
-#endif /* Windows Host only support */
+#endif
 
    bufReady = VMCIQueueHeader_BufReady(consumeQ->qHeader,
                                        produceQ->qHeader,
@@ -665,26 +720,32 @@ DequeueLocked(VMCIQueue *produceQ,                        // IN
       return (ssize_t)bufReady;
    }
 
-   written = (size_t)(bufReady > bufSize ? bufSize : bufReady);
+   read = (size_t)(bufReady > bufSize ? bufSize : bufReady);
    head = VMCIQueueHeader_ConsumerHead(produceQ->qHeader);
-   if (LIKELY(head + written < consumeQSize)) {
-      memcpyFromQueue(buf, 0, consumeQ, head, written, bufType);
+   if (LIKELY(head + read < consumeQSize)) {
+      result = memcpyFromQueue(buf, 0, consumeQ, head, read, bufType);
    } else {
       /* Head pointer wraps around. */
 
       const size_t tmp = (size_t)(consumeQSize - head);
 
-      memcpyFromQueue(buf, 0, consumeQ, head, tmp, bufType);
-      memcpyFromQueue(buf, tmp, consumeQ, 0, written - tmp, bufType);
+      result = memcpyFromQueue(buf, 0, consumeQ, head, tmp, bufType);
+      if (result >= VMCI_SUCCESS) {
+         result = memcpyFromQueue(buf, tmp, consumeQ, 0, read - tmp, bufType);
+      }
+   }
+
+   if (result < VMCI_SUCCESS) {
+      return result;
    }
 
    if (updateConsumer) {
       VMCIQueueHeader_AddConsumerHead(produceQ->qHeader,
-                                      written,
+                                      read,
                                       consumeQSize);
    }
 
-   return written;
+   return read;
 }
 
 
@@ -705,6 +766,7 @@ DequeueLocked(VMCIQueue *produceQ,                        // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_Enqueue)
 ssize_t
 VMCIQPair_Enqueue(VMCIQPair *qpair,        // IN
                   const void *buf,         // IN
@@ -713,6 +775,10 @@ VMCIQPair_Enqueue(VMCIQPair *qpair,        // IN
 {
    ssize_t result;
 
+   if (!qpair || !buf) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    VMCIQPairLock(qpair);
 
    result =  EnqueueLocked(qpair->produceQ,
@@ -744,6 +810,7 @@ VMCIQPair_Enqueue(VMCIQPair *qpair,        // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_Dequeue)
 ssize_t
 VMCIQPair_Dequeue(VMCIQPair *qpair,        // IN
                   void *buf,               // IN
@@ -752,6 +819,10 @@ VMCIQPair_Dequeue(VMCIQPair *qpair,        // IN
 {
    ssize_t result;
 
+   if (!qpair || !buf) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    VMCIQPairLock(qpair);
 
    result =  DequeueLocked(qpair->produceQ,
@@ -785,6 +856,7 @@ VMCIQPair_Dequeue(VMCIQPair *qpair,        // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_Peek)
 ssize_t
 VMCIQPair_Peek(VMCIQPair *qpair,    // IN
                void *buf,           // IN
@@ -793,6 +865,10 @@ VMCIQPair_Peek(VMCIQPair *qpair,    // IN
 {
    ssize_t result;
 
+   if (!qpair || !buf) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    VMCIQPairLock(qpair);
 
    result =  DequeueLocked(qpair->produceQ,
@@ -807,15 +883,10 @@ VMCIQPair_Peek(VMCIQPair *qpair,    // IN
    return result;
 }
 
-#if defined __linux__ && !defined VMKERNEL
-EXPORT_SYMBOL(VMCIQPair_Enqueue);
-EXPORT_SYMBOL(VMCIQPair_Dequeue);
-EXPORT_SYMBOL(VMCIQPair_Peek);
-#endif
-
 
 #if defined (SOLARIS) || (defined(__APPLE__) && !defined (VMX86_TOOLS)) || \
-    (defined(__linux__) && defined(__KERNEL__))
+    (defined(__linux__) && defined(__KERNEL__)) || \
+    (defined(_WIN32) && defined(WINNT_DDK))
 
 /*
  *-----------------------------------------------------------------------------
@@ -834,13 +905,18 @@ EXPORT_SYMBOL(VMCIQPair_Peek);
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_EnqueueV)
 ssize_t
 VMCIQPair_EnqueueV(VMCIQPair *qpair,        // IN
                    void *iov,               // IN
                    size_t iovSize,          // IN
                    int bufType)             // IN
 {
-   int64 result;
+   ssize_t result;
+
+   if (!qpair || !iov) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
 
    VMCIQPairLock(qpair);
 
@@ -873,16 +949,21 @@ VMCIQPair_EnqueueV(VMCIQPair *qpair,        // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_DequeueV)
 ssize_t
 VMCIQPair_DequeueV(VMCIQPair *qpair,         // IN
                    void *iov,                // IN
                    size_t iovSize,           // IN
                    int bufType)              // IN
 {
-   int64 result;
+   ssize_t result;
 
    VMCIQPairLock(qpair);
 
+   if (!qpair || !iov) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    result =  DequeueLocked(qpair->produceQ,
                            qpair->consumeQ,
                            qpair->consumeQSize,
@@ -914,13 +995,18 @@ VMCIQPair_DequeueV(VMCIQPair *qpair,         // IN
  *-----------------------------------------------------------------------------
  */
 
+VMCI_EXPORT_SYMBOL(VMCIQPair_PeekV)
 ssize_t
 VMCIQPair_PeekV(VMCIQPair *qpair,           // IN
                 void *iov,                  // IN
                 size_t iovSize,             // IN
                 int bufType)                // IN
 {
-   int64 result;
+   ssize_t result;
+
+   if (!qpair || !iov) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
 
    VMCIQPairLock(qpair);
 
@@ -936,11 +1022,4 @@ VMCIQPair_PeekV(VMCIQPair *qpair,           // IN
    return result;
 }
 
-#if defined __linux__ && !defined VMKERNEL
-EXPORT_SYMBOL(VMCIQPair_EnqueueV);
-EXPORT_SYMBOL(VMCIQPair_DequeueV);
-EXPORT_SYMBOL(VMCIQPair_PeekV);
-#endif
-
 #endif /* Systems that support struct iovec */
-
