@@ -33,11 +33,14 @@
 #endif /* __linux__ */
 
 #include "vm_assert.h"
+#include "vm_atomic.h"
+#include "vmci_handle_array.h"
 #include "vmci_kernel_if.h"
+#include "vmciEvent.h"
+#include "vmciInt.h"
+#include "vmciKernelAPI.h"
 #include "vmciQueuePairInt.h"
 #include "vmciUtil.h"
-#include "vmciInt.h"
-#include "vmciEvent.h"
 #include "circList.h"
 
 #define LGPFX "VMCIQueuePair: "
@@ -53,15 +56,19 @@ typedef struct QueuePairEntry {
    void      *produceQ;
    void      *consumeQ;
    uint32     refCount;
+   Bool       hibernateFailure;
    ListItem   listItem;
 } QueuePairEntry;
 
 typedef struct QueuePairList {
-   ListItem  *head;
-   VMCIMutex mutex;
+   ListItem      *head;
+   Atomic_uint32  hibernate;
+   VMCIMutex      mutex;
 } QueuePairList;
 
 static QueuePairList queuePairList;
+static VMCIHandleArray *hibernateFailedList;
+static VMCILock hibernateFailedListLock;
 
 static QueuePairEntry *QueuePairList_FindEntry(VMCIHandle handle);
 static void QueuePairList_AddEntry(QueuePairEntry *entry);
@@ -79,7 +86,10 @@ static int VMCIQueuePairAllocHelper(VMCIHandle *handle, VMCIQueue **produceQ,
                                     uint64 consumeSize,
                                     VMCIId peer, uint32 flags);
 static int VMCIQueuePairDetachHelper(VMCIHandle handle);
+static int VMCIQueuePairDetachHyperCall(VMCIHandle handle);
 static int QueuePairNotifyPeerLocal(Bool attach, VMCIHandle handle);
+static void VMCIQPMarkHibernateFailed(QueuePairEntry *entry);
+static void VMCIQPUnmarkHibernateFailed(QueuePairEntry *entry);
 
 
 /*
@@ -194,7 +204,19 @@ void
 VMCIQueuePair_Init(void)
 {
    queuePairList.head = NULL;
+   Atomic_Write(&queuePairList.hibernate, 0);
    QueuePairLock_Init();
+   hibernateFailedList = VMCIHandleArray_Create(0);
+
+   /*
+    * The lock rank must be lower than subscriberLock in vmciEvent,
+    * since we hold the hibernateFailedListLock while generating
+    * detach events.
+    */
+
+   VMCI_InitLock(&hibernateFailedListLock,
+                 "VMCIQPHibernateFailed",
+                 VMCI_LOCK_RANK_MIDDLE_BH);
 }
 
 
@@ -244,8 +266,36 @@ VMCIQueuePair_Exit(void)
       QueuePairEntryDestroy(entry);
    }
 
+   Atomic_Write(&queuePairList.hibernate, 0);
    QueuePairList_Unlock();
    QueuePairLock_Destroy();
+   VMCI_CleanupLock(&hibernateFailedListLock);
+   VMCIHandleArray_Destroy(hibernateFailedList);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQueuePair_Sync --
+ *
+ *      Use this as a synchronization point when setting globals, for example,
+ *      during device shutdown.
+ *
+ * Results:
+ *      TRUE.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+VMCIQueuePair_Sync(void)
+{
+   QueuePairList_Lock();
+   QueuePairList_Unlock();
 }
 
 
@@ -683,11 +733,31 @@ VMCIQueuePairAllocHelper(VMCIHandle *handle,   // IN/OUT:
 
    QueuePairList_Lock();
 
+   /* Do not allow alloc/attach if the device is being shutdown. */
+   if (VMCI_DeviceShutdown()) {
+      result = VMCI_ERROR_DEVICE_NOT_FOUND;
+      goto error;
+   }
+
+   if ((Atomic_Read(&queuePairList.hibernate) == 1) &&
+       !(flags & VMCI_QPFLAG_LOCAL)) {
+      /*
+       * While guest OS is in hibernate state, creating non-local
+       * queue pairs is not allowed after the point where the VMCI
+       * guest driver converted the existing queue pairs to local
+       * ones.
+       */
+
+      result = VMCI_ERROR_UNAVAILABLE;
+      goto error;
+   }
+
    if ((queuePairEntry = QueuePairList_FindEntry(*handle))) {
       if (queuePairEntry->flags & VMCI_QPFLAG_LOCAL) {
          /* Local attach case. */
          if (queuePairEntry->refCount > 1) {
-            VMCI_LOG((LGPFX "Error attempting to attach more than once.\n"));
+            VMCI_DEBUG_LOG(4, (LGPFX"Error attempting to attach more than "
+                               "once.\n"));
             result = VMCI_ERROR_UNAVAILABLE;
             goto errorKeepEntry;
          }
@@ -695,7 +765,8 @@ VMCIQueuePairAllocHelper(VMCIHandle *handle,   // IN/OUT:
          if (queuePairEntry->produceSize != consumeSize ||
              queuePairEntry->consumeSize != produceSize ||
              queuePairEntry->flags != (flags & ~VMCI_QPFLAG_ATTACH_ONLY)) {
-            VMCI_LOG((LGPFX "Error mismatched queue pair in local attach.\n"));
+            VMCI_DEBUG_LOG(4, (LGPFX"Error mismatched queue pair in local "
+                               "attach.\n"));
             result = VMCI_ERROR_QUEUEPAIR_MISMATCH;
             goto errorKeepEntry;
          }
@@ -718,14 +789,14 @@ VMCIQueuePairAllocHelper(VMCIHandle *handle,   // IN/OUT:
 
    myProduceQ = VMCI_AllocQueue(produceSize);
    if (!myProduceQ) {
-      VMCI_LOG((LGPFX "Error allocating pages for produce queue.\n"));
+      VMCI_WARNING((LGPFX"Error allocating pages for produce queue.\n"));
       result = VMCI_ERROR_NO_MEM;
       goto error;
    }
 
    myConsumeQ = VMCI_AllocQueue(consumeSize);
    if (!myConsumeQ) {
-      VMCI_LOG((LGPFX "Error allocating pages for consume queue.\n"));
+      VMCI_WARNING((LGPFX"Error allocating pages for consume queue.\n"));
       result = VMCI_ERROR_NO_MEM;
       goto error;
    }
@@ -734,7 +805,7 @@ VMCIQueuePairAllocHelper(VMCIHandle *handle,   // IN/OUT:
                                          produceSize, consumeSize,
                                          myProduceQ, myConsumeQ);
    if (!queuePairEntry) {
-      VMCI_LOG((LGPFX "Error allocating memory in %s.\n", __FUNCTION__));
+      VMCI_WARNING((LGPFX"Error allocating memory in %s.\n", __FUNCTION__));
       result = VMCI_ERROR_NO_MEM;
       goto error;
    }
@@ -742,7 +813,7 @@ VMCIQueuePairAllocHelper(VMCIHandle *handle,   // IN/OUT:
    result = VMCI_AllocPPNSet(myProduceQ, numProducePages, myConsumeQ,
                              numConsumePages, &queuePairEntry->ppnSet);
    if (result < VMCI_SUCCESS) {
-      VMCI_LOG((LGPFX "VMCI_AllocPPNSet failed.\n"));
+      VMCI_WARNING((LGPFX"VMCI_AllocPPNSet failed.\n"));
       goto error;
    }
 
@@ -775,12 +846,14 @@ VMCIQueuePairAllocHelper(VMCIHandle *handle,   // IN/OUT:
    } else {
       result = VMCIQueuePairAlloc_HyperCall(queuePairEntry);
       if (result < VMCI_SUCCESS) {
-         VMCI_LOG((LGPFX "VMCIQueuePairAlloc_HyperCall result = %d.\n",
-                   result));
+         VMCI_WARNING((LGPFX"VMCIQueuePairAlloc_HyperCall result = %d.\n",
+                       result));
          goto error;
       }
    }
 
+   VMCI_InitQueueMutex((VMCIQueue *)myProduceQ, (VMCIQueue *)myConsumeQ);
+
    QueuePairList_AddEntry(queuePairEntry);
 
 out:
@@ -827,6 +900,37 @@ errorKeepEntry:
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQueuePairDetachHyperCall --
+ *
+ *      Helper to make a QueuePairDetach hypercall.
+ *
+ * Results:
+ *      Result of the hypercall.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+int
+VMCIQueuePairDetachHyperCall(VMCIHandle handle) // IN:
+{
+   VMCIQueuePairDetachMsg detachMsg;
+
+   detachMsg.hdr.dst = VMCI_MAKE_HANDLE(VMCI_HYPERVISOR_CONTEXT_ID,
+                                        VMCI_QUEUEPAIR_DETACH);
+   detachMsg.hdr.src = VMCI_ANON_SRC_HANDLE;
+   detachMsg.hdr.payloadSize = sizeof handle;
+   detachMsg.handle = handle;
+
+   return VMCI_SendDatagram((VMCIDatagram *)&detachMsg);
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -873,15 +977,27 @@ VMCIQueuePairDetachHelper(VMCIHandle handle)   // IN:
          }
       }
    } else {
-      VMCIQueuePairDetachMsg detachMsg;
+      result = VMCIQueuePairDetachHyperCall(handle);
+      if (entry->hibernateFailure) {
+         if (result == VMCI_ERROR_NOT_FOUND) {
+            /*
+             * If a queue pair detach failed when entering
+             * hibernation, the guest driver and the device may
+             * disagree on its existence when coming out of
+             * hibernation. The guest driver will regard it as a
+             * non-local queue pair, but the device state is gone,
+             * since the device has been powered off. In this case, we
+             * treat the queue pair as a local queue pair with no
+             * peer.
+             */
 
-      detachMsg.hdr.dst = VMCI_MAKE_HANDLE(VMCI_HYPERVISOR_CONTEXT_ID,
-                                           VMCI_QUEUEPAIR_DETACH);
-      detachMsg.hdr.src = VMCI_ANON_SRC_HANDLE;
-      detachMsg.hdr.payloadSize = sizeof handle;
-      detachMsg.handle = handle;
-
-      result = VMCI_SendDatagram((VMCIDatagram *)&detachMsg);
+            ASSERT(entry->refCount == 1);
+            result = VMCI_SUCCESS;
+         }
+         if (result == VMCI_SUCCESS) {
+            VMCIQPUnmarkHibernateFailed(entry);
+         }
+      }
    }
 
 out:
@@ -952,3 +1068,242 @@ QueuePairNotifyPeerLocal(Bool attach,           // IN: attach or detach?
 
    return VMCIEvent_Dispatch((VMCIDatagram *)eMsg);
 }
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPMarkHibernateFailed --
+ *
+ *      Helper function that marks a queue pair entry as not being
+ *      converted to a local version during hibernation. Must be
+ *      called with the queue pair list lock held.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static void
+VMCIQPMarkHibernateFailed(QueuePairEntry *entry) // IN
+{
+   VMCILockFlags flags;
+   VMCIHandle handle;
+
+   /*
+    * entry->handle is located in paged memory, so it can't be
+    * accessed while holding a spinlock.
+    */
+
+   handle = entry->handle;
+   entry->hibernateFailure = TRUE;
+   VMCI_GrabLock_BH(&hibernateFailedListLock, &flags);
+   VMCIHandleArray_AppendEntry(&hibernateFailedList, handle);
+   VMCI_ReleaseLock_BH(&hibernateFailedListLock, flags);
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCIQPUnmarkHibernateFailed --
+ *
+ *      Helper function that removes a queue pair entry from the group
+ *      of handles marked as having failed hibernation. Must be called
+ *      with the queue pair list lock held.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static void
+VMCIQPUnmarkHibernateFailed(QueuePairEntry *entry) // IN
+{
+   VMCILockFlags flags;
+   VMCIHandle handle;
+
+   /*
+    * entry->handle is located in paged memory, so it can't be
+    * accessed while holding a spinlock.
+    */
+
+   handle = entry->handle;
+   entry->hibernateFailure = FALSE;
+   VMCI_GrabLock_BH(&hibernateFailedListLock, &flags);
+   VMCIHandleArray_RemoveEntry(hibernateFailedList, handle);
+   VMCI_ReleaseLock_BH(&hibernateFailedListLock, flags);
+}
+
+
+/*
+ *----------------------------------------------------------------------------
+ *
+ * VMCIQueuePair_Convert --
+ *
+ *      Queue pairs may be converted to local ones in two cases: when
+ *      entering hibernation or when the device is powered off before
+ *      entering a sleep mode. Below we first discuss the case of
+ *      hibernation and then the case of entering sleep state.
+ *
+ *      When the guest enters hibernation, any non-local queue pairs
+ *      will disconnect no later than at the time the VMCI device
+ *      powers off. To preserve the content of the non-local queue
+ *      pairs for this guest, we make a local copy of the content and
+ *      disconnect from the queue pairs. This will ensure that the
+ *      peer doesn't continue to update the queue pair state while the
+ *      guest OS is checkpointing the memory (otherwise we might end
+ *      up with a inconsistent snapshot where the pointers of the
+ *      consume queue are checkpointed later than the data pages they
+ *      point to, possibly indicating that non-valid data is
+ *      valid). While we are in hibernation mode, we block the
+ *      allocation of new non-local queue pairs. Note that while we
+ *      are doing the conversion to local queue pairs, we are holding
+ *      the queue pair list lock, which will prevent concurrent
+ *      creation of additional non-local queue pairs.
+ *
+ *      The hibernation cannot fail, so if we are unable to either
+ *      save the queue pair state or detach from a queue pair, we deal
+ *      with it by keeping the queue pair around, and converting it to
+ *      a local queue pair when going out of hibernation. Since
+ *      failing a detach is highly unlikely (it would require a queue
+ *      pair being actively used as part of a DMA operation), this is
+ *      an acceptable fall back. Once we come back from hibernation,
+ *      these queue pairs will no longer be external, so we simply
+ *      mark them as local at that point.
+ *
+ *      For the sleep state, the VMCI device will also be put into the
+ *      D3 power state, which may make the device inaccessible to the
+ *      guest driver (Windows unmaps the I/O space). When entering
+ *      sleep state, the hypervisor is likely to suspend the guest as
+ *      well, which will again convert all queue pairs to local ones.
+ *      However, VMCI device clients, e.g., VMCI Sockets, may attempt
+ *      to use queue pairs after the device has been put into the D3
+ *      power state, so we convert the queue pairs to local ones in
+ *      that case as well. When exiting the sleep states, the device
+ *      has not been reset, so all device state is still in sync with
+ *      the device driver, so no further processing is necessary at
+ *      that point.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Queue pairs are detached.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+void
+VMCIQueuePair_Convert(Bool toLocal,     // IN
+                      Bool deviceReset) // IN
+{
+   if (toLocal) {
+      ListItem *next;
+
+      QueuePairList_Lock();
+
+      LIST_SCAN(next, queuePairList.head) {
+         QueuePairEntry *entry = LIST_CONTAINER(next, QueuePairEntry, listItem);
+
+         if (!(entry->flags & VMCI_QPFLAG_LOCAL)) {
+            VMCIQueue *prodQ;
+            VMCIQueue *consQ;
+            void *oldProdQ;
+            void *oldConsQ;
+            int result;
+
+            prodQ = (VMCIQueue *)entry->produceQ;
+            consQ = (VMCIQueue *)entry->consumeQ;
+            oldConsQ = oldProdQ = NULL;
+
+            VMCI_AcquireQueueMutex(prodQ);
+
+            result = VMCI_ConvertToLocalQueue(consQ, prodQ, entry->consumeSize,
+                                              TRUE, &oldConsQ);
+            if (result != VMCI_SUCCESS) {
+               VMCI_WARNING((LGPFX"Hibernate failed to create local consume "
+                             "queue from handle %x:%x (error: %d)\n",
+                             entry->handle.context, entry->handle.resource,
+                             result));
+               VMCI_ReleaseQueueMutex(prodQ);
+               VMCIQPMarkHibernateFailed(entry);
+               continue;
+            }
+            result = VMCI_ConvertToLocalQueue(prodQ, consQ, entry->produceSize,
+                                              FALSE, &oldProdQ);
+            if (result != VMCI_SUCCESS) {
+               VMCI_WARNING((LGPFX"Hibernate failed to create local produce "
+                             "queue from handle %x:%x (error: %d)\n",
+                             entry->handle.context, entry->handle.resource,
+                             result));
+               VMCI_RevertToNonLocalQueue(consQ, oldConsQ, entry->consumeSize);
+               VMCI_ReleaseQueueMutex(prodQ);
+               VMCIQPMarkHibernateFailed(entry);
+               continue;
+            }
+
+            /*
+             * Now that the contents of the queue pair has been saved,
+             * we can detach from the non-local queue pair. This will
+             * discard the content of the non-local queues.
+             */
+
+            result = VMCIQueuePairDetachHyperCall(entry->handle);
+            if (result < VMCI_SUCCESS) {
+               VMCI_WARNING((LGPFX"Hibernate failed to detach from handle "
+                             "%x:%x\n",
+                             entry->handle.context, entry->handle.resource));
+               VMCI_RevertToNonLocalQueue(consQ, oldConsQ, entry->consumeSize);
+               VMCI_RevertToNonLocalQueue(prodQ, oldProdQ, entry->produceSize);
+               VMCI_ReleaseQueueMutex(prodQ);
+               VMCIQPMarkHibernateFailed(entry);
+               continue;
+            }
+
+            entry->flags |= VMCI_QPFLAG_LOCAL;
+
+            VMCI_ReleaseQueueMutex(prodQ);
+
+            VMCI_FreeQueueBuffer(oldProdQ, entry->produceSize);
+            VMCI_FreeQueueBuffer(oldConsQ, entry->consumeSize);
+
+            QueuePairNotifyPeerLocal(FALSE, entry->handle);
+         }
+      }
+      Atomic_Write(&queuePairList.hibernate, 1);
+
+      QueuePairList_Unlock();
+   } else {
+      VMCILockFlags flags;
+      VMCIHandle handle;
+
+      /*
+       * When a guest enters hibernation, there may be queue pairs
+       * around, that couldn't be converted to local queue
+       * pairs. When coming out of hibernation, these queue pairs
+       * will be restored as part of the guest main mem by the OS
+       * hibernation code and they can now be regarded as local
+       * versions. Since they are no longer connected, detach
+       * notifications are sent to the local endpoint.
+       */
+
+      VMCI_GrabLock_BH(&hibernateFailedListLock, &flags);
+      while (VMCIHandleArray_GetSize(hibernateFailedList) > 0) {
+         handle = VMCIHandleArray_RemoveTail(hibernateFailedList);
+         if (deviceReset) {
+            QueuePairNotifyPeerLocal(FALSE, handle);
+         }
+      }
+      VMCI_ReleaseLock_BH(&hibernateFailedListLock, flags);
+
+      Atomic_Write(&queuePairList.hibernate, 0);
+   }
+}
