@@ -55,7 +55,6 @@
 #include "vmci_kernel_if.h"
 #include "vmciGuestKernelIf.h"
 #include "vmciInt.h"
-#include "vmciProcess.h"
 #include "vmciDatagram.h"
 #include "vmciUtil.h"
 #include "vmciEvent.h"
@@ -94,8 +93,8 @@ VMCIUtil_Init(void)
    if (VMCIEvent_Subscribe(VMCI_EVENT_CTX_ID_UPDATE, VMCI_FLAG_EVENT_NONE,
                            VMCIUtilCidUpdate, NULL,
                            &ctxUpdateSubID) < VMCI_SUCCESS) {
-      VMCI_LOG(("VMCIUtil: Failed to subscribe to event %d.\n",
-                VMCI_EVENT_CTX_ID_UPDATE));
+      VMCI_WARNING((LGPFX"Failed to subscribe to event %d.\n",
+                    VMCI_EVENT_CTX_ID_UPDATE));
    }
 }
 
@@ -120,8 +119,9 @@ void
 VMCIUtil_Exit(void)
 {
    if (VMCIEvent_Unsubscribe(ctxUpdateSubID) < VMCI_SUCCESS) {
-      VMCI_LOG(("VMCIUtil: Failed to unsubscribe to event %d with subscriber "
-                "id %d.\n", VMCI_EVENT_CTX_ID_UPDATE, ctxUpdateSubID));
+      VMCI_WARNING((LGPFX"Failed to unsubscribe to event %d with "
+                    "subscriber id %d.\n", VMCI_EVENT_CTX_ID_UPDATE,
+                    ctxUpdateSubID));
    }
 }
 
@@ -150,14 +150,14 @@ VMCIUtilCidUpdate(VMCIId subID,               // IN:
    VMCIEventPayload_Context *evPayload = VMCIEventDataPayload(eventData);
 
    if (subID != ctxUpdateSubID) {
-      VMCI_LOG(("VMCIUtil: Invalid subscriber id. %d.\n", subID));
+      VMCI_WARNING((LGPFX"Invalid subscriber id. %d.\n", subID));
       return;
    }
    if (eventData == NULL || evPayload->contextID == VMCI_INVALID_ID) {
-      VMCI_LOG(("VMCIUtil: Invalid event data.\n"));
+      VMCI_WARNING((LGPFX"Invalid event data.\n"));
       return;
    }
-   VMCI_LOG(("VMCIUtil: Updating context id from 0x%x to 0x%x on event %d.\n",
+   VMCI_LOG((LGPFX"Updating context id from 0x%x to 0x%x on event %d.\n",
              Atomic_Read(&vmContextID),
              evPayload->contextID,
              eventData->event));
@@ -196,7 +196,7 @@ VMCIUtil_CheckHostCapabilities(void)
    VMCIDatagram *checkMsg = VMCI_AllocKernelMem(msgSize, VMCI_MEMORY_NONPAGED);
 
    if (checkMsg == NULL) {
-      VMCI_LOG((LGPFX"Check host: Insufficient memory.\n"));
+      VMCI_WARNING((LGPFX"Check host: Insufficient memory.\n"));
       return FALSE;
    }
 
@@ -233,10 +233,7 @@ VMCIUtil_CheckHostCapabilities(void)
  *-----------------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCI_GetContextID);
-#endif
-
+VMCI_EXPORT_SYMBOL(VMCI_GetContextID)
 VMCIId
 VMCI_GetContextID(void)
 {
@@ -277,7 +274,6 @@ Bool
 VMCI_CheckHostCapabilities(void)
 {
    Bool result = VMCIEvent_CheckHostCapabilities();
-   result &= VMCIProcess_CheckHostCapabilities();
    result &= VMCIDatagram_CheckHostCapabilities();
    result &= VMCIUtil_CheckHostCapabilities();
 
@@ -302,10 +298,7 @@ VMCI_CheckHostCapabilities(void)
  *----------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCI_Version);
-#endif
-
+VMCI_EXPORT_SYMBOL(VMCI_Version)
 uint32
 VMCI_Version()
 {
@@ -366,13 +359,14 @@ VMCI_InInterrupt()
  *----------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCI_DeviceGet);
-#endif
-
+VMCI_EXPORT_SYMBOL(VMCI_DeviceGet)
 Bool
-VMCI_DeviceGet(void)
+VMCI_DeviceGet(uint32 *apiVersion)
 {
+   if (*apiVersion > VMCI_KERNEL_API_VERSION) {
+      *apiVersion = VMCI_KERNEL_API_VERSION;
+      return FALSE;
+   }
    return VMCI_DeviceEnabled();
 }
 
@@ -393,10 +387,7 @@ VMCI_DeviceGet(void)
  *----------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCI_DeviceRelease);
-#endif
-
+VMCI_EXPORT_SYMBOL(VMCI_DeviceRelease)
 void
 VMCI_DeviceRelease(void)
 {
@@ -503,8 +494,8 @@ VMCI_ReadDatagramsFromPort(VMCIIoHandle ioHandle,  // IN
             result = VMCIDatagram_Dispatch(dg->src.context, dg);
          }
          if (result < VMCI_SUCCESS) {
-            VMCI_LOG(("Datagram with resource %d failed with err %x.\n",
-                      dg->dst.resource, result));
+            VMCI_DEBUG_LOG(4, (LGPFX"Datagram with resource %d failed with "
+                               "err %x.\n", dg->dst.resource, result));
          }
 
          /* On to the next datagram. */
@@ -516,8 +507,8 @@ VMCI_ReadDatagramsFromPort(VMCIIoHandle ioHandle,  // IN
           * Datagram doesn't fit in datagram buffer of maximal size. We drop it.
           */
 
-         VMCI_LOG(("Failed to receive datagram of size %u.\n",
-                   dgInSize));
+         VMCI_DEBUG_LOG(4, (LGPFX"Failed to receive datagram of size %u.\n",
+                            dgInSize));
 
          bytesToSkip = dgInSize - remainingBytes;
          if (currentDgInBufferSize != dgInBufferSize) {
@@ -562,10 +553,7 @@ VMCI_ReadDatagramsFromPort(VMCIIoHandle ioHandle,  // IN
  *----------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCIContext_GetPrivFlags);
-#endif
-
+VMCI_EXPORT_SYMBOL(VMCIContext_GetPrivFlags)
 VMCIPrivilegeFlags
 VMCIContext_GetPrivFlags(VMCIId contextID) // IN
 {
@@ -589,10 +577,7 @@ VMCIContext_GetPrivFlags(VMCIId contextID) // IN
  *----------------------------------------------------------------------
  */
 
-#ifdef __linux__
-EXPORT_SYMBOL(VMCI_ContextID2HostVmID);
-#endif
-
+VMCI_EXPORT_SYMBOL(VMCI_ContextID2HostVmID)
 int
 VMCI_ContextID2HostVmID(VMCIId contextID,    // IN
                         void *hostVmID,      // OUT
