@@ -27,33 +27,41 @@
 #include <linux/moduleparam.h>
 #include <linux/poll.h>
 
+#include "compat_init.h"
+#include "compat_interrupt.h"
+#include "compat_ioport.h"
 #include "compat_kernel.h"
 #include "compat_module.h"
-#include "compat_pci.h"
-#include "compat_init.h"
-#include "compat_ioport.h"
-#include "compat_interrupt.h"
-#include "compat_page.h"
 #include "compat_mutex.h"
+#include "compat_page.h"
+#include "compat_pci.h"
+
+#include "driverLog.h"
+
 #include "vm_basic_types.h"
 #include "vm_device_version.h"
-#include "kernelStubs.h"
-#include "vmci_iocontrols.h"
+
 #include "vmci_defs.h"
-#include "vmciInt.h"
 #include "vmci_infrastructure.h"
+#include "vmci_iocontrols.h"
+#include "vmci_version.h"
 #include "vmciDatagram.h"
-#include "vmciProcess.h"
-#include "vmciUtil.h"
 #include "vmciEvent.h"
+#include "vmciInt.h"
 #include "vmciNotifications.h"
 #include "vmciQueuePairInt.h"
-#include "vmci_version.h"
-#include "vmciCommonInt.h"
+#include "vmciUtil.h"
 
 #define LGPFX "VMCI: "
 #define VMCI_DEVICE_MINOR_NUM 0
 
+/* MSI-X has performance problems in < 2.6.19 */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
+#  define VMCI_DISABLE_MSIX   0
+#else
+#  define VMCI_DISABLE_MSIX   1
+#endif
+
 typedef struct vmci_device {
    compat_mutex_t    lock;
 
@@ -73,8 +81,11 @@ static int vmci_probe_device(struct pci_dev *pdev,
 static void vmci_remove_device(struct pci_dev* pdev);
 static int vmci_open(struct inode *inode, struct file *file);
 static int vmci_close(struct inode *inode, struct file *file);
-static int vmci_ioctl(struct inode *inode, struct file *file,
-                      unsigned int cmd, unsigned long arg);
+static long vmci_ioctl(struct file *file, unsigned int cmd, unsigned long arg);
+#if !defined(HAVE_UNLOCKED_IOCTL)
+static int vmci_legacy_ioctl(struct inode *dummy, struct file *file,
+                             unsigned int cmd, unsigned long arg);
+#endif
 static unsigned int vmci_poll(struct file *file, poll_table *wait);
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 static compat_irqreturn_t vmci_interrupt(int irq, void *dev_id,
@@ -97,7 +108,14 @@ static struct file_operations vmci_ops = {
    .owner   = THIS_MODULE,
    .open    = vmci_open,
    .release = vmci_close,
-   .ioctl   = vmci_ioctl,
+#ifdef HAVE_UNLOCKED_IOCTL
+   .unlocked_ioctl = vmci_ioctl,
+#else
+   .ioctl   = vmci_legacy_ioctl,
+#endif
+#ifdef HAVE_COMPAT_IOCTL
+   .compat_ioctl = vmci_ioctl,
+#endif
    .poll    = vmci_poll,
 };
 
@@ -105,7 +123,7 @@ static struct pci_driver vmci_driver = {
    .name     = "vmci",
    .id_table = vmci_ids,
    .probe = vmci_probe_device,
-   .remove = vmci_remove_device,
+   .remove = __devexit_p(vmci_remove_device),
 };
 
 static vmci_device vmci_dev;
@@ -113,6 +131,9 @@ static vmci_device vmci_dev;
 /* We dynamically request the device major number at init time. */
 static int device_major_nr = 0;
 
+static int vmci_disable_msi;
+static int vmci_disable_msix = VMCI_DISABLE_MSIX;
+
 DECLARE_TASKLET(vmci_dg_tasklet, dispatch_datagrams,
                 (unsigned long)&vmci_dev);
 
@@ -151,20 +172,12 @@ static uint8 *notification_bitmap = NULL;
  *-----------------------------------------------------------------------------
  */
 
-static int
+static int __init
 vmci_init(void)
 {
-   int err = -ENOMEM;
+   int err;
 
-   /* Register device node ops. */
-   err = register_chrdev(0, "vmci", &vmci_ops);
-   if (err < 0) {
-      printk(KERN_ERR "Unable to register vmci device\n");
-      return err;
-   }
-   device_major_nr = err;
-
-   printk("VMCI: Major device number is: %d\n", device_major_nr);
+   DriverLog_Init("/dev/vmci");
 
    /* Initialize device data. */
    compat_mutex_init(&vmci_dev.lock);
@@ -174,20 +187,32 @@ vmci_init(void)
    vmci_dev.enabled = FALSE;
 
    data_buffer = vmalloc(data_buffer_size);
-   if (data_buffer == NULL) {
-      goto error;
+   if (!data_buffer) {
+      return -ENOMEM;
    }
 
+   /* Register device node ops. */
+   err = register_chrdev(0, "vmci", &vmci_ops);
+   if (err < 0) {
+      printk(KERN_ERR "Unable to register vmci device\n");
+      goto err_free_mem;
+   }
+
+   device_major_nr = err;
+
+   printk("VMCI: Major device number is: %d\n", device_major_nr);
+
    /* This should be last to make sure we are done initializing. */
    err = pci_register_driver(&vmci_driver);
    if (err < 0) {
-      goto error;
+      goto err_unregister_chrdev;
    }
 
    return 0;
 
-error:
+err_unregister_chrdev:
    unregister_chrdev(device_major_nr, "vmci");
+err_free_mem:
    vfree(data_buffer);
    return err;
 }
@@ -209,7 +234,7 @@ error:
  *-----------------------------------------------------------------------------
  */
 
-static void
+static void __exit
 vmci_exit(void)
 {
    pci_unregister_driver(&vmci_driver);
@@ -273,7 +298,7 @@ vmci_enable_msix(struct pci_dev *pdev) // IN
  *-----------------------------------------------------------------------------
  */
 
-static int
+static int __devinit
 vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
                   const struct pci_device_id *id) // IN: matching device ID
 {
@@ -284,15 +309,15 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
 
    printk(KERN_INFO "Probing for vmci/PCI.\n");
 
-   result = compat_pci_enable_device(pdev);
+   result = pci_enable_device(pdev);
    if (result) {
       printk(KERN_ERR "Cannot VMCI device %s: error %d\n",
-             compat_pci_name(pdev), result);
+             pci_name(pdev), result);
       return result;
    }
-   compat_pci_set_master(pdev); /* To enable QueuePair functionality. */
-   ioaddr = compat_pci_resource_start(pdev, 0);
-   ioaddr_size = compat_pci_resource_len(pdev, 0);
+   pci_set_master(pdev); /* To enable QueuePair functionality. */
+   ioaddr = pci_resource_start(pdev, 0);
+   ioaddr_size = pci_resource_len(pdev, 0);
 
    /*
     * Request I/O region with adjusted base address and size. The adjusted
@@ -301,7 +326,7 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
 
    if (!compat_request_region(ioaddr, ioaddr_size, "vmci")) {
       printk(KERN_INFO "vmci: Another driver already loaded "
-                       "for device in slot %s.\n", compat_pci_name(pdev));
+                       "for device in slot %s.\n", pci_name(pdev));
       goto pci_disable;
    }
 
@@ -387,7 +412,6 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
     * components, and it may be invoked once request_irq() has
     * registered the handler (as the irq line may be shared).
     */
-   VMCIProcess_Init();
    VMCIDatagram_Init();
    VMCIEvent_Init();
    VMCIUtil_Init();
@@ -398,10 +422,10 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
     * Enable interrupts.  Try MSI-X first, then MSI, and then fallback on
     * legacy interrupts.
     */
-   if (!vmci_enable_msix(pdev)) {
+   if (!vmci_disable_msix && !vmci_enable_msix(pdev)) {
       vmci_dev.intr_type = VMCI_INTR_TYPE_MSIX;
       vmci_dev.irq = vmci_dev.msix_entries[0].vector;
-   } else if (!pci_enable_msi(pdev)) {
+   } else if (!vmci_disable_msi && !pci_enable_msi(pdev)) {
       vmci_dev.intr_type = VMCI_INTR_TYPE_MSI;
       vmci_dev.irq = pdev->irq;
    } else {
@@ -457,7 +481,6 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
    VMCINotifications_Exit();
    VMCIUtil_Exit();
    VMCIEvent_Exit();
-   VMCIProcess_Exit();
    if (vmci_dev.intr_type == VMCI_INTR_TYPE_MSIX) {
       pci_disable_msix(pdev);
    } else if (vmci_dev.intr_type == VMCI_INTR_TYPE_MSI) {
@@ -475,7 +498,7 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
    }
    release_region(ioaddr, ioaddr_size);
  pci_disable:
-   compat_pci_disable_device(pdev);
+   pci_disable_device(pdev);
    return -EBUSY;
 }
 
@@ -496,7 +519,7 @@ vmci_probe_device(struct pci_dev *pdev,           // IN: vmci PCI device
  *-----------------------------------------------------------------------------
  */
 
-static void
+static void __devexit
 vmci_remove_device(struct pci_dev* pdev)
 {
    struct vmci_device *dev = pci_get_drvdata(pdev);
@@ -509,7 +532,6 @@ vmci_remove_device(struct pci_dev* pdev)
    VMCIUtil_Exit();
    VMCIEvent_Exit();
    //VMCIDatagram_Exit();
-   VMCIProcess_Exit();
 
    compat_mutex_lock(&dev->lock);
    printk(KERN_INFO "Resetting vmci device\n");
@@ -542,7 +564,7 @@ vmci_remove_device(struct pci_dev* pdev)
    printk(KERN_INFO "Unregistered vmci device.\n");
    compat_mutex_unlock(&dev->lock);
 
-   compat_pci_disable_device(pdev);
+   pci_disable_device(pdev);
 }
 
 
@@ -627,11 +649,6 @@ vmci_close(struct inode *inode,  // IN
       (VMCIGuestDeviceHandle *) file->private_data;
 
    if (devHndl) {
-      if (devHndl->objType == VMCIOBJ_PROCESS) {
-         VMCIProcess_Destroy((VMCIProcess *) devHndl->obj);
-      } else if (devHndl->objType == VMCIOBJ_DATAGRAM_PROCESS) {
-         VMCIDatagramProcess_Destroy((VMCIDatagramProcess *) devHndl->obj);
-      }
       VMCI_FreeKernelMem(devHndl, sizeof *devHndl);
       file->private_data = NULL;
    }
@@ -655,184 +672,43 @@ vmci_close(struct inode *inode,  // IN
  *-----------------------------------------------------------------------------
  */
 
-static int
-vmci_ioctl(struct inode *inode,  // IN
-           struct file *file,    // IN
+static long
+vmci_ioctl(struct file *file,    // IN
            unsigned int cmd,     // IN
            unsigned long arg)    // IN
 {
-#ifndef VMX86_DEVEL
    return -ENOTTY;
-#else
-   int retval;
-   VMCIGuestDeviceHandle *devHndl =
-      (VMCIGuestDeviceHandle *) file->private_data;
-
-   if (devHndl == NULL) {
-      return -EINVAL;
-   }
-
-   switch (cmd) {
-   case IOCTL_VMCI_CREATE_PROCESS: {
-      if (devHndl->objType != VMCIOBJ_NOT_SET) {
-         printk("VMCI: Received IOCTLCMD_VMCI_CREATE_PROCESS on "
-                "initialized handle.\n");
-         retval = -EINVAL;
-         break;
-      }
-      ASSERT(!devHndl->obj);
-      retval = VMCIProcess_Create((VMCIProcess **) &devHndl->obj);
-      if (retval != 0) {
-         printk("VMCI: Failed to create process.\n");
-         break;
-      }
-      devHndl->objType = VMCIOBJ_PROCESS;
-      break;
-   }
-
-   case IOCTL_VMCI_CREATE_DATAGRAM_PROCESS: {
-      VMCIDatagramCreateProcessInfo createInfo;
-      VMCIDatagramProcess *dgmProc;
-
-      if (devHndl->objType != VMCIOBJ_NOT_SET) {
-         printk("VMCI: Received IOCTLCMD_VMCI_CREATE_DATAGRAM_PROCESS on "
-                "initialized handle.\n");
-         retval = -EINVAL;
-         break;
-      }
-      ASSERT(!devHndl->obj);
-
-      retval = copy_from_user(&createInfo, (void *)arg, sizeof createInfo);
-      if (retval != 0) {
-	 printk("VMCI: Error getting datagram create info, %d.\n", retval);
-	 retval = -EFAULT;
-	 break;
-      }
-
-      if (VMCIDatagramProcess_Create(&dgmProc, &createInfo,
-                                     0 /* Unused */) < VMCI_SUCCESS) {
-	 retval = -EINVAL;
-	 break;
-      }
-
-      retval = copy_to_user((void *)arg, &createInfo, sizeof createInfo);
-      if (retval != 0) {
-         VMCIDatagramProcess_Destroy(dgmProc);
-         printk("VMCI: Failed to create datagram process.\n");
-	 retval = -EFAULT;
-         break;
-      }
-      devHndl->obj = dgmProc;
-      devHndl->objType = VMCIOBJ_DATAGRAM_PROCESS;
-      break;
-   }
-
-   case IOCTL_VMCI_DATAGRAM_SEND: {
-      VMCIDatagramSendRecvInfo sendInfo;
-      VMCIDatagram *dg = NULL;
-
-      if (devHndl->objType != VMCIOBJ_DATAGRAM_PROCESS) {
-         printk("VMCI: Ioctl %d only valid for process datagram handle.\n",
-		cmd);
-         retval = -EINVAL;
-         break;
-      }
-
-      retval = copy_from_user(&sendInfo, (void *) arg, sizeof sendInfo);
-      if (retval) {
-         printk("VMCI: copy_from_user failed.\n");
-         retval = -EFAULT;
-         break;
-      }
-
-      if (sendInfo.len > VMCI_MAX_DG_SIZE) {
-         printk("VMCI: datagram size too big.\n");
-	 retval = -EINVAL;
-	 break;
-      }
-
-      dg = VMCI_AllocKernelMem(sendInfo.len, VMCI_MEMORY_NORMAL);
-      if (dg == NULL) {
-         printk("VMCI: Cannot allocate memory to dispatch datagram.\n");
-         retval = -ENOMEM;
-         break;
-      }
-
-      retval = copy_from_user(dg, (char *)(VA)sendInfo.addr, sendInfo.len);
-      if (retval != 0) {
-         printk("VMCI: Error getting datagram: %d\n", retval);
-         VMCI_FreeKernelMem(dg, sendInfo.len);
-         retval = -EFAULT;
-         break;
-      }
-
-      DEBUG_ONLY(printk("VMCI: Datagram dst handle 0x%x:0x%x, src handle "
-			"0x%x:0x%x, payload size %"FMT64"u.\n",
-			dg->dst.context, dg->dst.resource,
-			dg->src.context, dg->src.resource, dg->payloadSize));
-
-      sendInfo.result = VMCIDatagram_Send(dg);
-      VMCI_FreeKernelMem(dg, sendInfo.len);
-
-      retval = copy_to_user((void *)arg, &sendInfo, sizeof sendInfo);
-      break;
-   }
-
-   case IOCTL_VMCI_DATAGRAM_RECEIVE: {
-      VMCIDatagramSendRecvInfo recvInfo;
-      VMCIDatagram *dg = NULL;
-
-      if (devHndl->objType != VMCIOBJ_DATAGRAM_PROCESS) {
-         printk("VMCI: Ioctl %d only valid for process datagram handle.\n",
-		cmd);
-         retval = -EINVAL;
-         break;
-      }
-
-      retval = copy_from_user(&recvInfo, (void *) arg, sizeof recvInfo);
-      if (retval) {
-         printk("VMCI: copy_from_user failed.\n");
-         retval = -EFAULT;
-         break;
-      }
-
-      ASSERT(devHndl->obj);
-      recvInfo.result =
-	 VMCIDatagramProcess_ReadCall((VMCIDatagramProcess *)devHndl->obj,
-				      recvInfo.len, &dg);
-      if (recvInfo.result < VMCI_SUCCESS) {
-	 retval = -EINVAL;
-	 break;
-      }
-      ASSERT(dg);
-      retval = copy_to_user((void *) ((uintptr_t) recvInfo.addr), dg,
-			    VMCI_DG_SIZE(dg));
-      VMCI_FreeKernelMem(dg, VMCI_DG_SIZE(dg));
-      if (retval != 0) {
-	 break;
-      }
-      retval = copy_to_user((void *)arg, &recvInfo, sizeof recvInfo);
-      break;
-   }
-
-   case IOCTL_VMCI_GET_CONTEXT_ID: {
-      VMCIId cid = VMCI_GetContextID();
-
-      retval = copy_to_user((void *)arg, &cid, sizeof cid);
-      break;
-   }
- 
-   default:
-      printk(KERN_DEBUG "vmci_ioctl(): unknown ioctl 0x%x.\n", cmd);
-      retval = -EINVAL;
-      break;
-   }
-
-   return retval;
-#endif
 }
 
 
+#if !defined(HAVE_UNLOCKED_IOCTL)
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * vmci_legacy_ioctl --
+ *
+ *      IOCTL interface for kernels that do not have unlocked_ioctl.
+ *
+ * Results:
+ *      Negative error code, or per-ioctl result.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static int
+vmci_legacy_ioctl(struct inode *inode,  // IN: unused
+                  struct file *filp,    // IN:
+                  u_int iocmd,          // IN:
+                  unsigned long ioarg)  // IN:
+{
+   return vmci_ioctl(filp, iocmd, ioarg);
+}
+#endif
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -853,34 +729,7 @@ static unsigned int
 vmci_poll(struct file *file, // IN
           poll_table *wait)  // IN
 {
-   VMCILockFlags flags;
-   unsigned int mask = 0;
-   VMCIGuestDeviceHandle *devHndl =
-      (VMCIGuestDeviceHandle *) file->private_data;
-
-   /*
-    * Check for call to this VMCI process.
-    */
-
-   if (!devHndl) {
-      return mask;
-   }
-   if (devHndl->objType == VMCIOBJ_DATAGRAM_PROCESS) {
-      VMCIDatagramProcess *dgmProc = (VMCIDatagramProcess *) devHndl->obj;
-      ASSERT(dgmProc);
-
-      if (wait != NULL) {
-         poll_wait(file, &dgmProc->host.waitQueue, wait);
-      }
-
-      VMCI_GrabLock_BH(&dgmProc->datagramQueueLock, &flags);
-      if (dgmProc->pendingDatagrams > 0) {
-         mask = POLLIN;
-      }
-      VMCI_ReleaseLock_BH(&dgmProc->datagramQueueLock, flags);
-   }
-
-   return mask;
+   return 0;
 }
 
 
@@ -1171,6 +1020,13 @@ module_init(vmci_init);
 module_exit(vmci_exit);
 MODULE_DEVICE_TABLE(pci, vmci_ids);
 
+module_param_named(disable_msi, vmci_disable_msi, bool, 0);
+MODULE_PARM_DESC(disable_msi, "Disable MSI use in driver - (default=0)");
+
+module_param_named(disable_msix, vmci_disable_msix, bool, 0);
+MODULE_PARM_DESC(disable_msix, "Disable MSI-X use in driver - (default="
+		 __stringify(VMCI_DISABLE_MSIX) ")");
+
 /* Module information. */
 MODULE_AUTHOR("VMware, Inc.");
 MODULE_DESCRIPTION("VMware Virtual Machine Communication Interface");
