@@ -24,9 +24,9 @@
  */
 
 #include "driver-config.h"
+#include <linux/list.h>
 #include <linux/socket.h>
 #include "compat_sock.h"
-#include "compat_list.h"
 
 #include "af_vsock.h"
 #include "util.h"
@@ -371,8 +371,8 @@ __VSockVmciFindBoundSocket(struct sockaddr_vm *addr)  // IN
          sk = sk_vsock(vsk);
 
          /* We only store stream sockets in the bound table. */
-         ASSERT(sk->compat_sk_socket ?
-                   sk->compat_sk_socket->type == SOCK_STREAM :
+         ASSERT(sk->sk_socket ?
+                   sk->sk_socket->type == SOCK_STREAM :
                    1);
          goto found;
       }
