@@ -74,6 +74,17 @@
 #define VMXNET3_PT_REG_SIZE     4096    /* BAR 0 */
 #define VMXNET3_VD_REG_SIZE     4096    /* BAR 1 */
 
+/*
+ * The two Vmxnet3 MMIO Register PCI BARs (BAR 0 at offset 10h and BAR 1 at
+ * offset 14h)  as well as the MSI-X BAR are combined into one PhysMem region:
+ * <-VMXNET3_PT_REG_SIZE-><-VMXNET3_VD_REG_SIZE-><-VMXNET3_MSIX_BAR_SIZE-->
+ * -------------------------------------------------------------------------
+ * |Pass Thru Registers  | Virtual Dev Registers | MSI-X Vector/PBA Table  |
+ * -------------------------------------------------------------------------
+ * VMXNET3_MSIX_BAR_SIZE is defined in "vmxnet3Int.h"
+ */
+#define VMXNET3_PHYSMEM_PAGES   4
+
 #define VMXNET3_REG_ALIGN       8  /* All registers are 8-byte aligned. */
 #define VMXNET3_REG_ALIGN_MASK  0x7
 
@@ -119,9 +130,13 @@ typedef enum {
    VMXNET3_CMD_GET_DID_LO,
    VMXNET3_CMD_GET_DID_HI,
    VMXNET3_CMD_GET_DEV_EXTRA_INFO,
-   VMXNET3_CMD_GET_CONF_INTR
+   VMXNET3_CMD_GET_CONF_INTR,
+   VMXNET3_CMD_GET_ADAPTIVE_RING_INFO
 } Vmxnet3_Cmd;
 
+/* Adaptive Ring Info Flags */
+#define VMXNET3_DISABLE_ADAPTIVE_RING 1
+
 /*
  *	Little Endian layout of bitfields -
  *	Byte 0 :	7.....len.....0
@@ -721,4 +736,6 @@ do {\
 
 #define VMXWIFI_DRIVER_SHARED_LEN 8192
 
+#define VMXNET3_DID_PASSTHRU    0xFFFF
+
 #endif /* _VMXNET3_DEFS_H_ */
