@@ -26,8 +26,6 @@
 
 #define G_LOG_DOMAIN "dndcp"
 
-#include "copyPasteDnDWrapper.h"
-
 #if defined(HAVE_GTKMM)
 #include "copyPasteDnDX11.h"
 #endif
@@ -36,6 +34,13 @@
 #include "copyPasteDnDWin32.h"
 #endif
 
+#if defined(__APPLE__)
+#include "copyPasteDnDMac.h"
+#endif
+
+#include "copyPasteDnDWrapper.h"
+#include "guestDnDCPMgr.hh"
+
 extern "C" {
 #include "vmware.h"
 #include "rpcout.h"
@@ -49,7 +54,7 @@ extern "C" {
  * CopyPasteDnDWrapper is a singleton, here is a pointer to its only instance.
  */
 
-CopyPasteDnDWrapper *CopyPasteDnDWrapper::m_instance = 0;
+CopyPasteDnDWrapper *CopyPasteDnDWrapper::m_instance = NULL;
 
 /**
  *
@@ -70,6 +75,22 @@ CopyPasteDnDWrapper::GetInstance()
 }
 
 
+/**
+ *
+ * Destroy the singleton object.
+ */
+
+void
+CopyPasteDnDWrapper::Destroy()
+{
+   if (m_instance) {
+      g_debug("%s: destroying self\n", __FUNCTION__);
+      delete m_instance;
+      m_instance = NULL;
+   }
+}
+
+
 /**
  *
  * Constructor.
@@ -116,12 +137,25 @@ void
 CopyPasteDnDWrapper::Init(ToolsAppCtx *ctx)
 {
    m_ctx = ctx;
+
+   /*
+    * We only support DnD/CP V3 and greater.
+    */
+   if (GetDnDVersion() >= 3 && GetCPVersion() >= 3 || GetCPVersion() == 1) {
+      GuestDnDCPMgr *p = GuestDnDCPMgr::GetInstance();
+      ASSERT(p);
+      p->Init(ctx);
+   }
+
    if (!m_pimpl) {
 #if defined(HAVE_GTKMM)
       m_pimpl = new CopyPasteDnDX11();
 #endif
 #if defined(WIN32)
       m_pimpl = new CopyPasteDnDWin32();
+#endif
+#if defined(__APPLE__)
+      m_pimpl = new CopyPasteDnDMac();
 #endif
       if (m_pimpl) {
          m_pimpl->Init(ctx);
@@ -147,6 +181,7 @@ CopyPasteDnDWrapper::~CopyPasteDnDWrapper()
       }
       delete m_pimpl;
    }
+   GuestDnDCPMgr::Destroy();
 }
 
 
@@ -489,7 +524,7 @@ CopyPasteDnDWrapper::OnReset()
    GSource *src;
 
    g_debug("%s: enter\n", __FUNCTION__);
-   src = g_timeout_source_new(RPC_POLL_TIME * 30);
+   src = VMTools_CreateTimer(RPC_POLL_TIME * 30);
    if (src) {
       VMTOOLSAPP_ATTACH_SOURCE(m_ctx, src, DnDPluginResetSent, this, NULL);
       g_source_unref(src);
@@ -509,7 +544,7 @@ CopyPasteDnDWrapper::OnCapReg(gboolean set)
    g_debug("%s: enter\n", __FUNCTION__);
    char *reply;
    size_t replyLen;
-   char *toolsDnDVersion = TOOLS_DND_VERSION_3;
+   const char *toolsDnDVersion = TOOLS_DND_VERSION_4;
    char *toolsCopyPasteVersion = NULL;
    int version;
 
@@ -525,7 +560,8 @@ CopyPasteDnDWrapper::OnCapReg(gboolean set)
          version = 1;
          SetDnDVersion(version);
       } else {
-         char *vmxDnDVersion = QUERY_VMX_DND_VERSION;
+         char const *vmxDnDVersion = QUERY_VMX_DND_VERSION;
+
          if (!RpcChannel_Send(ctx->rpc, vmxDnDVersion,
                               strlen(vmxDnDVersion), &reply, &replyLen)) {
             g_debug("%s: could not get VMX dnd version capability, assuming v1\n",
@@ -534,9 +570,25 @@ CopyPasteDnDWrapper::OnCapReg(gboolean set)
             SetDnDVersion(version);
          } else {
             int version = atoi(reply);
-            ASSERT(version > 1);
+            ASSERT(version >= 1);
             SetDnDVersion(version);
             g_debug("%s: VMX is dnd version %d\n", __FUNCTION__, GetDnDVersion());
+            if (version == 3) {
+               /*
+                * VMDB still has version 4 in it, which will cause a V3
+                * host to fail. So, change to version 3. Since we don't
+                * support any other version, we only do this for V3.
+                */
+               toolsDnDVersion = TOOLS_DND_VERSION_3;
+               if (!RpcChannel_Send(ctx->rpc, toolsDnDVersion,
+                                    strlen(toolsDnDVersion), NULL, NULL)) {
+
+                  g_debug("%s: could not set VMX dnd version capability, assuming v1\n",
+                           __FUNCTION__);
+                  version = 1;
+                  SetDnDVersion(version);
+               }
+            }
          }
          vm_free(reply);
        }
@@ -545,7 +597,7 @@ CopyPasteDnDWrapper::OnCapReg(gboolean set)
        * Now CopyPaste.
        */
 
-      toolsCopyPasteVersion = g_strdup_printf(TOOLS_COPYPASTE_VERSION" %d", 3);
+      toolsCopyPasteVersion = g_strdup_printf(TOOLS_COPYPASTE_VERSION" %d", 4);
       if (!RpcChannel_Send(ctx->rpc, toolsCopyPasteVersion,
                            strlen(toolsCopyPasteVersion),
                            NULL, NULL)) {
@@ -554,7 +606,8 @@ CopyPasteDnDWrapper::OnCapReg(gboolean set)
          version = 1;
          SetCPVersion(version);
       } else {
-         char *vmxCopyPasteVersion = QUERY_VMX_COPYPASTE_VERSION;
+         char const *vmxCopyPasteVersion = QUERY_VMX_COPYPASTE_VERSION;
+
          if (!RpcChannel_Send(ctx->rpc, vmxCopyPasteVersion,
                               strlen(vmxCopyPasteVersion), &reply, &replyLen)) {
             g_debug("%s: could not get VMX copypaste version capability, assuming v1\n",
@@ -567,6 +620,23 @@ CopyPasteDnDWrapper::OnCapReg(gboolean set)
             SetCPVersion(version);
             g_debug("%s: VMX is copypaste version %d\n", __FUNCTION__,
                   GetCPVersion());
+            if (version == 3) {
+               /*
+                * VMDB still has version 4 in it, which will cause a V3
+                * host to fail. So, change to version 3. Since we don't
+                * support any other version, we only do this for V3.
+                */
+               g_free(toolsCopyPasteVersion);
+               toolsCopyPasteVersion = g_strdup_printf(TOOLS_COPYPASTE_VERSION" %d", 3);
+               if (!RpcChannel_Send(ctx->rpc, toolsCopyPasteVersion,
+                                    strlen(toolsCopyPasteVersion), NULL, NULL)) {
+
+                  g_debug("%s: could not set VMX copypaste version, assuming v1\n",
+                           __FUNCTION__);
+                  version = 1;
+                  SetCPVersion(version);
+               }
+            }
          }
          vm_free(reply);
       }
