@@ -33,6 +33,7 @@
 #include "copyPasteDnDWrapper.h"
 #include "copyPasteUIX11.h"
 #include "dndFileList.hh"
+#include "guestDnDCPMgr.hh"
 
 extern "C" {
    #include "vmblock.h"
@@ -86,13 +87,16 @@ GdkAtom GDK_SELECTION_TYPE_UTF8_STRING;
 CopyPasteUIX11::CopyPasteUIX11()
  : mClipboardEmpty(true),
    mHGStagingDir(""),
-   mIsClipboardOwner(false),
    mHGGetFilesInitiated(false),
    mFileTransferDone(false),
    mBlockAdded(false),
    mBlockCtrl(0),
    mInited(false)
 {
+   GuestDnDCPMgr *p = GuestDnDCPMgr::GetInstance();
+   ASSERT(p);
+   mCP = p->GetCopyPasteMgr();
+   ASSERT(mCP);
 }
 
 
@@ -125,17 +129,19 @@ CopyPasteUIX11::Init()
 
    Gtk::TargetEntry gnome(FCP_TARGET_NAME_GNOME_COPIED_FILES);
    Gtk::TargetEntry kde(FCP_TARGET_NAME_URI_LIST);
+   Gtk::TargetEntry vmware(VMWARE_TARGET);
    gnome.set_info(FCP_TARGET_INFO_GNOME_COPIED_FILES);
    kde.set_info(FCP_TARGET_INFO_URI_LIST);
 
    mListTargets.push_back(gnome);
    mListTargets.push_back(kde);
+   mListTargets.push_back(vmware);
 
-   mCP.newClipboard.connect(
+   mCP->srcRecvClipChanged.connect(
       sigc::mem_fun(this, &CopyPasteUIX11::GetRemoteClipboardCB));
-   mCP.localGetClipboard.connect(
+   mCP->destRequestClipChanged.connect(
       sigc::mem_fun(this, &CopyPasteUIX11::GetLocalClipboard));
-   mCP.localGetFilesDoneChanged.connect(
+   mCP->getFilesDoneChanged.connect(
       sigc::mem_fun(this, &CopyPasteUIX11::GetLocalFilesDone));
 
    mInited = true;
@@ -186,8 +192,9 @@ void
 CopyPasteUIX11::VmxCopyPasteVersionChanged(RpcChannel *chan,    // IN
                                            uint32 version)      // IN
 {
+   ASSERT(mCP);
    g_debug("%s: new version is %d\n", __FUNCTION__, version);
-   mCP.VmxCopyPasteVersionChanged(chan, version);
+   mCP->VmxCopyPasteVersionChanged(version);
 }
 
 
@@ -209,24 +216,32 @@ CopyPasteUIX11::VmxCopyPasteVersionChanged(RpcChannel *chan,    // IN
  *-----------------------------------------------------------------------------
  */
 
-bool
-CopyPasteUIX11::GetLocalClipboard(CPClipboard *clip) // OUT
+void
+CopyPasteUIX11::GetLocalClipboard(void)
 {
-   g_debug("%s: enter.\n", __FUNCTION__);
-
-   if (mIsClipboardOwner) {
-      g_debug("%s: is clipboard owner, set changed to false and return.\n", __FUNCTION__);
-      CPClipboard_SetChanged(clip, FALSE);
-      return true;
-   }
-
-   if (!mCP.IsCopyPasteAllowed()) {
-      g_debug("%s: copyPaste is not allowed\n", __FUNCTION__);
-      return true;
-   }
-
    Glib::RefPtr<Gtk::Clipboard> refClipboard =
       Gtk::Clipboard::get(GDK_SELECTION_CLIPBOARD);
+   Glib::RefPtr<Gtk::Clipboard> refPrimary =
+      Gtk::Clipboard::get(GDK_SELECTION_PRIMARY);
+
+   g_debug("%s: enter.\n", __FUNCTION__);
+
+   if (refClipboard->wait_is_target_available(VMWARE_TARGET) &&
+       refPrimary->wait_is_target_available(VMWARE_TARGET)) {
+      /* If we are clipboard owner, send a not-changed clip to host. */
+      g_debug("%s: we are owner, send unchanged clip back.\n", __FUNCTION__);
+      CPClipboard clip;
+      CPClipboard_Init(&clip);
+      CPClipboard_SetChanged(&clip, FALSE);
+      mCP->DestUISendClip(&clip);
+      CPClipboard_Destroy(&clip);
+      return;
+   }
+
+   if (!mCP->IsCopyPasteAllowed()) {
+      g_debug("%s: copyPaste is not allowed\n", __FUNCTION__);
+      return;
+   }
 
    mClipTime = 0;
    mPrimTime = 0;
@@ -234,7 +249,6 @@ CopyPasteUIX11::GetLocalClipboard(CPClipboard *clip) // OUT
    g_debug("%s: retrieving timestamps\n", __FUNCTION__);
    refClipboard->request_contents(TARGET_NAME_TIMESTAMP,
       sigc::mem_fun(this, &CopyPasteUIX11::LocalClipboardTimestampCB));
-   return false;
 }
 
 
@@ -291,7 +305,6 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
                                       guint info)                    // IN:
 {
    g_debug("%s: enter.\n", __FUNCTION__);
-   mHGCopiedUriList = "";
    VmTimeType curTime;
    mBlockAdded = false;
 
@@ -310,8 +323,8 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
       return;
    }
 
-   if (!mIsClipboardOwner || !mCP.IsCopyPasteAllowed()) {
-      g_debug("%s: not clipboard ownder, or copy paste not allowed, returning.\n",
+   if (!mCP->IsCopyPasteAllowed()) {
+      g_debug("%s: copy paste is not allowed, returning.\n",
             __FUNCTION__);
       return;
    }
@@ -329,7 +342,7 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
       size_t index = 0;
       mFileTransferDone = false;
 
-      hgStagingDir = static_cast<utf::string>(mCP.GetFiles());
+      hgStagingDir = static_cast<utf::string>(mCP->SrcUIRequestFiles());
       g_debug("%s: Getting files. Staging dir: %s", __FUNCTION__,
             hgStagingDir.c_str());
 
@@ -459,7 +472,7 @@ CopyPasteUIX11::LocalGetTextOrRTFRequestCB(Gtk::SelectionData& sd, // IN/OUT
 {
    sd.set(sd.get_target().c_str(), "");
 
-   if (!mCP.IsCopyPasteAllowed()) {
+   if (!mCP->IsCopyPasteAllowed()) {
       return;
    }
 
@@ -517,7 +530,6 @@ void
 CopyPasteUIX11::LocalClearClipboardCB(void)
 {
    g_debug("%s: got clear callback\n", __FUNCTION__);
-   mIsClipboardOwner = FALSE;
 }
 
 
@@ -609,6 +621,13 @@ again:
 
    CPClipboard_Clear(&mClipboard);
 
+   if (refClipboard->wait_is_target_available(VMWARE_TARGET)) {
+      g_debug("%s: we are owner, send unchanged clip back.\n", __FUNCTION__);
+      CPClipboard_SetChanged(&mClipboard, FALSE);
+      mCP->DestUISendClip(&mClipboard);
+      return;
+   }
+
    /* First check for URIs. This must always be done first */
    bool haveURIs = false;
    std::string format;
@@ -638,13 +657,12 @@ again:
           bufSize <= (int)CPCLIPITEM_MAX_SIZE_V3 &&
           CPClipboard_SetItem(&mClipboard, CPFORMAT_IMG_PNG,
                               buf, bufSize)) {
-         mCP.SetRemoteClipboard(&mClipboard);
+         validDataInClip = true;
          g_debug("%s: Got PNG: %"FMTSZ"u\n", __FUNCTION__, bufSize);
       } else {
          g_debug("%s: Failed to get PNG\n", __FUNCTION__);
       }
       g_free(buf);
-      return;
    }
 
    /* Try to get RTF data from clipboard. */
@@ -702,7 +720,7 @@ again:
       /*
        * RTF or text data (or both) in the clipboard.
        */
-      mCP.SetRemoteClipboard(&mClipboard);
+      mCP->DestUISendClip(&mClipboard);
    } else if (!flipped) {
       /*
        * If we get here, we got nothing (no image, URI, text) so
@@ -714,6 +732,10 @@ again:
                      GDK_SELECTION_CLIPBOARD : GDK_SELECTION_PRIMARY;
       flipped = true;
       goto again;
+   } else {
+      g_debug("%s: got nothing, send empty clip back.\n",
+            __FUNCTION__);
+      mCP->DestUISendClip(&mClipboard);
    }
 }
 
@@ -744,7 +766,7 @@ CopyPasteUIX11::LocalReceivedFileListCB(const Gtk::SelectionData& sd)        //
    if (target == FCP_TARGET_NAME_GNOME_COPIED_FILES ||
        target == FCP_TARGET_NAME_URI_LIST) {
       LocalGetSelectionFileList(sd);
-      mCP.SetRemoteClipboard(&mClipboard);
+      mCP->DestUISendClip(&mClipboard);
    }
 }
 
@@ -872,6 +894,23 @@ CopyPasteUIX11::LocalGetSelectionFileList(const Gtk::SelectionData& sd)      //
    while ((newPath = DnD_UriListGetNextFile(source.c_str(),
                                             &index,
                                             &newPathLen)) != NULL) {
+#if defined(linux)
+      if (DnD_UriIsNonFileSchemes(newPath)) {
+         /* Try to get local file path for non file uri. */
+         GFile *file = g_file_new_for_uri(newPath);
+         free(newPath);
+         if (!file) {
+            g_debug("%s: g_file_new_for_uri failed\n", __FUNCTION__);
+            return;
+         }
+         newPath = g_file_get_path(file);
+         g_object_unref(file);
+         if (!newPath) {
+            g_debug("%s: g_file_get_path failed\n", __FUNCTION__);
+            return;
+         }
+      }
+#endif
 
       /*
        * Parse relative path.
@@ -1043,12 +1082,14 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
    }
 
    /* Clear the clipboard contents if we are the owner. */
-   if (mIsClipboardOwner) {
+   if (refClipboard->wait_is_target_available(VMWARE_TARGET)) {
       refClipboard->clear();
-      refPrimary->clear();
-      mIsClipboardOwner = FALSE;
       g_debug("%s: Cleared local clipboard", __FUNCTION__);
    }
+   if (refPrimary->wait_is_target_available(VMWARE_TARGET)) {
+      refPrimary->clear();
+      g_debug("%s: Cleared local primary selection", __FUNCTION__);
+   }
 
    mHGTextData.clear();
    mHGRTFData.clear();
@@ -1063,28 +1104,28 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
          Gtk::TargetEntry plainText(TARGET_NAME_TEXT_PLAIN);
          Gtk::TargetEntry utf8Text(TARGET_NAME_UTF8_STRING);
          Gtk::TargetEntry compountText(TARGET_NAME_COMPOUND_TEXT);
+         Gtk::TargetEntry vmware(VMWARE_TARGET);
 
          g_debug("%s: Text data, size %"FMTSZ"u.\n", __FUNCTION__, sz);
          targets.push_back(stringText);
          targets.push_back(plainText);
          targets.push_back(utf8Text);
          targets.push_back(compountText);
+         targets.push_back(vmware);
          mHGTextData = utf::string(reinterpret_cast<char *>(buf),
                                    STRING_ENCODING_UTF8);
-
-         mIsClipboardOwner = TRUE;
       }
 
       if (CPClipboard_GetItem(clip, CPFORMAT_RTF, &buf, &sz)) {
          g_debug("%s: RTF data, size %"FMTSZ"u.\n", __FUNCTION__, sz);
          Gtk::TargetEntry appRtf(TARGET_NAME_APPLICATION_RTF);
          Gtk::TargetEntry textRtf(TARGET_NAME_TEXT_RICHTEXT);
+         Gtk::TargetEntry vmware(VMWARE_TARGET);
 
          targets.push_back(appRtf);
          targets.push_back(textRtf);
+         targets.push_back(vmware);
          mHGRTFData = utf::string((const char *)buf);
-
-         mIsClipboardOwner = TRUE;
       }
 
       refClipboard->set(targets,
@@ -1123,7 +1164,6 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
                       sigc::mem_fun(this, &CopyPasteUIX11::LocalGetFileRequestCB),
                       sigc::mem_fun(this, &CopyPasteUIX11::LocalClearClipboardCB));
 
-      mIsClipboardOwner = TRUE;
       mHGGetListTime = GetCurrentTime();
       mHGGetFilesInitiated = false;
       mHGCopiedUriList = "";
@@ -1138,7 +1178,6 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
          refPrimary->set(mListTargets,
                          sigc::mem_fun(this, &CopyPasteUIX11::LocalGetFileContentsRequestCB),
                          sigc::mem_fun(this, &CopyPasteUIX11::LocalClearClipboardCB));
-         mIsClipboardOwner = TRUE;
       }
    }
 }
@@ -1365,7 +1404,10 @@ CopyPasteUIX11::GetLocalFilesDone(bool success)
 
    if (mBlockAdded) {
       g_debug("%s: removing block for %s\n", __FUNCTION__, mHGStagingDir.c_str());
-      mBlockCtrl->RemoveBlock(mBlockCtrl->fd, mHGStagingDir.c_str());
+      /* We need to make sure block subsystem has not been shut off. */
+      if (DnD_BlockIsReady(mBlockCtrl)) {
+         mBlockCtrl->RemoveBlock(mBlockCtrl->fd, mHGStagingDir.c_str());
+      }
       mBlockAdded = false;
    }
 
@@ -1381,7 +1423,6 @@ CopyPasteUIX11::GetLocalFilesDone(bool success)
       /* Copied files are already removed in common layer. */
       mHGStagingDir.clear();
    }
-   mHGGetFilesInitiated = false;
 }
 
 
