@@ -58,6 +58,7 @@
 /* Strings used for formatting various types of data */
 #define DND_URI_LIST_PRE     "file://"
 #define DND_URI_LIST_PRE_KDE "file:"
+#define DND_URI_NON_FILE_SCHEMES {"ssh", "sftp", "smb", "dav", "davs", NULL}
 #define DND_URI_LIST_POST    "\r\n"
 #define DND_TEXT_PLAIN_PRE   ""
 #define DND_TEXT_PLAIN_POST  ""
@@ -134,6 +135,17 @@ typedef struct {
    CPClipItem items[CPFORMAT_MAX - 1];
 } CPClipboard;
 
+#if !defined(SWIG)
+
+/*
+ * Comment out the following for SWIG. We don't currently need to use any of
+ * these data structures or call any of these functions from test scripts, and
+ * we would have to link in extra libraries if so. Only DnD V3 transport layer
+ * will call these functions. At some later time, may want to refactor this
+ * file to separate CPClipboard definitions from all these transport-related
+ * stuff (it is just the CPClipboard code that test scripts need).
+ */
+
 /* Definitions for transport layer big buffer support (>= V3). */
 typedef enum
 {
@@ -226,9 +238,11 @@ EXTERN Bool DnD_GetFileList(HDROP hDrop,
 /*
  * Posix-specific functions
  */
+
 EXTERN char *DnD_UriListGetNextFile(char const *uriList,
                                     size_t *index,
                                     size_t *length);
+EXTERN Bool DnD_UriIsNonFileSchemes(char const *uri);
 #endif
 
 /*
@@ -278,5 +292,6 @@ size_t DnD_TransportMsgToPacket(uint8 *msg,
                                 DnDTransportPacketHeader **packet);
 size_t DnD_TransportReqPacket(DnDTransportBuffer *buf,
                               DnDTransportPacketHeader **packet);
+#endif // !SWIG
 
 #endif // _DND_H_
