@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -30,8 +30,7 @@
 
 #include "dndClipboard.h"
 #include "dndInt.h"
-#include "dndCPMsgV4.h"
-#include "unicode.h"
+
 
 #define CPFormatToIndex(x) ((unsigned int)(x) - 1)
 
@@ -115,12 +114,11 @@ CPClipItemCopy(CPClipItem *dest,        // IN: dest clipboard item
 
    if (src->buf) {
       void *tmp = dest->buf;
-      dest->buf = realloc(dest->buf, src->size + 1);
+      dest->buf = realloc(dest->buf, src->size);
       if (!dest->buf) {
          dest->buf = tmp;
          return FALSE;
       }
-      ((uint8 *)dest->buf)[src->size] = 0;
       memcpy(dest->buf, src->buf, src->size);
    }
 
@@ -245,7 +243,7 @@ CPClipboard_SetItem(CPClipboard *clip,          // IN/OUT: the clipboard
                     const size_t size)          // IN: the item size
 {
    CPClipItem *item;
-   uint8 *newBuf = NULL;
+   void *newBuf = NULL;
    /*
     * Image, rtf and text may be put into a clipboard at same time, and total
     * size may be more than limit. Image data will be first dropped, then
@@ -271,22 +269,11 @@ CPClipboard_SetItem(CPClipboard *clip,          // IN/OUT: the clipboard
    item = &clip->items[CPFormatToIndex(fmt)];
 
    if (clipitem) {
-      /* It has to be valid utf8 for plain text format. */
-      if (CPFORMAT_TEXT == fmt) {
-         char *str = (char *)clipitem;
-         if (!Unicode_IsBufferValid(str,
-                                    size,
-                                    STRING_ENCODING_UTF8)) {
-            return FALSE;
-         }
-      }
-
-      newBuf = malloc(size + 1);
+      newBuf = malloc(size);
       if (!newBuf) {
          return FALSE;
       }
       memcpy(newBuf, clipitem, size);
-      newBuf[size] = 0;
    }
 
    item->buf = newBuf;
@@ -380,8 +367,6 @@ CPClipboard_GetItem(const CPClipboard *clip,    // IN: the clipboard
    if (clip->items[CPFormatToIndex(fmt)].exists) {
       *buf = clip->items[CPFormatToIndex(fmt)].buf;
       *size = clip->items[CPFormatToIndex(fmt)].size;
-      ASSERT(*buf);
-      ASSERT((*size > 0) && (*size < CPCLIPITEM_MAX_SIZE_V3));
       return TRUE;
    } else {
       ASSERT(!clip->items[CPFormatToIndex(fmt)].size);
@@ -705,51 +690,3 @@ error:
    CPClipboard_Destroy(clip);
    return FALSE;
 }
-
-
-/*
- *----------------------------------------------------------------------------
- *
- * CPClipboard_Strip --
- *
- *      Remove clipboard items based on the passed in capabilities mask.
- *      Introduced in DnDV4.
- *
- *      XXX This function assumes that the bits in mask are such that if the
- *      check is being made for copy paste, that the corresponding bit for
- *      DnD is set to zero. Otherwise, the format cleared by copy paste will
- *      not be removed. Similar for the other case. A way to make this clearer
- *      would be to pass a flag to this function that tells it which bits
- *      to check, with no dependencies on the other bits being in proper
- *      state.
- *
- * Results:
- *      TRUE if clipboard is empty as a result, else FALSE.
- *
- *----------------------------------------------------------------------------
- */
-
-Bool
-CPClipboard_Strip(CPClipboard *clip,    // IN/OUT: the clipboard
-                  uint32 mask)          // IN: if TRUE, DnD.
-{
-   if (!(mask & DND_CP_CAP_PLAIN_TEXT_DND) &&
-       !(mask & DND_CP_CAP_PLAIN_TEXT_CP)) {
-      CPClipboard_ClearItem(clip, CPFORMAT_TEXT);
-   }
-   if (!(mask & DND_CP_CAP_RTF_DND) && !(mask & DND_CP_CAP_RTF_CP)) {
-      CPClipboard_ClearItem(clip, CPFORMAT_RTF);
-   }
-   if (!(mask & DND_CP_CAP_IMAGE_DND) && !(mask & DND_CP_CAP_IMAGE_CP)) {
-      CPClipboard_ClearItem(clip, CPFORMAT_IMG_PNG);
-   }
-   if (!(mask & DND_CP_CAP_FILE_DND) && !(mask & DND_CP_CAP_FILE_CP)) {
-      CPClipboard_ClearItem(clip, CPFORMAT_FILELIST);
-      CPClipboard_ClearItem(clip, CPFORMAT_FILELIST_URI);
-   }
-   if (!(mask & DND_CP_CAP_FILE_CONTENT_DND) &&
-       !(mask & DND_CP_CAP_FILE_CONTENT_CP)) {
-      CPClipboard_ClearItem(clip, CPFORMAT_FILECONTENTS);
-   }
-   return CPClipboard_IsEmpty(clip);
-}
