@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2005-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2005 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -54,14 +54,6 @@
 #ifdef sun
 #define ACCESSPERMS           (S_IRWXU | S_IRWXG | S_IRWXO)
 #endif
-#ifdef __ANDROID__
-/*
- * Android doesn't support setmntent(), endmntent() or MOUNTED.
- */
-#define NO_SETMNTENT
-#define NO_ENDMNTENT
-#define ACCESSPERMS           (S_IRWXU | S_IRWXG | S_IRWXO)
-#endif
 
 
 /*
@@ -80,13 +72,48 @@
  *-----------------------------------------------------------------------------
  */
 
-const char *
+ConstUnicode
 DnD_GetFileRoot(void)
 {
    return "/tmp/VMwareDnD/";
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * DnD_PrependFileRoot --
+ *
+ *    Given a buffer of '\0' delimited filenames, this prepends the file root
+ *    to each one and uses '\0' delimiting for the output buffer.  The buffer
+ *    pointed to by *src will be freed and *src will point to a new buffer
+ *    containing the results.  *srcSize is set to the size of the new buffer,
+ *    not including the NUL-terminator.
+ *
+ * Results:
+ *    TRUE on success, FALSE on failure.
+ *
+ * Side effects:
+ *    *src will be freed, and a new buffer will be allocated. This buffer must
+ *    be freed by the caller.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+DnD_PrependFileRoot(const char *fileRoot,  // IN    : file root to append
+                    char **src,            // IN/OUT: NUL-delimited list of paths
+                    size_t *srcSize)       // IN/OUT: size of list
+{
+   ASSERT(fileRoot);
+   ASSERT(src);
+   ASSERT(*src);
+   ASSERT(srcSize);
+
+   return DnDPrependFileRoot(fileRoot, '\0', src, srcSize);
+}
+
+
 /*
  *----------------------------------------------------------------------------
  *
@@ -299,8 +326,6 @@ Bool
 DnD_AddBlockLegacy(int blockFd,                    // IN
                    const char *blockPath)          // IN
 {
-   LOG(1, ("%s: placing block on %s\n", __func__, blockPath));
-
    ASSERT(blockFd >= 0);
 
    if (VMBLOCK_CONTROL(blockFd, VMBLOCK_ADD_FILEBLOCK, blockPath) != 0) {
@@ -334,8 +359,6 @@ Bool
 DnD_RemoveBlockLegacy(int blockFd,                    // IN
                       const char *blockedPath)        // IN
 {
-   LOG(1, ("%s: removing block on %s\n", __func__, blockedPath));
-
    if (blockFd >= 0) {
       if (VMBLOCK_CONTROL(blockFd, VMBLOCK_DEL_FILEBLOCK, blockedPath) != 0) {
          Log("%s: Cannot delete block on %s (%s)\n",
@@ -398,8 +421,6 @@ Bool
 DnD_AddBlockFuse(int blockFd,                    // IN
                  const char *blockPath)          // IN
 {
-   LOG(1, ("%s: placing block on %s\n", __func__, blockPath));
-
    ASSERT(blockFd >= 0);
 
    if (VMBLOCK_CONTROL_FUSE(blockFd, VMBLOCK_FUSE_ADD_FILEBLOCK,
@@ -434,8 +455,6 @@ Bool
 DnD_RemoveBlockFuse(int blockFd,                    // IN
                     const char *blockedPath)        // IN
 {
-   LOG(1, ("%s: removing block on %s\n", __func__, blockedPath));
-
    if (blockFd >= 0) {
       if (VMBLOCK_CONTROL_FUSE(blockFd, VMBLOCK_FUSE_DEL_FILEBLOCK,
                                blockedPath) != 0) {
@@ -474,7 +493,7 @@ static Bool
 DnD_CheckBlockFuse(int blockFd)                    // IN
 {
    char buf[sizeof(VMBLOCK_FUSE_READ_RESPONSE)];
-   ssize_t size;
+   size_t size;
 
    size = read(blockFd, buf, sizeof(VMBLOCK_FUSE_READ_RESPONSE));
    if (size < 0) {
@@ -485,11 +504,8 @@ DnD_CheckBlockFuse(int blockFd)                    // IN
    }
 
    if (size != sizeof(VMBLOCK_FUSE_READ_RESPONSE)) {
-      /*
-       * Refer to bug 817761 of casting size to size_t.
-       */
-      LOG(4, ("%s: Response too short (%"FMTSZ"u vs. %"FMTSZ"u).\n",
-              __func__, (size_t)size, sizeof(VMBLOCK_FUSE_READ_RESPONSE)));
+      LOG(4, ("%s: Response too short (%"FMTSZ"d vs. %"FMTSZ"u).\n",
+              __func__, size, sizeof(VMBLOCK_FUSE_READ_RESPONSE)));
 
       return FALSE;
    }
@@ -530,36 +546,17 @@ DnD_TryInitVmblock(const char *vmbFsName,          // IN
                    mode_t vmbDeviceMode,           // IN
                    Bool (*verifyBlock)(int fd))    // IN
 {
-#if defined NO_SETMNTENT || defined NO_ENDMNTENT
-   NOT_IMPLEMENTED();
-   errno = ENOSYS;
-   return -1;
-#else
    Bool found = FALSE;
    int blockFd = -1;
-   char *realMntPoint;
    MNTHANDLE fp;
    DECLARE_MNTINFO(mnt);
 
-   ASSERT(vmbFsName);
-   ASSERT(vmbMntPoint);
-   ASSERT(vmbDevice);
-
-   /* Resolve desired mount point in case it is symlinked somewhere */
-   realMntPoint = Posix_RealPath(vmbMntPoint);
-   if (!realMntPoint) {
-      /*
-       * If resolve failed for some reason try to fall back to
-       * original mount point specification.
-       */
-      realMntPoint = Util_SafeStrdup(vmbMntPoint);
-   }
-
    /* Make sure the vmblock file system is mounted. */
    fp = OPEN_MNTFILE("r");
    if (fp == NULL) {
       LOG(1, ("%s: could not open mount file\n", __func__));
-      goto out;
+
+      return -1;
    }
 
    while (GETNEXT_MNTINFO(fp, mnt)) {
@@ -569,7 +566,7 @@ DnD_TryInitVmblock(const char *vmbFsName,          // IN
        */
 
       if (strcmp(MNTINFO_FSTYPE(mnt), vmbFsName) == 0 &&
-          strcmp(MNTINFO_MNTPT(mnt), realMntPoint) == 0) {
+          strcmp(MNTINFO_MNTPT(mnt), vmbMntPoint) == 0) {
          found = TRUE;
          break;
       }
@@ -595,10 +592,7 @@ DnD_TryInitVmblock(const char *vmbFsName,          // IN
       }
    }
 
-out:
-   free(realMntPoint);
    return blockFd;
-#endif
 }
 
 
@@ -757,7 +751,7 @@ DnD_CompleteBlockInitialization(int fd,                     // IN
  */
 
 Bool
-DnDRootDirUsable(const char *pathName)  // IN:
+DnDRootDirUsable(ConstUnicode pathName)  // IN:
 {
    struct stat buf;
 
@@ -790,7 +784,7 @@ DnDRootDirUsable(const char *pathName)  // IN:
  */
 
 Bool
-DnDSetPermissionsOnRootDir(const char *pathName)  // IN:
+DnDSetPermissionsOnRootDir(ConstUnicode pathName)  // IN:
 {
    return Posix_Chmod(pathName, S_ISVTX | DND_ROOTDIR_PERMS) == 0;
 }
@@ -815,7 +809,7 @@ DnDSetPermissionsOnRootDir(const char *pathName)  // IN:
  */
 
 Bool
-DnDStagingDirectoryUsable(const char *pathName)  // IN:
+DnDStagingDirectoryUsable(ConstUnicode pathName)  // IN:
 {
    struct stat buf;
 
@@ -844,7 +838,7 @@ DnDStagingDirectoryUsable(const char *pathName)  // IN:
  */
 
 Bool
-DnDSetPermissionsOnStagingDir(const char *pathName)  // IN:
+DnDSetPermissionsOnStagingDir(ConstUnicode pathName)  // IN:
 {
    return Posix_Chmod(pathName, DND_STAGINGDIR_PERMS) == 0;
 }
