@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -67,23 +67,11 @@ CopyPasteRpcV4::CopyPasteRpcV4(DnDCPTransport *transport)
  */
 
 void
-CopyPasteRpcV4::Init()
+CopyPasteRpcV4::Init(void)
 {
    ASSERT(mTransport);
    mTransport->RegisterRpc(this, mTransportInterface);
-}
-
-
-/**
- * Send Ping message to controller.
- *
- * @param[in] caps capabilities value.
- */
-
-void
-CopyPasteRpcV4::SendPing(uint32 caps)
-{
-   mUtil.SendPingMsg(DEFAULT_CONNECTION_ID, caps);
+   mUtil.SendPingMsg(DEFAULT_CONNECTION_ID, 0);
 }
 
 
@@ -295,13 +283,6 @@ CopyPasteRpcV4::HandleMsg(RpcParams *params,
                                binary,
                                binarySize);
       break;
-   case DNDCP_CMD_PING_REPLY:
-      pingReplyChanged.emit(params->optional.version.capability);
-      break;
-   case DNDCP_CMP_REPLY:
-      LOG(0, ("%s: Got cmp reply command %d.\n", __FUNCTION__, params->cmd));
-      cmdReplyChanged.emit(params->cmd, params->status);
-      break;
    default:
       LOG(0, ("%s: Got unknown command %d.\n", __FUNCTION__, params->cmd));
       break;
