@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -68,23 +68,11 @@ DnDRpcV4::DnDRpcV4(DnDCPTransport *transport)
  */
 
 void
-DnDRpcV4::Init()
+DnDRpcV4::Init(void)
 {
    ASSERT(mTransport);
    mTransport->RegisterRpc(this, mTransportInterface);
-}
-
-
-/**
- * Send Ping message to controller.
- *
- * @param[in] caps capabilities value.
- */
-
-void
-DnDRpcV4::SendPing(uint32 caps)
-{
-   mUtil.SendPingMsg(DEFAULT_CONNECTION_ID, caps);
+   mUtil.SendPingMsg(DEFAULT_CONNECTION_ID, 0);
 }
 
 
@@ -730,7 +718,6 @@ DnDRpcV4::HandleMsg(RpcParams *params,
                                binarySize);
       break;
    case DNDCP_CMD_PING_REPLY:
-      pingReplyChanged.emit(params->optional.version.capability);
       break;
    case DNDCP_CMD_TEST_BIG_BINARY:
    {
@@ -757,10 +744,6 @@ DnDRpcV4::HandleMsg(RpcParams *params,
 
       break;
    }
-   case DNDCP_CMP_REPLY:
-      LOG(0, ("%s: Got cmp reply command %d.\n", __FUNCTION__, params->cmd));
-      cmdReplyChanged.emit(params->cmd, params->status);
-      break;
    default:
       LOG(0, ("%s: Got unknown command %d.\n", __FUNCTION__, params->cmd));
       break;
