@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -22,11 +22,9 @@
  * Implementation of common layer GuestDnDMgr object for guest.
  */
 
-#include "tracer.hh"
 #include "guestDnD.hh"
 #include "dndRpcV4.hh"
 #include "dndRpcV3.hh"
-#include "guestDnDCPMgr.hh"
 
 extern "C" {
    #include "debug.h"
@@ -50,8 +48,6 @@ extern "C" {
 static gboolean
 DnDUngrabTimeout(void *clientData)
 {
-   TRACE_CALL();
-
    ASSERT(clientData);
    GuestDnDMgr *dnd = (GuestDnDMgr *)clientData;
    /* Call actual callback. */
@@ -72,8 +68,7 @@ DnDUngrabTimeout(void *clientData)
 static gboolean
 DnDHideDetWndTimer(void *clientData)
 {
-   TRACE_CALL();
-
+   Debug("%s: entering\n", __FUNCTION__);
    ASSERT(clientData);
    GuestDnDMgr *dnd = (GuestDnDMgr *)clientData;
    dnd->SetHideDetWndTimer(NULL);
@@ -94,8 +89,7 @@ DnDHideDetWndTimer(void *clientData)
 static gboolean
 DnDUnityDetTimeout(void *clientData)
 {
-   TRACE_CALL();
-
+   Debug("%s: entering\n", __FUNCTION__);
    ASSERT(clientData);
    GuestDnDMgr *dnd = (GuestDnDMgr *)clientData;
    dnd->UnityDnDDetTimeout();
@@ -123,8 +117,7 @@ GuestDnDMgr::GuestDnDMgr(DnDCPTransport *transport,
    mUngrabTimeout(NULL),
    mToolsAppCtx(ctx),
    mDnDAllowed(false),
-   mDnDTransport(transport),
-   mCapabilities(0xffffffff)
+   mDnDTransport(transport)
 {
    ASSERT(transport);
    ASSERT(mToolsAppCtx);
@@ -160,8 +153,8 @@ GuestDnDMgr::~GuestDnDMgr(void)
 void
 GuestDnDMgr::ResetDnD(void)
 {
-   TRACE_CALL();
-
+   Debug("%s: state %d, session id %d before reset\n",
+         __FUNCTION__, mDnDState, mSessionId);
    if (mSrc) {
       srcCancelChanged.emit();
       DelayHideDetWnd();
@@ -175,11 +168,9 @@ GuestDnDMgr::ResetDnD(void)
       delete mDest;
       mDest = NULL;
    }
-
    SetState(GUEST_DND_READY);
-
-   g_debug("%s: change to state %d, session id %d\n", __FUNCTION__, mDnDState,
-           mSessionId);
+   Debug("%s: change to state %d, session id %d\n",
+         __FUNCTION__, mDnDState, mSessionId);
 }
 
 
@@ -190,12 +181,10 @@ GuestDnDMgr::ResetDnD(void)
 void
 GuestDnDMgr::SrcUIDragBeginDone(void)
 {
-   TRACE_CALL();
-
    if (mSrc) {
       mSrc->UIDragBeginDone();
    } else {
-      g_debug("%s: mSrc is NULL\n", __FUNCTION__);
+      Debug("%s: mSrc is NULL\n", __FUNCTION__);
    }
 }
 
@@ -209,12 +198,10 @@ GuestDnDMgr::SrcUIDragBeginDone(void)
 void
 GuestDnDMgr::SrcUIUpdateFeedback(DND_DROPEFFECT feedback)
 {
-   TRACE_CALL();
-
    if (mSrc) {
       mSrc->UIUpdateFeedback(feedback);
    } else {
-      g_debug("%s: mSrc is NULL\n", __FUNCTION__);
+      Debug("%s: mSrc is NULL\n", __FUNCTION__);
    }
 }
 
@@ -229,7 +216,7 @@ GuestDnDMgr::SrcUIUpdateFeedback(DND_DROPEFFECT feedback)
 void
 GuestDnDMgr::DestUIDragEnter(const CPClipboard *clip)
 {
-   TRACE_CALL();
+   Debug("%s: enter\n", __FUNCTION__);
 
    /* Remove untriggered ungrab timer. */
    RemoveUngrabTimeout();
@@ -253,7 +240,7 @@ GuestDnDMgr::DestUIDragEnter(const CPClipboard *clip)
     */
    if (mDnDState != GUEST_DND_QUERY_EXITING &&
        mDnDState != GUEST_DND_READY) {
-      g_debug("%s: Bad state: %d, reset\n", __FUNCTION__, mDnDState);
+      Debug("%s: Bad state: %d, reset\n", __FUNCTION__, mDnDState);
       ResetDnD();
       return;
    }
@@ -265,7 +252,7 @@ GuestDnDMgr::DestUIDragEnter(const CPClipboard *clip)
    }
 
    if (mDest) {
-      g_debug("%s: mDest is not NULL\n", __FUNCTION__);
+      Debug("%s: mDest is not NULL\n", __FUNCTION__);
       delete mDest;
       mDest = NULL;
    }
@@ -289,26 +276,27 @@ void
 GuestDnDMgr::OnRpcSrcDragBegin(uint32 sessionId,
                                const CPClipboard *clip)
 {
-   TRACE_CALL();
+   Debug("%s: enter\n", __FUNCTION__);
 
    if (!mDnDAllowed) {
-      g_debug("%s: DnD is not allowed.\n", __FUNCTION__);
+      Debug("%s: DnD is not allowed.\n", __FUNCTION__);
       return;
    }
 
    if  (GUEST_DND_READY != mDnDState) {
-      g_debug("%s: Bad state: %d, reset\n", __FUNCTION__, mDnDState);
+      Debug("%s: Bad state: %d, reset\n", __FUNCTION__, mDnDState);
       ResetDnD();
       return;
    }
 
    if (mSrc) {
-      g_debug("%s: mSrc is not NULL\n", __FUNCTION__);
+      Debug("%s: mSrc is not NULL\n", __FUNCTION__);
       delete mSrc;
       mSrc = NULL;
    }
 
-   SetSessionId(sessionId);
+   mSessionId = sessionId;
+   Debug("%s: change sessionId to %d\n", __FUNCTION__, mSessionId);
 
    ASSERT(clip);
    mSrc = new GuestDnDSrc(this);
@@ -331,24 +319,24 @@ GuestDnDMgr::OnRpcQueryExiting(uint32 sessionId,
                                int32 x,
                                int32 y)
 {
-   TRACE_CALL();
-
    if (!mDnDAllowed) {
-      g_debug("%s: DnD is not allowed.\n", __FUNCTION__);
+      Debug("%s: DnD is not allowed.\n", __FUNCTION__);
       return;
    }
 
    if (GUEST_DND_READY != mDnDState) {
       /* Reset DnD for any wrong state. */
-      g_debug("%s: Bad state: %d\n", __FUNCTION__, mDnDState);
+      Debug("%s: Bad state: %d\n", __FUNCTION__, mDnDState);
       ResetDnD();
       return;
    }
 
    /* Show detection window to detect pending GH DnD. */
    ShowDetWnd(x, y);
-   SetSessionId(sessionId);
+   mSessionId = sessionId;
    SetState(GUEST_DND_QUERY_EXITING);
+   Debug("%s: state changed to QUERY_EXITING, session id changed to %d\n",
+         __FUNCTION__, mSessionId);
 
    /*
     * Add event to fire and hide our window if a DnD is not pending.  Note that
@@ -356,7 +344,7 @@ GuestDnDMgr::OnRpcQueryExiting(uint32 sessionId,
     * for some reason.
     */
    if (NULL == mUngrabTimeout) {
-      g_debug("%s: adding UngrabTimeout\n", __FUNCTION__);
+      Debug("%s: adding UngrabTimeout\n", __FUNCTION__);
       mUngrabTimeout = g_timeout_source_new(UNGRAB_TIMEOUT);
       VMTOOLSAPP_ATTACH_SOURCE(mToolsAppCtx, mUngrabTimeout, DnDUngrabTimeout, this, NULL);
       g_source_unref(mUngrabTimeout);
@@ -373,13 +361,11 @@ GuestDnDMgr::OnRpcQueryExiting(uint32 sessionId,
 void
 GuestDnDMgr::UngrabTimeout(void)
 {
-   TRACE_CALL();
-
    mUngrabTimeout = NULL;
 
    if (mDnDState != GUEST_DND_QUERY_EXITING) {
       /* Reset DnD for any wrong state. */
-      g_debug("%s: Bad state: %d\n", __FUNCTION__, mDnDState);
+      Debug("%s: Bad state: %d\n", __FUNCTION__, mDnDState);
       ResetDnD();
       return;
    }
@@ -389,6 +375,8 @@ GuestDnDMgr::UngrabTimeout(void)
 
    HideDetWnd();
    SetState(GUEST_DND_READY);
+   Debug("%s: state changed to GUEST_DND_READY, session id changed to %d\n",
+         __FUNCTION__, mSessionId);
 }
 
 
@@ -407,23 +395,16 @@ GuestDnDMgr::OnRpcUpdateUnityDetWnd(uint32 sessionId,
                                     bool show,
                                     uint32 unityWndId)
 {
-   TRACE_CALL();
-
    if (show && mDnDState != GUEST_DND_READY) {
       /*
        * Reset DnD for any wrong state. Only do this when host asked to
        * show the window.
        */
-      g_debug("%s: Bad state: %d\n", __FUNCTION__, mDnDState);
+      Debug("%s: Bad state: %d\n", __FUNCTION__, mDnDState);
       ResetDnD();
       return;
    }
 
-   if (mUnityDnDDetTimeout) {
-      g_source_destroy(mUnityDnDDetTimeout);
-      mUnityDnDDetTimeout = NULL;
-   }
-
    if (show) {
       /*
        * When showing full screen window, also show the small top-most
@@ -432,14 +413,18 @@ GuestDnDMgr::OnRpcUpdateUnityDetWnd(uint32 sessionId,
        * this window to accept drop in cancel case.
        */
       UpdateDetWnd(show, 1, 1);
+      if (mUnityDnDDetTimeout) {
+         g_source_unref(mUnityDnDDetTimeout);
+         mUnityDnDDetTimeout = NULL;
+      }
       mUnityDnDDetTimeout = g_timeout_source_new(UNITY_DND_DET_TIMEOUT);
       VMTOOLSAPP_ATTACH_SOURCE(mToolsAppCtx,
                                mUnityDnDDetTimeout,
                                DnDUnityDetTimeout,
                                this,
                                NULL);
-      g_source_unref(mUnityDnDDetTimeout);
-      SetSessionId(sessionId);
+      mSessionId = sessionId;
+      Debug("%s: change sessionId to %d\n", __FUNCTION__, mSessionId);
    } else {
       /*
        * If there is active DnD, the regular detection window will be hidden
@@ -452,8 +437,8 @@ GuestDnDMgr::OnRpcUpdateUnityDetWnd(uint32 sessionId,
 
    /* Show/hide the full screen detection window. */
    updateUnityDetWndChanged.emit(show, unityWndId, false);
-   g_debug("%s: updating Unity detection window, show %d, id %u\n",
-           __FUNCTION__, show, unityWndId);
+   Debug("%s: updating Unity detection window, show %d, id %u\n",
+         __FUNCTION__, show, unityWndId);
 }
 
 
@@ -465,8 +450,6 @@ GuestDnDMgr::OnRpcUpdateUnityDetWnd(uint32 sessionId,
 void
 GuestDnDMgr::UnityDnDDetTimeout(void)
 {
-   TRACE_CALL();
-
    mUnityDnDDetTimeout = NULL;
    updateUnityDetWndChanged.emit(true, 0, true);
 }
@@ -486,14 +469,12 @@ GuestDnDMgr::OnRpcMoveMouse(uint32 sessionId,
                             int32 x,
                             int32 y)
 {
-   TRACE_CALL();
-
    if (GUEST_DND_SRC_DRAGGING != mDnDState &&
        GUEST_DND_PRIV_DRAGGING != mDnDState) {
-      g_debug("%s: not in valid state %d, ignoring\n", __FUNCTION__, mDnDState);
+      Debug("%s: not in valid state %d, ignoring\n", __FUNCTION__, mDnDState);
       return;
    }
-   g_debug("%s: move to %d, %d\n", __FUNCTION__, x, y);
+   Debug("%s: move to %d, %d\n", __FUNCTION__, x, y);
    moveMouseChanged.emit(x, y);
 }
 
@@ -511,15 +492,13 @@ GuestDnDMgr::UpdateDetWnd(bool show,
                           int32 x,
                           int32 y)
 {
-   TRACE_CALL();
-
    if (mHideDetWndTimer) {
       g_source_destroy(mHideDetWndTimer);
       mHideDetWndTimer = NULL;
    }
 
-   g_debug("%s: %s window at %d, %d\n", __FUNCTION__, show ? "show" : "hide",
-           x, y);
+   Debug("%s: %s window at %d, %d\n",
+         __FUNCTION__, show ? "show" : "hide", x, y);
    updateDetWndChanged.emit(show, x, y);
 }
 
@@ -535,16 +514,14 @@ GuestDnDMgr::UpdateDetWnd(bool show,
 void
 GuestDnDMgr::DelayHideDetWnd(void)
 {
-   TRACE_CALL();
-
    if (NULL == mHideDetWndTimer) {
-      g_debug("%s: add timer to hide detection window.\n", __FUNCTION__);
+      Debug("%s: add timer to hide detection window.\n", __FUNCTION__);
       mHideDetWndTimer = g_timeout_source_new(HIDE_DET_WND_TIMER);
       VMTOOLSAPP_ATTACH_SOURCE(mToolsAppCtx, mHideDetWndTimer,
                                DnDHideDetWndTimer, this, NULL);
       g_source_unref(mHideDetWndTimer);
    } else {
-      g_debug("%s: mHideDetWndTimer is not NULL, quit.\n", __FUNCTION__);
+      Debug("%s: mHideDetWndTimer is not NULL, quit.\n", __FUNCTION__);
    }
 }
 
@@ -556,8 +533,6 @@ GuestDnDMgr::DelayHideDetWnd(void)
 void
 GuestDnDMgr::RemoveUngrabTimeout(void)
 {
-   TRACE_CALL();
-
    if (mUngrabTimeout) {
       g_source_destroy(mUngrabTimeout);
       mUngrabTimeout = NULL;
@@ -574,23 +549,6 @@ GuestDnDMgr::RemoveUngrabTimeout(void)
 void
 GuestDnDMgr::SetState(GUEST_DND_STATE state)
 {
-#ifdef VMX86_DEVEL
-   static const char* states[] = {
-      "GUEST_DND_INVALID",
-      "GUEST_DND_READY",
-      /* As destination. */
-      "GUEST_DND_QUERY_EXITING",
-      "GUEST_DND_DEST_DRAGGING",
-      /* In private dragging mode. */
-      "GUEST_DND_PRIV_DRAGGING",
-      /* As source. */
-      "GUEST_DND_SRC_DRAGBEGIN_PENDING",
-      "GUEST_DND_SRC_CANCEL_PENDING",
-      "GUEST_DND_SRC_DRAGGING",
-   };
-   g_debug("%s: %s => %s\n", __FUNCTION__, states[mDnDState], states[state]);
-#endif
-
    mDnDState = state;
    stateChanged.emit(state);
    if (GUEST_DND_READY == state) {
@@ -628,8 +586,6 @@ GuestDnDMgr::IsDragEnterAllowed(void)
 void
 GuestDnDMgr::VmxDnDVersionChanged(uint32 version)
 {
-   TRACE_CALL();
-
    g_debug("GuestDnDMgr::%s: enter version %d\n", __FUNCTION__, version);
    ASSERT(version >= 3);
 
@@ -654,8 +610,7 @@ GuestDnDMgr::VmxDnDVersionChanged(uint32 version)
       break;
    }
    if (mRpc) {
-      mRpc->pingReplyChanged.connect(
-         sigc::mem_fun(this, &GuestDnDMgr::OnPingReply));
+      mRpc->Init();
       mRpc->srcDragBeginChanged.connect(
          sigc::mem_fun(this, &GuestDnDMgr::OnRpcSrcDragBegin));
       mRpc->queryExitingChanged.connect(
@@ -664,48 +619,7 @@ GuestDnDMgr::VmxDnDVersionChanged(uint32 version)
          sigc::mem_fun(this, &GuestDnDMgr::OnRpcUpdateUnityDetWnd));
       mRpc->moveMouseChanged.connect(
          sigc::mem_fun(this, &GuestDnDMgr::OnRpcMoveMouse));
-      mRpc->Init();
-      mRpc->SendPing(GuestDnDCPMgr::GetInstance()->GetCaps() &
-                     (DND_CP_CAP_DND | DND_CP_CAP_FORMATS_DND |
-                      DND_CP_CAP_VALID));
    }
 
    ResetDnD();
 }
-
-
-/**
- * Check if a request is allowed based on resolved capabilities.
- *
- * @param[in] capsRequest requested capabilities.
- *
- * @return TRUE if allowed, FALSE otherwise.
- */
-
-Bool
-GuestDnDMgr::CheckCapability(uint32 capsRequest)
-{
-   Bool allowed = FALSE;
-
-   if ((mCapabilities & capsRequest) == capsRequest) {
-      allowed = TRUE;
-   }
-   return allowed;
-}
-
-
-/**
- * Got pingReplyChanged message. Update capabilities.
- *
- * @param[in] capability modified capabilities from VMX controller.
- */
-
-void
-GuestDnDMgr::OnPingReply(uint32 capabilities)
-{
-   TRACE_CALL();
-
-   g_debug("%s: dnd ping reply caps are %x\n", __FUNCTION__, capabilities);
-   mCapabilities = capabilities;
-}
-
