@@ -26,6 +26,7 @@
 #define G_LOG_DOMAIN "dndcp"
 
 #include "dndUIX11.h"
+#include "guestDnDCPMgr.hh"
 
 extern "C" {
 #include "vmblock.h"
@@ -70,7 +71,7 @@ DnDUIX11::DnDUIX11(ToolsAppCtx *ctx)
       m_unityMode(false),
       m_inHGDrag(false),
       m_effect(DROP_NONE),
-      m_isFileDnD(false),
+      m_fileTransferStarted(false),
       m_mousePosX(0),
       m_mousePosY(0),
       m_dc(NULL),
@@ -88,9 +89,6 @@ DnDUIX11::DnDUIX11(ToolsAppCtx *ctx)
 DnDUIX11::~DnDUIX11()
 {
    g_debug("%s: enter\n", __FUNCTION__);
-   if (m_DnD) {
-      delete m_DnD;
-   }
    if (m_detWnd) {
       delete m_detWnd;
    }
@@ -109,13 +107,13 @@ DnDUIX11::Init()
    g_debug("%s: enter\n", __FUNCTION__);
    bool ret = true;
 
-   ASSERT(m_ctx);
    CPClipboard_Init(&m_clipboard);
-   m_DnD = new DnD(m_ctx);
-   if (!m_DnD) {
-      g_debug("%s: unable to allocate DnD object\n", __FUNCTION__);
-      goto fail;
-   }
+
+   GuestDnDCPMgr *p = GuestDnDCPMgr::GetInstance();
+   ASSERT(p);
+   m_DnD = p->GetDnDMgr();
+   ASSERT(m_DnD);
+
    m_detWnd = new DragDetWnd();
    if (!m_detWnd) {
       g_debug("%s: unable to allocate DragDetWnd object\n", __FUNCTION__);
@@ -137,27 +135,29 @@ DnDUIX11::Init()
    SetTargetsAndCallbacks();
 
    /* Set common layer callbacks. */
-   m_DnD->dragStartChanged.connect(
+   m_DnD->srcDragBeginChanged.connect(
       sigc::mem_fun(this, &DnDUIX11::CommonDragStartCB));
-   m_DnD->fileCopyDoneChanged.connect(
+   m_DnD->srcDropChanged.connect(
+      sigc::mem_fun(this, &DnDUIX11::CommonSourceDropCB));
+   m_DnD->srcCancelChanged.connect(
+      sigc::mem_fun(this, &DnDUIX11::CommonSourceCancelCB));
+   m_DnD->getFilesDoneChanged.connect(
       sigc::mem_fun(this, &DnDUIX11::CommonSourceFileCopyDoneCB));
+
+   m_DnD->destCancelChanged.connect(
+      sigc::mem_fun(this, &DnDUIX11::CommonDestCancelCB));
+   m_DnD->privDropChanged.connect(
+      sigc::mem_fun(this, &DnDUIX11::CommonDestPrivateDropCB));
+
    m_DnD->updateDetWndChanged.connect(
       sigc::mem_fun(this, &DnDUIX11::CommonUpdateDetWndCB));
-   m_DnD->updateUnityDetWndChanged.connect(
-      sigc::mem_fun(this, &DnDUIX11::CommonUpdateUnityDetWndCB));
-   m_DnD->moveDetWndToMousePos.connect(
-      sigc::mem_fun(this, &DnDUIX11::CommonMoveDetWndToMousePos));
-   m_DnD->sourceCancelChanged.connect(
-      sigc::mem_fun(this, &DnDUIX11::CommonSourceCancelCB));
-   m_DnD->targetPrivateDropChanged.connect(
-      sigc::mem_fun(this, &DnDUIX11::CommonDestPrivateDropCB));
-   m_DnD->ghCancel.connect(
-      sigc::mem_fun(this, &DnDUIX11::CommonDestCancelCB));
-   m_DnD->sourceDropChanged.connect(
-      sigc::mem_fun(this, &DnDUIX11::CommonSourceDropCB));
-   m_DnD->updateMouseChanged.connect(
+   m_DnD->moveMouseChanged.connect(
       sigc::mem_fun(this, &DnDUIX11::CommonUpdateMouseCB));
 
+   m_DnD->updateUnityDetWndChanged.connect(
+      sigc::mem_fun(this, &DnDUIX11::CommonUpdateUnityDetWndCB));
+   m_DnD->destMoveDetWndToMousePosChanged.connect(
+      sigc::mem_fun(this, &DnDUIX11::CommonMoveDetWndToMousePos));
    /* Set Gtk+ callbacks for source. */
    m_detWnd->signal_drag_begin().connect(
       sigc::mem_fun(this, &DnDUIX11::GtkSourceDragBeginCB));
@@ -166,6 +166,27 @@ DnDUIX11::Init()
    m_detWnd->signal_drag_end().connect(
       sigc::mem_fun(this, &DnDUIX11::GtkSourceDragEndCB));
 
+   m_detWnd->signal_enter_notify_event().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkEnterEventCB));
+   m_detWnd->signal_leave_notify_event().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkLeaveEventCB));
+   m_detWnd->signal_map_event().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkMapEventCB));
+   m_detWnd->signal_unmap_event().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkUnmapEventCB));
+   m_detWnd->signal_realize().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkRealizeEventCB));
+   m_detWnd->signal_unrealize().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkUnrealizeEventCB));
+   m_detWnd->signal_motion_notify_event().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkMotionNotifyEventCB));
+   m_detWnd->signal_configure_event().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkConfigureEventCB));
+   m_detWnd->signal_button_press_event().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkButtonPressEventCB));
+   m_detWnd->signal_button_release_event().connect(
+      sigc::mem_fun(this, &DnDUIX11::GtkButtonReleaseEventCB));
+
    CommonUpdateDetWndCB(false, 0, 0);
    CommonUpdateUnityDetWndCB(false, 0, false);
    goto out;
@@ -207,9 +228,9 @@ DnDUIX11::SetTargetsAndCallbacks()
    targets.push_back(Gtk::TargetEntry(TARGET_NAME_TEXT_RICHTEXT));
 
    /* Plain text DnD. */
+   targets.push_back(Gtk::TargetEntry(TARGET_NAME_UTF8_STRING));
    targets.push_back(Gtk::TargetEntry(TARGET_NAME_STRING));
    targets.push_back(Gtk::TargetEntry(TARGET_NAME_TEXT_PLAIN));
-   targets.push_back(Gtk::TargetEntry(TARGET_NAME_UTF8_STRING));
    targets.push_back(Gtk::TargetEntry(TARGET_NAME_COMPOUND_TEXT));
 
    /*
@@ -250,7 +271,7 @@ DnDUIX11::CommonResetCB(void)
    m_effect = DROP_NONE;
    m_inHGDrag = false;
    m_dc = NULL;
-   m_isFileDnD = false;
+   m_fileTransferStarted = false;
    RemoveBlock();
 }
 
@@ -343,7 +364,7 @@ DnDUIX11::CommonDragStartCB(const CPClipboard *clip, std::string stagingDir)
    /* Tell Gtk that a drag should be started from this widget. */
    m_detWnd->drag_begin(targets, actions, 1, (GdkEvent *)&event);
    m_blockAdded = false;
-   m_isFileDnD = false;
+   m_fileTransferStarted = false;
    SourceDragStartDone();
    /* Initialize host hide feedback to DROP_NONE. */
    m_effect = DROP_NONE;
@@ -454,18 +475,26 @@ DnDUIX11::CommonSourceDropCB(void)
  * copying from host to guest staging directory.
  *
  * @param[in] success if true, transfer was successful
- * @param[in] path of staging dir (which will have a block that needs removing)
  */
 
 void
-DnDUIX11::CommonSourceFileCopyDoneCB(bool success,
-                                     std::vector<uint8> stagingDir)
+DnDUIX11::CommonSourceFileCopyDoneCB(bool success)
 {
    g_debug("%s: %s\n", __FUNCTION__, success ? "success" : "failed");
-   /* Copied files are already removed in common layer. */
-   stagingDir.clear();
-   CommonResetCB();
+
+   /*
+    * If hg drag is not done yet, only remove block. GtkSourceDragEndCB will
+    * call CommonResetCB(). Otherwise destination may miss the data because
+     * we are already reset.
+    */
+
    m_HGGetDataInProgress = false;
+
+   if (!m_inHGDrag) {
+      CommonResetCB();
+   } else {
+      RemoveBlock();
+   }
 }
 
 
@@ -502,7 +531,7 @@ DnDUIX11::CommonUpdateDetWndCB(bool bShow,
        * Stolen from the legacy V2 DnD code.
        */
 
-      SendFakeMouseMove(x, y);
+      SendFakeMouseMove(x + 2, y + 2);
       m_detWnd->SetIsVisible(true);
    } else {
       g_debug("%s: hide\n", __FUNCTION__);
@@ -897,9 +926,9 @@ DnDUIX11::GtkSourceDragDataGetCB(const Glib::RefPtr<Gdk::DragContext> &dc,
        * Doing both of these addresses bug
        * http://bugzilla.eng.vmware.com/show_bug.cgi?id=391661.
        */
-      if (!m_blockAdded && m_inHGDrag) {
+      if (!m_blockAdded && m_inHGDrag && !m_fileTransferStarted) {
          m_HGGetDataInProgress = true;
-         m_isFileDnD = true;
+         m_fileTransferStarted = true;
          AddBlock();
       } else {
          g_debug("%s: not calling AddBlock\n", __FUNCTION__);
@@ -939,7 +968,6 @@ DnDUIX11::GtkSourceDragDataGetCB(const Glib::RefPtr<Gdk::DragContext> &dc,
 
    /* Can not get any valid data, cancel this HG DnD. */
    g_debug("%s: no valid data for HG DnD\n", __FUNCTION__);
-   m_DnD->SourceCancel();
    CommonResetCB();
 }
 
@@ -967,10 +995,11 @@ DnDUIX11::GtkSourceDragEndCB(const Glib::RefPtr<Gdk::DragContext> &dc)
    }
 
    /*
-    * If we are a file DnD, don't call CommonResetCB() here, since
-    * we will do so in the fileCopyDoneChanged callback.
+    * If we are a file DnD and file transfer is not done yet, don't call
+    * CommonResetCB() here, since we will do so in the fileCopyDoneChanged
+    * callback.
     */
-   if (!m_isFileDnD) {
+   if (!m_fileTransferStarted || !m_HGGetDataInProgress) {
       CommonResetCB();
    }
    m_inHGDrag = false;
@@ -1151,7 +1180,23 @@ DnDUIX11::SetCPClipboardFromGtk(const Gtk::SelectionData& sd) // IN
       while ((newPath = DnD_UriListGetNextFile(source.c_str(),
                                                &index,
                                                &newPathLen)) != NULL) {
-
+#if defined(linux)
+         if (DnD_UriIsNonFileSchemes(newPath)) {
+            /* Try to get local file path for non file uri. */
+            GFile *file = g_file_new_for_uri(newPath);
+            free(newPath);
+            if (!file) {
+               g_debug("%s: g_file_new_for_uri failed\n", __FUNCTION__);
+               return false;
+            }
+            newPath = g_file_get_path(file);
+            g_object_unref(file);
+            if (!newPath) {
+               g_debug("%s: g_file_get_path failed\n", __FUNCTION__);
+               return false;
+            }
+         }
+#endif
          /*
           * Parse relative path.
           */
@@ -1663,7 +1708,10 @@ DnDUIX11::RemoveBlock()
    g_debug("%s: enter\n", __FUNCTION__);
    if (m_blockAdded && !m_HGGetDataInProgress) {
       g_debug("%s: removing block for %s\n", __FUNCTION__, m_HGStagingDir.c_str());
-      m_blockCtrl->RemoveBlock(m_blockCtrl->fd, m_HGStagingDir.c_str());
+      /* We need to make sure block subsystem has not been shut off. */
+      if (DnD_BlockIsReady(m_blockCtrl)) {
+         m_blockCtrl->RemoveBlock(m_blockCtrl->fd, m_HGStagingDir.c_str());
+      }
       m_blockAdded = false;
    } else {
       g_debug("%s: not removing block m_blockAdded %d m_HGGetDataInProgress %d\n",
@@ -1900,7 +1948,6 @@ exit:
 
 
 /**
- *
  * Tell host that we are done with HG DnD initialization.
  */
 
@@ -1909,7 +1956,7 @@ DnDUIX11::SourceDragStartDone(void)
 {
    g_debug("%s: enter\n", __FUNCTION__);
    m_inHGDrag = true;
-   m_DnD->HGDragStartDone();
+   m_DnD->SrcUIDragBeginDone();
 }
 
 
@@ -1927,7 +1974,6 @@ DnDUIX11::SetBlockControl(DnDBlockControl *blockCtrl)
 
 
 /**
- *
  * Got feedback from our DropSource, send it over to host. Called by
  * drag motion callback.
  *
@@ -1938,12 +1984,11 @@ void
 DnDUIX11::SourceUpdateFeedback(DND_DROPEFFECT effect)
 {
    g_debug("%s: entering\n", __FUNCTION__);
-   m_DnD->SetFeedback(effect);
+   m_DnD->SrcUIUpdateFeedback(effect);
 }
 
 
 /**
- *
  * This is triggered when user drags valid data from guest to host. Try to
  * get clip data and notify host to start GH DnD.
  */
@@ -1956,7 +2001,7 @@ DnDUIX11::TargetDragEnter(void)
    /* Check if there is valid data with current detection window. */
    if (!CPClipboard_IsEmpty(&m_clipboard)) {
       g_debug("%s: got valid data from detWnd.\n", __FUNCTION__);
-      m_DnD->DragEnter(&m_clipboard);
+      m_DnD->DestUIDragEnter(&m_clipboard);
    }
 
    /*
@@ -1968,7 +2013,6 @@ DnDUIX11::TargetDragEnter(void)
 
 
 /**
- *
  * Get Unix time in milliseconds. See man 2 gettimeofday for details.
  *
  * @return unix time in milliseconds.
@@ -1982,3 +2026,169 @@ DnDUIX11::GetTimeInMillis(void)
    Hostinfo_GetTimeOfDay(&atime);
    return((unsigned long)(atime / 1000));
 }
+
+
+/**
+ * Update version information in mDnD.
+ *
+ * @param[ignored] chan RpcChannel pointer
+ * @param[in] version the version negotiated with host.
+ */
+
+void
+DnDUIX11::VmxDnDVersionChanged(RpcChannel *chan, uint32 version)
+{
+   ASSERT(m_DnD);
+   m_DnD->VmxDnDVersionChanged(version);
+}
+
+
+/**
+ * Track enter events on detection window.
+ *
+ * @param[ignored] event event data (ignored)
+ *
+ * @return always true.
+ */
+
+bool
+DnDUIX11::GtkEnterEventCB(GdkEventCrossing *ignored)
+{
+   g_debug("%s: enter\n", __FUNCTION__);
+   return true;
+}
+
+/**
+ * Track enter events on detection window.
+ *
+ * @param[ignored] event event data (ignored)
+ *
+ * @return always true.
+ */
+
+bool
+DnDUIX11::GtkLeaveEventCB(GdkEventCrossing *ignored)
+{
+   g_debug("%s: enter\n", __FUNCTION__);
+   return true;
+}
+
+/**
+ * Track enter events on detection window.
+ *
+ * @param[ignored] event event data (ignored)
+ *
+ * @return always true.
+ */
+
+bool
+DnDUIX11::GtkMapEventCB(GdkEventAny *event)
+{
+   g_debug("%s: enter\n", __FUNCTION__);
+   return true;
+}
+
+
+/**
+ * Track enter events on detection window.
+ *
+ * @param[ignored] event event data (ignored)
+ *
+ * @return always true.
+ */
+
+bool
+DnDUIX11::GtkUnmapEventCB(GdkEventAny *event)
+{
+   g_debug("%s: enter\n", __FUNCTION__);
+   return true;
+}
+
+
+/**
+ * Track realize events on detection window.
+ */
+
+void
+DnDUIX11::GtkRealizeEventCB()
+{
+   g_debug("%s: enter\n", __FUNCTION__);
+}
+
+
+/**
+ * Track unrealize events on detection window.
+ */
+
+void
+DnDUIX11::GtkUnrealizeEventCB()
+{
+   g_debug("%s: enter\n", __FUNCTION__);
+}
+
+/**
+ * Track motion notify events on detection window.
+ *
+ * @param[in] event event data
+ *
+ * @return always true.
+ */
+
+bool
+DnDUIX11::GtkMotionNotifyEventCB(GdkEventMotion *event)
+{
+   g_debug("%s: enter x %f y %f state 0x%x\n", __FUNCTION__,
+           event->x, event->y, event->state);
+   return true;
+}
+
+
+/**
+ * Track configure events on detection window.
+ *
+ * @param[in] event event data
+ *
+ * @return always true.
+ */
+
+bool
+DnDUIX11::GtkConfigureEventCB(GdkEventConfigure *event)
+{
+   g_debug("%s: enter x %d y %d width %d height %d\n",
+           __FUNCTION__, event->x, event->y, event->width, event->height);
+   return true;
+}
+
+
+/**
+ * Track button press events on detection window.
+ *
+ * @param[ignored] event event data (ignored)
+ *
+ * @return always true.
+ */
+
+bool
+DnDUIX11::GtkButtonPressEventCB(GdkEventButton *event)
+{
+   g_debug("%s: enter\n", __FUNCTION__);
+   return true;
+}
+
+
+/**
+ * Track button release events on detection window.
+ *
+ * @param[ignored] event event data (ignored)
+ *
+ * @return always true.
+ */
+
+bool
+DnDUIX11::GtkButtonReleaseEventCB(GdkEventButton *event)
+{
+   g_debug("%s: enter\n", __FUNCTION__);
+   return true;
+}
+
+
