@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -28,12 +28,10 @@
 #include <sstream>
 #include <iostream>
 
-#include "autoCPtr.hh"
 #include "string.hh"
 #include "unicode.h"
 #include "util.h"
 
-
 namespace utf {
 
 /*
@@ -86,18 +84,62 @@ string::string()
  *-----------------------------------------------------------------------------
  */
 
-string::string(const char *s) // IN
+string::string(ConstUnicode s) // IN
    : mUstr(),
      mUtf16Cache(NULL),
      mUtf16Length(npos)
 {
    ASSERT(s);
-   mUstr = s;
+   mUstr = Unicode_GetUTF8(s);
    ASSERT(Validate(mUstr));
 }
 
 
 #ifdef _WIN32
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * utf::string::init_bstr_t --
+ *
+ *      Utility function to construct from a _bstr_t object.
+ *      Copies the UTF-16 representation of the _bstr_t.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Makes a copy of the _bstr_t data and frees that data when
+ *      the utf::string is destroyed.
+ *
+ * Note:
+ *      WIN32 only call
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+string::init_bstr_t(const _bstr_t &s) // IN
+{
+   // If the input is empty, then there's nothing to do.
+   if (s.length() == 0) {
+      return;
+   }
+
+   Unicode utf8 = Unicode_AllocWithUTF16(static_cast<const utf16_t *>(s));
+
+   try {
+      mUstr = utf8;
+      Unicode_Free(utf8);
+   } catch (...) {
+      Unicode_Free(utf8);
+      throw;
+   }
+
+   ASSERT(Validate(mUstr));
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -160,16 +202,93 @@ string::string(const _bstr_t &s) // IN
      mUtf16Cache(NULL),
      mUtf16Length(npos)
 {
+   init_bstr_t(s);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * utf::string::string --
+ *
+ *      Constructor from a uvariant_t object. Copies the UTF-16 representation
+ *      of the ubstr_t interface.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Makes a copy of the uvariant_t data and frees that data when the
+ *      utf::string is destroyed.
+ *
+ * Note:
+ *      WIN32 only call
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+string::string(const uvariant_t &v) // IN
+   : mUstr(),
+     mUtf16Cache(NULL),
+     mUtf16Length(npos)
+{
+   ubstr_t s;
+
+   try {
+      s = v;
+   } catch (...) {
+      Warning("Invalid uvariant_t to ubstr_t conversion.\n");
+      throw;
+   }
+
    // If the input is empty, then there's nothing to do.
    if (s.length() == 0) {
       return;
    }
 
-   mUstr = AutoCPtr<char>(
-      Unicode_AllocWithUTF16(static_cast<const utf16_t *>(s)),
-      free).get();
+   mUstr = static_cast<const char *>(s);
    ASSERT(Validate(mUstr));
 }
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * utf::string::string --
+ *
+ *      Constructor from a _variant_t object. Copies the UTF-16 representation
+ *      of the _variant_t.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      Makes a copy of the _variant_t data and frees that data when
+ *      the utf::string is destroyed.
+ *
+ * Note:
+ *      WIN32 only call
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+string::string(const _variant_t &v) // IN
+   : mUstr(),
+     mUtf16Cache(NULL),
+     mUtf16Length(npos)
+{
+   _bstr_t s;
+
+   try {
+      s = v;
+   } catch (...) {
+      Warning("Invalid _variant_t to _bstr_t conversion.\n");
+      throw;
+   }
+
+   init_bstr_t(s);
+}
+
 #endif
 
 
@@ -233,8 +352,16 @@ string::string(const utf16_t *s) // IN
     */
    mUtf16Cache = Unicode_UTF16Strdup(s);
 
-   mUstr = AutoCPtr<char>(Unicode_AllocWithUTF16(s),
-                          free).get();
+   Unicode utf8 = Unicode_AllocWithUTF16(s);
+
+   try {
+      mUstr = utf8;
+      Unicode_Free(utf8);
+   } catch (...) {
+      Unicode_Free(utf8);
+      throw;
+   }
+
    ASSERT(Validate(mUstr));
 }
 
@@ -263,8 +390,16 @@ string::string(const char *s,           // IN
 {
    ASSERT(s != NULL);
 
-   mUstr = AutoCPtr<char>(Unicode_Alloc(s, encoding),
-                          free).get();
+   Unicode utf8 = Unicode_Alloc(s, encoding);
+
+   try {
+      mUstr = utf8;
+      Unicode_Free(utf8);
+   } catch (...) {
+      Unicode_Free(utf8);
+      throw;
+   }
+
    ASSERT(Validate(mUstr));
 }
 
@@ -276,6 +411,12 @@ string::string(const char *s,           // IN
  *
  *      Constructor.
  *
+ *      XXX: When initializing mUstr, we do a deep copy of the string data
+ *      instead of just calling mUstr(s). This is because Glib::ustring is very
+ *      smart about sharing storage, and zero_clear is very dumb. Once we get
+ *      rid of zero_clear and have a separate sensitive-string class, this can
+ *      go back to being simple.
+ *
  * Results:
  *      None.
  *
@@ -285,8 +426,8 @@ string::string(const char *s,           // IN
  *-----------------------------------------------------------------------------
  */
 
-string::string(const Glib::ustring& s) // IN
-   : mUstr(s),
+string::string(const Glib::ustring &s)    // IN
+   : mUstr(s.c_str()),
      mUtf16Cache(NULL),
      mUtf16Length(npos)
 {
@@ -301,6 +442,12 @@ string::string(const Glib::ustring& s) // IN
  *
  *      Copy constructor.
  *
+ *      XXX: When initializing mUstr, we do a deep copy of the string data
+ *      instead of just calling mUstr(s). This is because Glib::ustring is very
+ *      smart about sharing storage, and zero_clear is very dumb. Once we get
+ *      rid of zero_clear and have a separate sensitive-string class, this can
+ *      go back to being simple.
+ *
  * Results:
  *      None.
  *
@@ -310,8 +457,8 @@ string::string(const Glib::ustring& s) // IN
  *-----------------------------------------------------------------------------
  */
 
-string::string(const string& s) // IN
-   : mUstr(s.mUstr),
+string::string(const string &s) // IN
+   : mUstr(s.mUstr.c_str()),
      mUtf16Cache(NULL),
      mUtf16Length(npos)
 {
@@ -465,12 +612,11 @@ string::operator+=(value_type uc) // IN
  */
 
 void
-string::swap(string& s) // IN/OUT
+string::swap(string &s) // IN/OUT
 {
-   using std::swap;
    mUstr.swap(s.mUstr);
-   swap(mUtf16Cache, s.mUtf16Cache);
-   swap(mUtf16Length, s.mUtf16Length);
+   std::swap(mUtf16Cache, s.mUtf16Cache);
+   std::swap(mUtf16Length, s.mUtf16Length);
 }
 
 
@@ -499,29 +645,6 @@ string::resize(size_type n,  // IN
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * utf::string::reserve --
- *
- *      Change the amount of memory allocated for the utf::string.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-string::reserve(size_type n) // IN/OPT
-{
-   mUstr.reserve(n);
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -758,7 +881,10 @@ string
 string::trim()
    const
 {
-   return CopyAndFree(Unicode_Trim(c_str()), free);
+   Unicode trim = Unicode_Trim(c_str());
+   string result(trim);
+   Unicode_Free(trim);
+   return result;
 }
 
 
@@ -782,7 +908,10 @@ string
 string::trimLeft()
    const
 {
-   return CopyAndFree(Unicode_TrimLeft(c_str()), free);
+   Unicode trim = Unicode_TrimLeft(c_str());
+   string result(trim);
+   Unicode_Free(trim);
+   return result;
 }
 
 
@@ -806,7 +935,10 @@ string
 string::trimRight()
    const
 {
-   return CopyAndFree(Unicode_TrimRight(c_str()), free);
+   Unicode trim = Unicode_TrimRight(c_str());
+   string result(trim);
+   Unicode_Free(trim);
+   return result;
 }
 
 
@@ -856,10 +988,14 @@ string::toLower(const char *locale) // IN
    const
 {
 #ifdef USE_ICU
-   return CopyAndFree(Unicode_ToLower(c_str(), locale), free);
+   Unicode lower = Unicode_ToLower(c_str(), locale);
+   string results(lower);
+   Unicode_Free(lower);
 #else
-   return mUstr.lowercase();
+   string results(mUstr.lowercase());
 #endif
+
+   return results;
 }
 
 
@@ -885,14 +1021,20 @@ string::toUpper(const char *locale) // IN
    const
 {
 #ifdef USE_ICU
-   return CopyAndFree(Unicode_ToUpper(c_str(), locale), free);
+   Unicode upper = Unicode_ToUpper(c_str(), locale);
+   string results(upper);
+   Unicode_Free(upper);
 #else
-   return mUstr.uppercase();
+   string results(mUstr.uppercase());
 #endif
+
+
+   return results;
 }
 
 
 #ifdef USE_ICU
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -914,8 +1056,13 @@ string
 string::toTitle(const char *locale) // IN
    const
 {
-   return CopyAndFree(Unicode_ToTitle(c_str(), locale), free);
+   Unicode title = Unicode_ToTitle(c_str(), locale);
+   string results(title);
+   Unicode_Free(title);
+
+   return results;
 }
+
 #endif
 
 
@@ -1038,7 +1185,7 @@ string::assign(const string &s) // IN
 
 string&
 string::insert(size_type i,      // IN
-               const string& s)  // IN
+               const string &s)  // IN
 {
    InvalidateCache();
    mUstr.insert(i, s.mUstr);
@@ -1057,16 +1204,6 @@ string::insert(size_type i,      // IN
 }
 
 
-string&
-string::insert(iterator p,    // IN
-               value_type uc) // IN
-{
-   InvalidateCache();
-   mUstr.insert(p, uc);
-   return *this;
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1091,6 +1228,46 @@ string::clear()
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * utf::string::zero_clear --
+ *
+ *      Zeroes and clears this string.
+ *
+ *      XXX: This is temporary until we have a separate string class for
+ *      passwords.
+ *
+ * Results:
+ *      None
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+string::zero_clear()
+{
+   if (mUtf16Cache != NULL) {
+      Util_ZeroFree(mUtf16Cache,
+                    Unicode_UTF16Strlen(mUtf16Cache) * sizeof *mUtf16Cache);
+      mUtf16Cache = NULL;
+   }
+
+   /*
+    * This is a best effort.  We aren't guaranteed that Glib::ustring doesn't
+    * leave behind any internal copies of the string.
+    */
+   if (mUstr.c_str() != mUstr.data()) {
+      Util_Zero(const_cast<char *>(mUstr.c_str()), mUstr.bytes());
+   }
+   Util_Zero(const_cast<char *>(mUstr.data()), mUstr.bytes());
+   mUstr.clear();
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1169,7 +1346,7 @@ string::erase(iterator pbegin,    // IN
 string&
 string::replace(size_type i,     // IN
                 size_type n,     // IN
-                const string& s) // IN
+                const string &s) // IN
 {
    InvalidateCache();
    mUstr.replace(i, n, s.mUstr);
@@ -1185,8 +1362,6 @@ string::replace(size_type i,     // IN
  *      Mutates this string by replacing all occurrences of one string with
  *      another.
  *
- *      Does nothing if the `from` string is empty.
- *
  * Results:
  *      A reference to this object.
  *
@@ -1197,18 +1372,13 @@ string::replace(size_type i,     // IN
  */
 
 string&
-string::replace(const string& from, // IN
-                const string& to)   // IN
+string::replace(const string &from, // IN
+                const string &to)   // IN
 {
-   if (from.empty()) {
-      return *this;
-   }
-
    size_type end;
    size_type start = 0;
    size_type fromSize = from.length();
    string result;
-   result.reserve(bytes() * to.bytes() / from.bytes());
 
    while ((end = find(from, start)) != string::npos) {
       result += substr(start, end - start);
@@ -1221,8 +1391,6 @@ string::replace(const string& from, // IN
       result += substr(start);
    }
 
-   result.reserve(0);
-
    swap(result);
    return *this;
 }
@@ -1672,15 +1840,11 @@ string::endsWith(const string &s, // IN
  *
  *      Return a vector of utf::strings.  The vector contains the elements of
  *      the string split by the passed in separator. Empty tokens are not
- *      skipped. If maxStrings is zero, any number of strings will be returned,
- *      otherwise parsing stops after maxStrings - 1 matches of the separator.
- *      In that case, the last string returned includes the rest of the
- *      original string.
+ *      skipped.
  *
  *      "1,2,3".split(",") -> ["1", "2", "3"]
  *      "1,,".split(",") -> ["1", "", ""]
  *      "1".split(",") -> ["1"]
- *      "1,2,3".split(",", 2) -> ["1", "2,3"]
  *
  *      XXX If this is to be used for things like command line parsing, support
  *      for quoted strings needs to be added.
@@ -1695,21 +1859,18 @@ string::endsWith(const string &s, // IN
  */
 
 std::vector<string>
-string::split(const string &sep, // IN
-              size_t maxStrings) // IN/OPT
+string::split(const string &sep) // IN
    const
 {
    std::vector<string> splitStrings;
    size_type sIndex = 0;
    size_type sepLen = sep.length();
-   size_t count = 0;
 
    ASSERT(sepLen > 0);
 
    while (true) {
       size_type index = find(sep, sIndex);
-      count++;
-      if (count == maxStrings || index == npos) {
+      if (index == npos) {
          splitStrings.push_back(substr(sIndex));
          break;
       }
@@ -2037,11 +2198,9 @@ string::end()
 bool
 Validate(const Glib::ustring& s) // IN
 {
-   bool isValid = Unicode_IsBufferValid(s.c_str(), s.bytes(),
-                                        STRING_ENCODING_UTF8);
+   bool isValid = s.validate();
    if (!isValid) {
-      char *escaped = Unicode_EscapeBuffer(s.c_str(), s.bytes(),
-                                           STRING_ENCODING_UTF8);
+      char *escaped = Unicode_EscapeBuffer(s.c_str(), -1, STRING_ENCODING_UTF8);
       Warning("Invalid UTF-8 string: \"%s\"\n", escaped);
       free(escaped);
    }
@@ -2074,8 +2233,16 @@ CreateWithLength(const void *buffer,      // IN
       throw ConversionError();
    }
 
-   return CopyAndFree(Unicode_AllocWithLength(buffer, lengthInBytes, encoding),
-                      free);
+   Unicode utf8 = Unicode_AllocWithLength(buffer, lengthInBytes, encoding);
+
+   try {
+      string result(utf8);
+      Unicode_Free(utf8);
+      return result;
+   } catch (...) {
+      Unicode_Free(utf8);
+      throw;
+   }
 }
 
 
@@ -2131,32 +2298,6 @@ CreateWithBOMBuffer(const void *buffer,      // IN
 }
 
 
-/*
- *----------------------------------------------------------------------
- *
- * utf::CopyAndFree --
- *
- *       Creates a utf::string from an allocated UTF-8 C string,
- *       automatically freeing it afterward.
- *
- * Results:
- *       A copy of the string.
- *
- * Side Effect:
- *       None.
- *
- *----------------------------------------------------------------------
- */
-
-string
-CopyAndFree(char* utf8,              // IN
-            void (*freeFunc)(void*)) // IN/OPT
-{
-   ASSERT(utf8 != NULL);
-   return AutoCPtr<char>(utf8, freeFunc).get();
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -2188,7 +2329,6 @@ IntToStr(int64 val) // IN
  * utf::CopyArray --
  *
  *      Copies an array to a vector.
- *      Guaranteed to not shrink the vector.
  *
  * Results:
  *      A vector containing a shallow copy of the array.
@@ -2205,13 +2345,10 @@ CopyArray(const T *p,          // IN:
           size_t n,            // IN: The number of array elements to copy.
           std::vector<T>& buf) // OUT:
 {
-   if (n > buf.size()) {
-      buf.resize(n);
-   }
-
+   buf.resize(n);
    if (!buf.empty()) {
       ASSERT(p != NULL);
-      memcpy(&buf[0], p, n * sizeof buf[0]);
+      memcpy(&buf[0], p, buf.size() * sizeof buf[0]);
    }
 }
 
@@ -2222,12 +2359,9 @@ CopyArray(const T *p,          // IN:
  * utf::CreateWritableBuffer --
  *
  *      Copies a utf::string to a writable buffer.
- *      Guaranteed to never shrink the size of the destination buffer.
  *
  * Results:
- *      A std::vector containing the NUL-terminated string data.
- *      The size of the resulting vector may exceed the length of the
- *      NUL-terminated string.
+ *      A std::vector containing the string data.
  *
  * Side effects:
  *      None
