@@ -47,7 +47,6 @@ extern "C" {
    #include "guestrpc/unityActive.h"
    #include "rpcin.h"
    #include "rpcout.h"
-   #include "str.h"
    #include "strutil.h"
 #if defined(OPEN_VM_TOOLS)
    #include "unitylib/unity.h"
@@ -436,7 +435,6 @@ gboolean
 UnityTcloSetDesktopWorkArea(RpcInData *data)    // IN/OUT
 {
    Bool success = FALSE;
-   const char *argList;
    unsigned int count;
    unsigned int i;
    UnityRect *workAreas = NULL;
@@ -479,8 +477,8 @@ UnityTcloSetDesktopWorkArea(RpcInData *data)    // IN/OUT
       }
    }
 
-   for (argList = data->args, i = 0; i < count; i++) {
-      argList = strchr(argList, ',');
+   for (i = 0; i < count; i++) {
+      const char *argList = strchr(data->args, ',');
       if (!argList) {
          RPCIN_SETRETVALS(data,
                           "Expected comma separated display list",
