@@ -94,9 +94,6 @@ struct VixToolsUserEnvironment {
 
 VixError
 VixToolsNewEnvIterator(void *userToken,                  // IN
-#ifdef __FreeBSD__
-                       char **envp,                      // IN
-#endif
                        VixToolsEnvIterator **envItr)     // OUT
 {
    VixError err = VIX_OK;
@@ -135,7 +132,14 @@ VixToolsNewEnvIterator(void *userToken,                  // IN
 #elif defined(__APPLE__)
    it->environ = *_NSGetEnviron();
 #elif defined(__FreeBSD__)
-   it->environ = envp;
+   /*
+    * Looking at /build/toolchain/bsd32/freebsd-6.3/usr/include/stand.h,
+    * environ is a pointer to a doubly linked list of structs. I guess they
+    * just want to be different. Anyway, this is something that needs
+    * work if we want to support FreeBSD.
+    */
+   err = VIX_E_NOT_SUPPORTED;
+   goto abort;
 #else
    it->environ = environ;
 #endif
